/*
 * MP3 Player, Main Program, http://www.pjrc.com/tech/mp3
 * Copyright (c) 2000, PJRC.COM, LLC
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a specific exception to the GPL, the executable object code built
 * from this source code may be combined with hardware configuration data
 * files.  A hardware configuration data file is a set of data that is
 * transmitted to an intergrated circuit that is not a general purpose
 * microprocessor or microcontroller, in order to establish its normal
 * operation.  The process of combining the executable ojbect code built
 * from the GPL licensed source code with the hardware configuration data
 * shall be considered an aggregation of another work not based on the
 * Program.  While the GPL does not restrict use of the program, any
 * use restriction associated with the hardware configuration data (for
 * example, that it only be used with particular hardware) shall apply
 * to the combined file which includes a copy of the hardware configuration
 * data.
 *
 * Contact: paul@pjrc.com
 */



#include "paulmon2.h"
#include "as31glue.h"
#include "startup.h"
#include "printf.h"
#include <reg51.h>


#ifndef SDCC_MODEL_LARGE
#error "SDCC's large memory model is required"
#endif

#define PARM_ATTN 1
#define DEFAULT_ATTN 12
#define MAX_ATTN 64

char is_state_normal(unsigned char state);
void main_directory_backup(char fd);
void read_directory(char dir, xdata struct dirent_struct *fileinfo);
char is_mp3_file(xdata struct dirent_struct *fileinfo);

#define S_READ_DIR		1
#define S_CACHING		2
#define S_PLAYING_AND_CACHING	3
#define S_PLAYING		4
#define S_FINISH_PLAYING	5
#define S_PAUSED		6

#define E_NOTHING		0
#define E_NEXT			1
#define E_PLAY_PAUSE		2
#define E_PREV			3
#define E_RANDOM		4
#define E_VOL_UP		5
#define E_VOL_DOWN		6

char rand_mode=0;


bit at 0x0E debug;


void main()
{
	unsigned char i, event, attn, num;
	char dir=-1, mp3_fd=-1;
	char r;
	char state=S_READ_DIR, prior_state=S_READ_DIR;
	char caching_step=0;
	xdata struct dirent_struct fileinfo;
	unsigned int block;


	/* initialize everything */
	pm2_pstr(welcome_message);
	if (hardware_init() != 0) pm2_exit();
	if (detect_filesystem() == 0) pm2_exit();
	print_memory_available();
	sta013_set_attenuation(read_param_1byte(PARM_ATTN, DEFAULT_ATTN));
	printf("\r\nMP3 Player, Begin Playback\r\n");

	debug = 0;

	/* read the root directory */
	dir = file_open_by_1st_cluster(get_root_1st_cluster());
	do_ide_loop(1);
	file_cache(dir, 0, 0xFFFFFFFF);	 // request entire directory cached
	do {
		do_ide_loop(4);
		r = file_cache_work(dir);
		if (r == 2) pm2_exit();
	} while (r == 0);
	do_ide_loop(8);
	file_seek(dir, 0);


	/* this infinite loop runs all the actions of the MP3 player */

	while (1) {
		/* first, process any events from the user interface */
		update_events();
		event = get_next_event();
		switch (event) {
		  case E_NOTHING: {
			break;
		  }
		  case E_PLAY_PAUSE: {
			if (state == S_PAUSED) {
				printf("Play\r\n");
				state = prior_state;
				play_resume();
			} else {
				if (is_state_normal(state)) {
					printf("Pause\r\n");
					prior_state = state;
					state = S_PAUSED;
					play_suspend();
				} else {
					put_back_event(event);
				}
			}
			break;
		  }

		  case E_NEXT:
			if (is_state_normal(state)) {
				printf("Next File\r\n");
				play_abort();
				ide_flush();
				state = S_FINISH_PLAYING;
			} else {
				put_back_event(event);
			}
			break;
			
		  case E_PREV:
			if (is_state_normal(state)) {
				printf("Previous File\r\n");
				play_abort();
				ide_flush();
				main_directory_backup(dir);
				state = S_FINISH_PLAYING;
			} else {
				put_back_event(event);
			}
			break;

		  case E_RANDOM:
			rand_mode = rand_mode ^ 1;
			if (rand_mode) {
				printf("Random On\r\n");
			} else {
				printf("Random On\r\n");
			}
			break;

		  case E_VOL_UP:
			printf("Vol Up\r\n");
			attn = sta013_get_attenuation();
			if (--attn < 0) attn = 0;
			sta013_set_attenuation(attn);
			write_flash_param(PARM_ATTN, &attn);
			break;

		  case E_VOL_DOWN:
			printf("Vol Down\r\n");
			attn = sta013_get_attenuation();
			if (++attn > MAX_ATTN) attn = MAX_ATTN;
			sta013_set_attenuation(attn);
			write_flash_param(PARM_ATTN, &attn);
			break;
		}

		do_ide_loop(6);

		/* and then do the next step to actually play MP3 files */

		switch(state) {
		  case S_READ_DIR:
			read_directory(dir, &fileinfo);
			printf("Playing: %s %s , clust=%lx, size=%lx\r\n",
				&(*fileinfo.short_name),
				&(*fileinfo.long_name),
				fileinfo.cluster, fileinfo.size);
			mp3_fd = file_open_by_1st_cluster(fileinfo.cluster);
			file_cache(mp3_fd, 0, fileinfo.size);
			state = S_CACHING;
			caching_step = 0;
			r = SP;
			printf("S_READ_DIR, mp3_fd=%d %x\r\n", mp3_fd, r);
			break;

		  case S_CACHING:
			r = SP;
			printf("S_CACHING, mp3_fd=%d %x\r\n", mp3_fd, r);
			r = file_cache_work(mp3_fd);
			/* TODO: check return value from file_cache_work */
			if (++caching_step > 6) {
				file_seek(mp3_fd, 0);
				state = S_PLAYING_AND_CACHING;
				//debug = 0;
			}
			break;

		  case S_PLAYING_AND_CACHING:
			printf("S_PLAYING_CACHING, mp3_fd=%d\r\n", mp3_fd);
			r = file_cache_work(mp3_fd);
			if (r) {
				state = S_PLAYING;
				ide_sleep();
				free_fat_memory();
			}
			/* fall through into S_PLAYING's code */

		  case S_PLAYING:
			num = play_queue_avail();
			printf("S_PLAYING, num=%d\r\n", num);
			for (i=0; i<num; i++) {
				block = file_read_block(mp3_fd);
				printf("FRB, r=%x\r\n", block);
				if (block == 0) break;  // not in cache yet
				if (block != 0xFFFF) {
					play_block(block, 4096);
				} else {
					state = S_FINISH_PLAYING;
					break;
				}
				// r = SP;
				// printf("PB, i=%d, sp=%x\r\n", i, r & 255);
			}
			num = play_queue_avail();
			printf("S_PLAYING(end), num=%d\r\n", num);
			break;

		  case S_FINISH_PLAYING:
			num = play_queue_avail();
			printf("S_FINISH_PLAYING, mp3_fd=%d, num=%d\r\n",
			   mp3_fd, num);
			if (is_play_queue_empty() && is_ide_idle()) {
				file_close(mp3_fd);
				free_fat_memory();
				print_memory_available();
				ide_wakeup();
				state = S_READ_DIR;
			}
			break;

		  case S_PAUSED:
			if (prior_state == S_PLAYING_AND_CACHING) {
				r = file_cache_work(mp3_fd);
				if (r) {
					prior_state = S_PLAYING;
					ide_sleep();
					free_fat_memory();
				}
			}
			break;

		  default:
			printf("ILLEGAL STATE!!!!!\r\n");

		}
	}

	pm2_exit();
}


void read_directory(char dir, xdata struct dirent_struct *fileinfo)
{
	char r;

	while (1) {
		r = dir_read(dir, fileinfo);
		printf("dir_read, r=%d\r\n", r);

		if (r) {
			file_seek(dir, 0);
			continue;
		}
		if (!is_mp3_file(fileinfo)) {
			continue;
		}
		/* do random number if random mode */
		return;
	}
}


char is_mp3_file(xdata struct dirent_struct *fileinfo)
{
	if ((fileinfo->attrib & 0x1E) != 0) return 0;
	if (fileinfo->short_name[9] != 'M') return 0;
	if (fileinfo->short_name[10] != 'P') return 0;
	if (fileinfo->short_name[11] != '3') return 0;
	return 1;
}




char is_state_normal(unsigned char state)
{
	if (state == S_PLAYING
	 || state == S_PLAYING_AND_CACHING
	 || state == S_FINISH_PLAYING) {
		return 1;
	}
	return 0;
}

void main_directory_backup(char fd)
{
	// TODO: implement this
	fd++;

}


void putchar(char c)
{
	while (!TI) ;
	TI = 0;
	SBUF = c;
	// pm2_cout(c);
}



