; MP3 Player, Device Drivers, http://www.pjrc.com/tech/mp3
; Copyright (c) 2000, PJRC.COM, LLC

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

; As a specific exception to the GPL, the executable object code built
; from this source code may be combined with hardware configuration data
; files.  A hardware configuration data file is a set of data that is
; transmitted to an intergrated circuit that is not a general purpose
; microprocessor or microcontroller, in order to establish its normal
; operation.  The process of combining the executable ojbect code built
; from the GPL licensed source code with the hardware configuration data
; shall be considered an aggregation of another work not based on the
; Program.  While the GPL does not restrict use of the program, any
; use restriction associated with the hardware configuration data (for
; example, that it only be used with particular hardware) shall apply
; to the combined file which includes a copy of the hardware configuration
; data.

; Contact: paul@pjrc.com


; drivers for MP3 player hardware.  These drivers are loaded into
; bank1 of the flash memory, and are intended to be called from a
; program running in bank0.



.equ	reg0, 0
.equ	reg1, 1
.equ	reg2, 2
.equ	reg3, 3
.equ	reg4, 4
.equ	reg5, 5
.equ	reg6, 6
.equ	reg7, 7




.equ    location, 0x3000        ;where this program will exist

.equ    params_storage, 0x8000
.equ	sta013_config_location, 0x9000
.equ	xilinx_bitstream_location, 0xD000
.equ	xilinx_alt_bitstream_location, 0xA000

.equ	return_addr, 0x0FF0	;jump here to return to caller in bank0
.equ	rst_devices, 0x006A	;give a reset to the STA013, Xilinx, and Flash

.equ	scl_pin, 0x90
.equ	sda_pin, 0x91
.equ	hc165_load_pin, 0x93
.equ	unused_pin, 0x94
.equ	xilinx_cclk_pin, 0x95
.equ	hc165_clk_pin, 0x95
.equ	hc165_data_pin, 0x96
.equ	xilinx_din_pin, 0x97
.equ	flash_a16_pin, 0xB4
.equ	shutdown_pin, 0xB5




;*************************************************************
;**                                                         **
;**                  Memory Allocation                      **
;**                                                         **
;*************************************************************





.equ	max_attenuation, 64
.equ	default_attenuation, 12



; we'll keep a 4k chunk of DRAM memory from 8000 to 8FFF

.equ	mm_free_list, 0x8000	;2 bytes, first free block
.equ	mm_num_tracked, 0x8002	;2 bytes, number of blocks in mem tracker
.equ	num_blks_played, 0x8004	;2 bytes, number of blocks sent to STA013
.equ	last_fat_sector, 0x8006	;4 bytes, remember which fat sector used last
.equ	last_fat_block, 0x800A	;2 bytes, remember which block it's in
.equ	last_fat_fd, 0x800C	;1 byte, remember which file desc also
.equ	mm_num_free, 0x800D	;2 bytes, number of free blocks

.equ	event_queue, 0x80C0	;64 bytes, 1 per event
.equ	event_queue_size, 64

.equ	sector_buffer, 0x8100	;512 bytes (this will become obsolete)

.equ	play_req, 0x8300	;240 bytes, 128 block numbers
.equ	play_req_size, 60
.equ	clust_read_req, 0x8400	;960 bytes, 128 read requests
.equ	clust_rreq_size, 120
.equ	buf, 0x8800		;256 bytes, general purpose string buffer

.equ	fat_read_req, 0x8900	;192 bytes, 24 read requests
.equ	fat_rreq_size, 24

.equ	dont_ever_write, 0x89D2	;1 byte, don't ever write here!!
.equ	file_info, 0x8A00	;??? bytes (64 files, 24 bytes each)






; 8051 external address space usage
;
; 0000 to 5FFF	Main Program, six pages of static memory
; 6000 to 7FFF	Main Program, two pages for dynamic memory
; 8000 to 8FFF	Drivers, static variables
; 9000 to 9FFF	STA013 device driver (DMA from here to the chip)
; A000 to AFFF	IDE device driver (DMA into this area)
; B000 to BFFF	FAT32, temporary use to read cached data
; C000 to CFFF
; D000 to DFFF  FAT32, (static) list of blocks holding FAT sectors
; E000 to EFFF  Memory Manager
; F000 to FEFF	Not implemented
; FF00 to FFFF	DRAM, DMA, IDE control registers






; functions built into Paulmon2... we can't call these anymore
; because the current DRAM bus arbitrator isn't very smary and
; it pauses DMA when the PSEN strobes stop, which occurs when
; one of these functions waits to send/receive a character.
;.equ	cout, 0x0030            ;Send Acc to serial port
;.equ	cin, 0x0032             ;Get Acc from serial port
;.equ	phex, 0x0034            ;Print Hex value of Acc
;.equ	phex1, 0x002E
;.equ	phex16, 0x0036          ;Print Hex value of DPTR
;.equ	pstr, 0x0038            ;Print string pointed to by DPTR,
;.equ	upper, 0x0040           ;Convert Acc to uppercase
;.equ	pint8u, 0x004D          ;print Acc at an integer, 0 to 255
;.equ	pint8, 0x0050           ;print Acc at an integer, -128 to 127
;.equ	pint16u, 0x0053         ;print DPTR as an integer, 0 to 65535
;.equ	newline, 0x0048         ;print CR/LF (13 and 10)
.equ    prgm, 0x0059		;program flash byte in A to @DPTR
.equ	erblock, 0x0067   	;PAULMON2 v2.1 req'd



; registers implemented in the DRAM, IDE, and DMA controller
.equ	dram_page_cfg,	0xFF00
.equ	ide_data,	0xFF60
.equ	ide_err,	0xFF62
.equ	ide_sec_cnt,	0xFF64
.equ	ide_sector,	0xFF66
.equ	ide_cyl_lsb,	0xFF68
.equ	ide_cyl_msb,	0xFF6A
.equ	ide_head,	0xFF6C
.equ	ide_command,	0xFF6E
.equ	ide_status,	0xFF6E
.equ	ide_control,	0xFF7C
.equ	ide_astatus,	0xFF7C
.equ	ide_rst_bit,	0xFF40
.equ	ide_data_buf_msb, 0xFF43
.equ	dma_ide_dest,	0xFF22
.equ	dma_ide_count,	0xFF24
.equ	irq_dma_ide_ack,0xFF5C
.equ	dma_ide_go,	0xFF58
.equ	irq_ident,	0xFF50
.equ	irq_mask,	0xFF51
.equ	dma_mp3_src,	0xFF28
.equ	dma_mp3_count,	0xFF2A
.equ	dma_mp3_go,	0xFF59
.equ	irq_dma_mp3_ack,0xFF5D
.equ	irq_memcpy_ack, 0xFF5E







;IDE Command Constants.  These should never change.
.equ    ide_cmd_recal, 0x10
.equ    ide_cmd_read, 0x20
.equ	ide_cmd_read_mult, 0xC4
.equ	ide_cmd_set_mult, 0xC6
.equ    ide_cmd_write, 0x30
.equ    ide_cmd_init, 0x91
.equ    ide_cmd_id, 0xEC
.equ    ide_cmd_spindown, 0xE0
.equ    ide_cmd_spinup, 0xE1
.equ	ide_cmd_idle, 0xE1
.equ	ide_cmd_sleep, 0xE6
.equ	ide_cmd_set_features, 0xEF


; internal 8051 memory allocation

; don't use locations 0x00 to 0x07, they're needed for the registers
; don't use locations 0x08 to 0x0F, they're used by interrupt routines


.equ	event_queue_head, 0x10		;1 byte
.equ	event_queue_tail, 0x11		;1 byte
.equ	button_state, 0x12		;1 byte
.equ	state_quit, 0x13		;1 byte
.equ	dma_bug_temp, 0x14		;1 byte
.equ	state_debug, 0x15		;1 byte
.equ	bsearch_tmp, 0x16		;2 bytes

.equ	fat_rreq_head, 0x18		;1 byte
.equ	fat_rreq_tail, 0x19		;1 byte
.equ	clust_rreq_head, 0x1A		;1 byte
.equ	clust_rreq_tail, 0x1B		;1 byte
.equ	ide_state, 0x1C			;1 byte
.equ	play_req_head, 0x1D		;1 byte
.equ	play_req_tail, 0x1E		;1 byte
.equ	play_state, 0x1E		;1 byte

.equ	reserved_pm2, 0x20		;1 byte
.equ	flags, 0x21			;1 byte
.equ	state_next, 0x22		;1 byte
.equ	state_prev, 0x23		;1 byte
.equ	state_play, 0x24		;1 byte

.equ	fd_root_dir, 0x25		;1 byte
.equ	dir_read_prev_entry, 0x26	;2 bytes
.equ	dir_read_current_entry, 0x28	;2 bytes
.equ	dir_read_lfn_cksum, 0x2A	;1 byte
.equ	file_desc, 0x2B			;1 byte

.equ	first_cluster, 0x2C		;4 bytes
.equ	file_size, 0x30			;4 bytes
.equ	first_byte, 0x34		;4 bytes
.equ	cluster, 0x38			;4 bytes
.equ	f32_sector, 0x3C		;4 bytes
.equ	prev_cluster_at_block, 0x40	;2 bytes
.equ	next_cluster_at_block, 0x42	;2 bytes
.equ	cluster_offset, 0x44		;3 bytes

.equ	reserved_sector_count, 0x47	;2 bytes (obsolete, remove)

;(this group wants to eventually get moved to external memory)
.equ	blocks_per_cluster, 0x49	;1 byte   --TODO: external memory
.equ	partition_first_sector, 0x4A	;4 bytes  --TODO: external memory
.equ	sectors_per_cluster, 0x4E	;1 byte   --TODO: external memory
.equ	master_slave, 0x4F		;1 byte, ide master/slave setting

.equ	first_data_sector, 0x50		;4 bytes  --TODO: external memory
.equ	root_dir_cluster, 0x54		;4 bytes  --TODO: external memory



; internal bit addressable memory allocation
.equ	debug, 0x0E
.equ	mute_is_on, 0x0F



.org    location
.db     0xA5,0xE5,0xE0,0xA5     ;signiture bytes
.db     0,255,0,0               ;id
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;user defined
.db     255,255,255,255         ;length and checksum (255=unused)
.db     "Hardware Drivers",0


.org	location+64

;entry table... call to 0x0FE0 (within PM2) with one of these
;addresses in DPTR to call these drivers from bank0

	ljmp	xdownload		;64  download xilinx bitstream
	ljmp	sta013_init		;67  initialize the sta013
	ljmp	shutdown		;70  shut off the power supply
	ljmp	alt_xdownload		;73  download xilinx bitstream
	ljmp	sta013_rd		;76  read from the sta013
	ljmp	sta013_wr		;79  write to the sta013


	ljmp	ide_hard_reset_c	;82
	ljmp	ide_init_c		;85
	ljmp	ide_flush_c		;88
	ljmp	do_ide_loop_c		;91
	ljmp	ide_sleep_c		;94
	ljmp	ide_wakeup_c		;97
	ljmp	is_ide_idle_c		;100

	ljmp	init_memory_mgr_c	;103
	ljmp	detect_simm_size_c	;106
	ljmp	print_memory_available	;109
	ljmp	0			;112
	ljmp	malloc_blocks_c		;115
	ljmp	free_blocks_c		;118
	ljmp	next_block_c		;121

	ljmp	play_block_c		;124
	ljmp	play_resume_c		;127
	ljmp	play_suspend_c		;130
	ljmp	play_abort_c		;133
	ljmp	play_queue_avail_c	;136
	ljmp	is_play_queue_empty_c	;139

	ljmp	sta013_get_attenuation	;142
	ljmp	sta013_set_attenuation	;145
	ljmp	test_sta013_c		;148

	ljmp	detect_filesystem	;151
	ljmp	file_open_by_1st_cluster;154
	ljmp	file_cache		;157
	ljmp	file_cache_work		;160
	ljmp	file_seek		;163
	ljmp	file_read_c		;166
	ljmp	file_read_block		;169
	ljmp	file_close		;172
	ljmp	free_fat_memory		;175
	ljmp	dir_read		;178

	ljmp	update_events		;181
	ljmp	get_next_event		;184
	ljmp	put_back_event		;187
	ljmp	read_buttons_c		;190
	ljmp	wait			;193

	ljmp	write_flash_param	;196
	ljmp	read_param_1byte	;199
	ljmp	check_old_firmware	;202
	ljmp	drivers_init		;205
	ljmp	play_dma_irq_enable	;208
	ljmp	get_root_1st_cluster	;211




	;initialize critical stuff here
drivers_init:
	clr	a
	mov	ie, a			;all interrupts off until enabled
	mov     flags, a
        mov     state_quit, a
        mov     state_play, a
        mov     state_next, a
        mov     state_prev, a
        mov     event_queue_head, a
        mov     event_queue_tail, a
        acall   read_buttons
        orl     a, 0x10
        mov     button_state, a

	setb	debug

	ljmp	return_addr








	;download a bitstream to the xilinx chip.
	;return DPL=0 if ok, DPL=1 if an error

xdownload:
	mov	dptr, #xilinx_bitstream_location - 1
	acall	get_next_byte
	cjne	a, #0xFF, no_bitstream	;fill byte
	acall	get_next_byte
	cjne	a, #0x04, no_bitstream	;preamble, upper 4 bits of length
	acall	get_next_byte
	cjne	a, #0xE8, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xFA, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xF9, no_bitstream	;lower 4 bits length, fill bits

	;reset the xilinx chip... also resets the STA013 and flash
	lcall	rst_devices

	;wait for 3ms for the xilinx chip to be ready
	;at 7.3728 MHz / 12, that's 1843 instruction cycles

	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	mov	r0, #153	;1 cycle
	djnz	r0, *		;306 cycles

	setb	xilinx_din_pin
	setb	xilinx_cclk_pin
	mov	dptr, #xilinx_bitstream_location
xd_loop1:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin

	mov	a, dpl
	cjne	a, #(xilinx_bitstream_location + 11968) & 255, xd_loop1
	mov	a, dph
	cjne	a, #(xilinx_bitstream_location + 11968) >> 8, xd_loop1

	setb	xilinx_din_pin
	mov	dpl, #0
	ljmp	return_addr

get_next_byte:
	inc	dptr
	clr	a
	movc	a, @a+dptr
	ret

no_bitstream:
	setb	c
	mov	dpl, #1
	ljmp	return_addr




	;download a bitstream to the xilinx chip.
	;return DPL=0 if ok, DPL=1 if an error

alt_xdownload:
	mov	dptr, #xilinx_alt_bitstream_location - 1
	acall	get_next_byte
	cjne	a, #0xFF, no_bitstream	;fill byte
	acall	get_next_byte
	cjne	a, #0x04, no_bitstream	;preamble, upper 4 bits of length
	acall	get_next_byte
	cjne	a, #0xE8, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xFA, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xF9, no_bitstream	;lower 4 bits length, fill bits

	;reset the xilinx chip... also resets the STA013 and flash
	lcall	rst_devices

	;wait for 3ms for the xilinx chip to be ready
	;at 7.3728 MHz / 12, that's 1843 instruction cycles

	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	mov	r0, #153	;1 cycle
	djnz	r0, *		;306 cycles

	setb	xilinx_din_pin
	setb	xilinx_cclk_pin
	mov	dptr, #xilinx_alt_bitstream_location
alt_xd_loop1:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin

	mov	a, dpl
	cjne	a, #(xilinx_alt_bitstream_location + 11968) & 255, alt_xd_loop1
	mov	a, dph
	cjne	a, #(xilinx_alt_bitstream_location + 11968) >> 8, alt_xd_loop1

	setb	xilinx_din_pin
	mov	dpl, #0
	ljmp	return_addr






cout_dot:
	mov	a, #'.'
	ljmp	cout



sta013_init:

	acall	cout_dot

	;read the STA013 ident register
	mov	r4, #1
	acall	sta013_read
	jc	sta013_init1
	cjne	r3, #0xAC, sta013_init0
	sjmp	sta013_init2

sta013_init0:
	mov	a, r3
	lcall	phex
sta013_init1:
	mov	a, #'*'
	lcall	cout

	;read the STA013 ident register
	mov	r4, #1
	acall	sta013_read
	jc	sta013_error
	cjne	r3, #0xAC, sta013_error

sta013_init2:
	acall	cout_dot


	;check to see if the sta013 data file is loaded into memory

	mov	dptr, #sta013_config_location
	acall	check_data_file
	jc	sta013_error

	acall	cout_dot

	;now attempt to download the config data
	;to the STA013 chip

	acall	cout_dot

	acall	sta013_config

	acall	cout_dot

	mov	r4, #20
	mov	r3, #1
	acall	sta013_write		;mute on
	jc	sta013_error
	setb	mute_is_on

	acall	cout_dot

	mov	r4, #114
	mov	r3, #1
	acall	sta013_write		;run command
	jc	sta013_error

	;acall	cout_dot

	;mov	r4, #19
	;mov	r3, #1
	;acall	sta013_write		;begin play
	;jc	sta013_error

	;acall	cout_dot

	;mov	r4, #20
	;mov	r3, #0
	;acall	sta013_write		;mute off
	;jc	sta013_error

	;acall	cout_dot

	clr	c
	mov	dpl, #0
	ljmp	return_addr

sta013_error:
	setb	c
	mov	dpl, #1
	ljmp	return_addr










sta013_config:
	mov	dptr, #sta013_config_location
	acall	get_16_bits	;r6/r7 has cksum addr
	mov	a, r6
	anl	a, #11111110b
	mov	r6, a
cfgloop:acall	get_8_bits	;get the address
	mov	r4, a
	acall	get_8_bits	;get the data byte
	mov	r3, a
	acall	sta013_write	;send this byte to its address
	mov	a, r4
	add	a, #250
	jnz	cfgloop2
	;delay at least 60 us if the write was a soft reboot cmd
	mov	r5, #200
	djnz	r5, *
cfgloop2:
	mov	a, r6
	cjne	a, dpl, cfgloop	;send all the bytes
	mov	a, r7
	cjne	a, dph, cfgloop
	ret



	;checks for a data file in memory @dptr.  The test
	;is simple... the first 2 bytes are the location of
	;the file's last two bytes, that are a simple 16 bit
	;checksum of all the data bytes

check_data_file:
	acall	get_16_bits	;r6/r7 has cksum addr
	clr	a
	mov	r0, a		;r0/r1 will hold the computed cksum
	mov	r1, a
ckloop:	acall	get_8_bits	;get each byte
	add	a, r0		;add it to the cksum
	mov	r0, a
	clr	a
	addc	a, r1
	mov	r1, a
	mov	a, r6
	cjne	a, dpl, ckloop	;check all the data bytes
	mov	a, r7
	cjne	a, dph, ckloop
	acall	get_8_bits	;get cksum lsb
	mov	b, r0
	cjne	a, b, ckfail
	acall	get_8_bits	;get cksum msb
	mov	b, r1
	cjne	a, b, ckfail
	clr	c
	ret
ckfail:	setb	c

	mov	a, #'!'
	lcall	cout
	ljmp	0

	ret



get_8_bits:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	ret

get_16_bits:
	acall	get_8_bits
	mov	r6, a
	acall	get_8_bits
	mov	r7, a
	ret
	


	;allow the main program to call the read and write routines
sta013_rd:
	acall	sta013_read
	clr	a
	addc	a, #0
	ljmp	return_addr
sta013_wr:
	acall	sta013_write
	clr	a
	addc	a, #0
	mov	dpl, a
	ljmp	return_addr




	;read a byte from the sta013 at address r4 and return it
	;in r3.  Carry cleared if success, set for fail
sta013_read:
	acall	i2c_start
	mov	a, #10000110b
	acall	i2c_write		;wr device addr, wr cmd
	jc	s013r3
	mov	a, r4
	acall	i2c_write		;wr addr of data to read
	jc	s013r3
	acall	i2c_stop
	acall	i2c_start
	mov	a, #10000111b		;wr device addr, rd cmd
	acall	i2c_write
	jc	s013r3
	acall	i2c_read
	mov	r3, a
	clr	c
s013r3: ajmp	i2c_stop



	;write a byte in r3 to the sta013 at address r4
	;Carry cleared if success, set for fail
sta013_write:
	acall	i2c_start
	mov	a, #10000110b
	acall	i2c_write
	jc	s013w3
	mov	a, r4
	acall	i2c_write
	jc	s013w3
	mov	a, r3
	acall	i2c_write
s013w3: ajmp	i2c_stop	;i2c stop doesn't change carry bit







i2c_read:  ;reads i2c bus into acc, send ack bit from Carry
	rlc	a
        mov     r5, #8
        setb    sda_pin
	nop
i2cr:   nop
        setb    scl_pin
        nop
        nop
        mov     c, sda_pin
        rlc     a
	nop
        clr     scl_pin
        djnz    r5, i2cr
        mov     sda_pin, c
	nop
	nop
        setb    scl_pin
	nop
	nop
        clr     scl_pin
        ret


i2c_write:  ;writes acc to i2c bus, return ack bit in Carry
        mov     r5, #8
i2cw:   rlc     a
        mov     sda_pin, c
	nop
        setb    scl_pin
	nop
        clr     scl_pin
        djnz    r5, i2cw
        rlc     a
        setb    sda_pin
	nop
        setb    scl_pin
	nop
        mov     c, sda_pin
        clr     scl_pin
        ret


i2c_start:
        setb    sda_pin
        setb    scl_pin
	nop
	nop
        clr     sda_pin
	nop
	nop
        clr     scl_pin
        ret

i2c_stop:
        clr     sda_pin
	nop
	nop
        setb    scl_pin
	nop
	nop
        setb    sda_pin
        ret

dly:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	ret
	


shutdown:
	mov	r0, #250
sd2:
	clr	shutdown_pin
	setb	shutdown_pin
	djnz	r0, sd2
	ljmp	return_addr












;*************************************************************
;**                                                         **
;**                     USER Interface                      **
;**                                                         **
;*************************************************************


.equ	event_next, 1
.equ	event_play_pause, 2
.equ	event_prev, 3
.equ	event_random, 4
.equ	event_vol_up, 5
.equ	event_vol_down, 6




	;return the next pending "event" in DPL (zero for none)
get_next_event:
	mov	a, event_queue_tail
	cjne	a, event_queue_head, nxtevt2
	;- reenable interrupt here
	mov	dpl, #0			;no events pending
	ljmp	return_addr
nxtevt2:
	inc	a
	cjne	a, #event_queue_size, nxtevt3
	clr	a
nxtevt3:
	mov	event_queue_tail, a
	add	a, #event_queue & 255	;add Acc + event_queue
	mov	dpl, a
	mov	dph, #event_queue >> 8	;assume event queue in 256 byte area
	movx	a, @dptr		;read the event byte from queue
	;- reenable interrupt here
	mov	dpl, a
	ljmp	return_addr




	;push an event in Acc back into the event queue.  This
	;will probably be called when the main program got an
	;event that is can't process now, but will be able to
	;process in the near future.
put_back_event:
	mov	a, r4
	;TODO: this version should "try harder" than add_new_event.
	;add_new_event should be changed to always leave a couple
	;free slots on the queue.

	;FIXME: this adds it back into the end of the queue, when
	;it ought to be "pushed" onto the front.

	;- disable interrupt here
	lcall	add_new_event
	;- reenable interrupt here
	ljmp	return_addr














;this will eventually be run from an interrupt routine (timer0)
;so that "events" will be detected automatically and added to
;the queue.

update_events:
wait:
	jb	ri, keypress
	lcall	check_buttons
	ljmp	return_addr

keypress:
	lcall	cin
	lcall	upper
	lcall	cout
	mov	r2, a

check_quit:
	mov	dptr, #str_quit
	mov	r0, #state_quit
	acall	lexer
	jnc	check_debug
	ljmp	quit

check_debug:
	mov	dptr, #str_debug
	mov	r0, #state_debug
	acall	lexer
	jnc	check_next
	cpl	debug
	ljmp	return_addr

check_next:
	mov	dptr, #str_next
	mov	r0, #state_next
	acall	lexer
	jnc	check_prev
	mov	a, #event_next
	lcall	add_new_event
	ljmp	return_addr

check_prev:
	mov	dptr, #str_previous
	mov	r0, #state_prev
	acall	lexer
	jnc	check_play
	mov	a, #event_prev
	lcall	add_new_event
	ljmp	return_addr

check_play:
	mov	dptr, #str_play
	mov	r0, #state_play
	acall	lexer
	jnc	check_all_done
	mov	a, #event_play_pause
	lcall	add_new_event
	ljmp	return_addr

check_all_done:
	ljmp	return_addr







;A simple fixed-string lexer.  For each character received from an
;input stream, this routine is called once for every string which
;is to be recognized within the input stream.

;  Input registers:
;    r0    Memory location to the index variable for this
;          particular string.
;    r2    The character we received.
;    dptr  Pointer to the string pattern we want to match.

;  Output registers:
;    c     Carry set if complete string found, clear if no match yet.
;    mem   Value @r0 adjusted appropriately for the next call.

lexer:
        mov     a, @r0
        inc     a               ;don't look at length byte
        movc    a, @a+dptr      ;get character from string 
        clr     c
        subb    a, r2
        jnz     lex_nope
           ;at this point, it matches the string, so now
           ;the question becomes "are we at the end?"
        inc     @r0             ;advance index
        clr     a
        movc    a, @a+dptr      ;get string length
        clr     c
        subb    a, @r0          ;subtract index
        jz      lex_match
           ;we're not at the end of the string yet
        clr     c
        ret
lex_match: ;we've found all of the string now
        mov     @r0, a          ;reset index automatically
        setb    c
        ret
lex_nope:  ;the received character doesn't match the string
           ;so we set the index back to zero, and return with
           ;carry clear
        mov     @r0, #0
        clr     c
        ret







	;put a new event (in Acc) into the event queue.  This is
	;intended (someday) to be called only from within the
	;interrupt routines
add_new_event:
	push	acc
	mov	a, event_queue_head
	inc	a
	cjne	a, #event_queue_size, addevt2
	clr	a
addevt2:cjne	a, event_queue_tail, addevt3
	;if we get here, the event queue is full and we are not
	;able to add another event to it, so this event will
	;just get discarded... hopefully is wasn't too important.
	pop	acc
	ret
addevt3:mov	event_queue_head, a
	add	a, #event_queue & 255	;add Acc + event_queue
	mov	dpl, a
	mov	dph, #event_queue >> 8	;assume event queue in 256 byte area
	pop	acc
	movx	@dptr, a		;put the event byte into event queue
	ret




;button 1: 02	forward
;button 2: 10	play
;button 3: 20	previous
;button 4: 08	random
;button 5: 01	vol_plus
;button 6: 04	vol_minus

check_buttons:
	lcall	read_buttons
	cjne	a, button_state, ckb2
	ret
ckb2:
	mov	b, button_state
	mov	button_state, a
	jnb	acc.4, ckb3
	jb	b.4, ckb3
	mov	a, #event_play_pause
	lcall	add_new_event
	ret
ckb3:
	jnb	acc.1, ckb4
	jb	b.1, ckb4
	mov	a, #event_next
	lcall	add_new_event
	ret
ckb4:
	jnb	acc.5, ckb5
	jb	b.5, ckb5
	mov	a, #event_prev
	lcall	add_new_event
	ret
ckb5:
	jnb	acc.3, ckb6
	jb	b.3, ckb6
	mov	a, #event_random
	lcall	add_new_event
	ret
ckb6:
	jnb	acc.0, ckb7
	jb	b.0, ckb7
	mov	a, #event_vol_up
	lcall	add_new_event
	ret
ckb7:
	jnb	acc.2, ckb8
	jb	b.2, ckb8
	mov	a, #event_vol_down
	lcall	add_new_event
	ret
ckb8:
	ret





read_buttons:
	clr	hc165_load_pin
	nop
	setb	hc165_load_pin
	mov	a, #255
	mov	c, hc165_data_pin
	clr	hc165_clk_pin
	rlc	a
	setb	hc165_clk_pin
	mov	c, hc165_data_pin
	clr	hc165_clk_pin
	rlc	a
	setb	hc165_clk_pin
	mov	c, hc165_data_pin
	clr	hc165_clk_pin
	rlc	a
	setb	hc165_clk_pin
	mov	c, hc165_data_pin
	clr	hc165_clk_pin
	rlc	a
	setb	hc165_clk_pin
	mov	c, hc165_data_pin
	clr	hc165_clk_pin
	rlc	a
	setb	hc165_clk_pin
	mov	c, hc165_data_pin
	rlc	a
	cpl	a
	ret

read_buttons_c:
	acall	read_buttons
	mov	dpl, a
	ljmp	return_addr





;*************************************************************
;**                                                         **
;**           MP3 Playback (DMA Request Queue)              **
;**                                                         **
;*************************************************************



	;add a block (r2/r3) to the playback request queue
	;c=0 for success, c=1 if request queue is full.  The
	;application calls here to cause MP3 data to be played
	;r4/r5 pass the number of bytes
play_block_c:

	push flags
	;setb debug
	 mov	dptr, #msg_play_block
	 lcall	pstr_d
	 mov	a, r3
	 lcall	phex_d
	 mov	a, r2
	 lcall	phex_d
	 lcall	space_d
	 lcall	block_desc_addr
	 inc	dptr
	 inc	dptr
	 inc	dptr
	 inc	dptr
	 movx	a, @dptr
	 inc	dptr
	 push	acc
	 movx	a, @dptr
	 inc	dptr
	 push	acc
	 movx	a, @dptr
	 inc	dptr
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 lcall	space_d
	 inc	dptr
	 inc	dptr
	 inc	dptr
	 inc	dptr
	 inc	dptr
	 movx	a, @dptr
	 inc	dptr
	 push	acc
	 movx	a, @dptr
	 inc	dptr
	 push	acc
	 movx	a, @dptr
	 inc	dptr
	 push	acc
	 movx	a, @dptr
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 lcall	newline_d
	pop flags

	clr	c
	mov	a, r5		;convert # bytes into 16 bit words
	rrc	a
	mov	r5, a
	mov	a, r4
	rrc	a
	addc	a, #0
	mov	r4, a
	mov	a, r5
	addc	a, #0
	mov	r5, a

	clr	ex0		; -- disable STA013 interrupt here
	mov	a, play_req_head
	cjne	a, #255, play_block2

	;kickstart the playing, since we're not expecting an interrupt
	clr	a
	mov	play_req_head, a
	mov	play_req_tail, a
	lcall	play_dma
	setb	ex0

	jb	mute_is_on, turn_mute_off
	mov	dpl, #0
	ljmp	return_addr

turn_mute_off:
	mov	r4, #20
	mov	r3, #0
	lcall	sta013_write		;mute off

	mov	r0, #100		;brief delay, to transfer enough
	djnz	r0, *			;mp3 data to get the sta013 to
					;move from "init" to "decode" mode
	mov	r4, #20
	mov	r3, #0
	lcall	sta013_write		;mute off

	mov	r4, #114
	mov	r3, #1
	lcall	sta013_write		;start playing
	clr	mute_is_on
	mov	dpl, #0
	ljmp	return_addr



play_block2:
	inc	a
	cjne	a, #play_req_size, play_block3
	clr	a
play_block3:
	cjne	a, play_req_tail, play_block4
	;if we get here, the play request queue is full, so we
	;can't add this block
	setb	ex0		; -- reenable STA013 interrupt here
	mov	dpl, #1
	ljmp	return_addr

play_block4:
	mov	play_req_head, a
	rl	a
	rl	a
	add	a, #play_req & 255
	mov	dpl, a
	clr	a
	addc	a, #play_req >> 8
	mov	dph, a			;now DPTR points to new request
	mov	a, r2
	movx	@dptr, a		;write block number to queue
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	mov	a, r4			;write data length to queue
	movx	@dptr, a
	inc	dptr
	mov	a, r5
	movx	@dptr, a
	setb	ex0		; -- reenable STA013 interrupt here
	mov	dpl, #0
	ljmp	return_addr




play_abort_c:
	clr	ex0			;disable STA013 interrupt
	mov	dptr, #dma_mp3_count+1
	clr	a
	movx	@dptr, a		;stop current DMA transfer, if any
	mov	dptr, #dma_mp3_count	;actually set it so that just one
	inc	a			;more word is to be transfered
	movx	@dptr, a		;by tweaking "dma_mp3_count".
	mov	a, play_req_head
	cpl	a
	jz	play_abort3
play_abort2:
	clr	a
	mov	play_req_tail, a	;flush play request queue
	mov	play_req_head, a
play_abort3:
	setb	ex0			;reenable STA013 interrupt
	;should tell the STA013 to flush its input buffer, if
	;that can be done somehow
	ljmp	return_addr



	;normally this is done by the interrupt routine, but for
	;the very first block played, we're not expecting an
	;interrupt, so we have to start it manually.

	; TODO: it would be cleaner if the FPGA had a feature
	; to manually set the interrupt... then we could just
	; set it and let "isr.asm" handle this task.
play_dma:
	;let's map the block with the MP3 data to 0x9000
	mov	dptr, #dram_page_cfg + (0x09 * 2)
	mov	a, r2
	movx	@dptr, a
	inc	dptr
	mov	a, r3
	movx	@dptr, a

	;finally, we're done with the memory management stuff, and
	;we can now set up the DMA transfer and start it.
	mov	dptr, #dma_mp3_src
	clr	a
	movx	@dptr, a		;write LSB of src addr
	inc	dptr
	mov	a, #0x90
	movx	@dptr, a		;write MSB of src addr
	mov	dptr, #dma_mp3_count
	mov	a, r4
	movx	@dptr, a		;write LSB of length
	inc	dptr
	mov	a, r5
	movx	@dptr, a		;write MSB of length
	clr	a
	mov	dptr, #dma_mp3_go
	movx	@dptr, a		;begin the transfer

	;now the DMA is happening... in all likelyhood, the STA013 has
	;already asserted its DATA_REQ signal by the time we could get
	;through all that work to set up the DMA transfer.  If the
	;latency of this code ever gets to be a problem, the FPGA
	;will probably have to be changed to double buffer STA013
	;DMA requests, which would allow 0.1 second latency for the
	;fastest MP3 (320 kbps) !!

	mov	dptr, #num_blks_played
	movx	a, @dptr
	add	a, #1		;increment "num_blks_played"
	movx	@dptr, a
	inc	dptr
	movx	a, @dptr
	addc	a, #0
	movx	@dptr, a

	ret





play_dma_irq_enable:
        clr     a
        mov     dptr, #irq_dma_ide_ack
        movx    @dptr, a
        mov     dptr, #irq_dma_mp3_ack
        movx    @dptr, a
        mov     dptr, #irq_memcpy_ack
        movx    @dptr, a
        mov     dptr, #irq_mask
        mov     a, #00000010b
        movx    @dptr, a
        mov     play_req_head, #255
        mov     play_req_tail, #255
        anl     tcon, #0xF0             ;low level trigger interrupts
        setb    ex0
        setb    ea
	ljmp	return_addr






	;return the number of available position in the play
	;queue.  If it's zero, the playback queue is full and
	;an attempt to call "play_block" will return with C=1
	; (perhaps there's a more efficient way to do this math?)
play_queue_avail_c:
	clr	ex0			;disable STA013 interrupt here
	mov	r0, play_req_tail
	mov	a, play_req_head
	setb	ex0			;reenable STA013 interrupt here
	cjne	a, #255, pqav2
	mov	a, #play_req_size-1
	mov	dpl, a
	ljmp	return_addr
pqav2:
	clr	c
	subb	a, r0
	jnc	pqav3
	add	a, #play_req_size
pqav3:	;now acc has the number that are in the queue
	mov	r0, a
	mov	a, #play_req_size
	clr	c
	subb	a, r0
	;now acc has the number of avaiable slots, but
	;one is never used, so we need to subtract 1
	jz	pqav4
	add	a, #255
pqav4:	mov	dpl, a
	ljmp	return_addr




	; TODO: simplify this function
	;return C=1 if the playback queue is completely empty
is_play_queue_empty_c:
	mov	dptr, #msg_play_queue_empty
	lcall	pstr_d
	clr	ex0
	mov	r0, play_req_head
	mov	r1, play_req_tail
	setb	ex0
	mov	a, r0
	lcall	phex_d
	mov	a, #'-'
	lcall	cout_d
	mov	a, r1
	lcall	phex_d
	lcall	newline_d
	mov	a, r0
	cpl	a
	jz	pqempty2
	clr	c
	mov	dpl, #0
	ljmp	return_addr
pqempty2:
	setb	c
	mov	dpl, #1
	ljmp	return_addr




play_suspend_c:
	clr	ex0
	mov	dptr, #irq_mask
	movx	a, @dptr
	anl	a, #11111101b
	movx	@dptr, a
	setb	ex0
	ljmp	return_addr



play_resume_c:
	clr	ex0
	mov	dptr, #irq_mask
	movx	a, @dptr
	orl	a, #00000010b
	movx	@dptr, a
	setb	ex0
	ljmp	return_addr









	;return the number of blocks played (since the last call
	;to this function).  This will allow the appication to track
	;the status of playback, perhaps to print elapsed/remaining
	;time on the LCD display.
play_num_sent:
	mov	dptr, #num_blks_played
	clr	ex0			;disable STA013 interrupt here
	movx	a, @dptr
	mov	r4, a
	clr	a
	movx	@dptr, a
	inc	dptr
	movx	a, @dptr
	mov	r5, a
	clr	a
	movx	@dptr, a
	setb	ex0			;reenable STA013 interrupt here
	ret




;this test may seem extreemly paranoid, but there have been problems
;initializing some STA013 chips, where the problem basically shows
;up as the data request pin being stuck either high or low and the
;chip produces no sound.  The chip otherwise appears to be working,
;so the only good way to detect the problem is to send a short clip
;of MP3 data, larger than the STA013's input buffer, that is a known
;length of time, and measure how long it takes the STA013 to consume
;all the input data.  This code sends a 1/2 second clip of silent
;MP3 (1046 bytes long), and measures the elapsed time with a simple
;delay loop.  DPL=0 is the correct time elapses, of DPL=1 if the STA013
;consumes the data too quickly or too slowly (including timeout).


test_sta013_c:
	;grab a block and map it at 0x9000.... this code is used
	;before the memory manager is initialized, so we can not
	;call malloc to get a block.  We'll just grab a block and
	;use it.
	mov	r2, #34
	mov	r3, #0
	mov	a, #9
	lcall	map_block

	;fill it with a null mp3 data that represents
        ;approximately 0.5 seconds of silence
        acall   fill_null_mp3

        mov     dptr, #msg_test_sta013
        lcall   pstr
        mov     dptr, #dma_mp3_src
        clr     a
        movx    @dptr, a                ;write LSB of src addr
        inc     dptr
        mov     a, #0x90
        movx    @dptr, a                ;write MSB of src addr
        mov     dptr, #dma_mp3_count
        mov     a, #0x08
        movx    @dptr, a                ;write LSB of length
        inc     dptr
        mov     a, #0x02
        movx    @dptr, a                ;write MSB of length
        clr     a
        mov     dptr, #dma_mp3_go
        movx    @dptr, a                ;begin the transfer

	;time how long it take for the STA013 to consume this data

        mov     r7, #0                  ;r7 to count # of loops
test_mp3_wait:
        mov     r6, #0
test_mp3_wait2:
        mov     dptr, #irq_ident        ;2
        movx    a, @dptr                ;2
        jb      acc.1, test_play_done   ;2
        djnz    r6, test_mp3_wait2      ;2
        inc     r7
        cjne    r7, #80, test_mp3_wait2

        ;now r7 will have an integer which represents how long we
        ;had to wait for the sta013 to completely consume this data
	; ... it should take r7 == 0x32

test_play_done:
        mov     dptr, #msg_play_len
        lcall   pstr
        mov     a, r7
        lcall   phex
        lcall   space
        lcall   space

        ;did we pass the test... 40 to 60 are ok
        mov     a, r7
        add     a, #216
        jnc     test_sta013_fail
        mov     a, r7
        add     a, #195
        jc      test_sta013_fail
        mov     dptr, #mesg_ok
        lcall   pstr
	mov	dpl, #0
	ljmp	return_addr

test_sta013_fail:
        mov     dptr, #mesg_error
        lcall   pstr
	mov	dpl, #1
	ljmp	return_addr



	;build up a 1/2 second MP3 clip of silence.  This code copies
	;only the first part required part of each block of the MP3
	;file, so that we can store the MP3 as only 150 bytes in the
	;code memory, plus 79 bytes for this little routine.
        ;total mp3 length = 1046 bytes

fill_null_mp3:
        mov     dptr, #0x9000
        mov     a, #0x55
        mov     r0, #210
fnull2:
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a
        inc     dptr
        djnz    r0, fnull2

        mov     dptr, #null_headers
        mov     a, sp
        mov     r0, a           ;16 byte temp buffer on the stack
        mov     r1, a
        add     a, #16
        mov     sp, a
        inc     r1
        mov     r2, #10
fnull3:
        mov     r3, #15
        mov     a, r1
        mov     r0, a
fnull4:
        clr     a
        movc    a, @a+dptr
        inc     dptr
        mov     @r0, a
        inc     r0
        djnz    r3, fnull4
        push    dph
        push    dpl

        mov     a, r1
        mov     r0, a
        mov     a, @r0
        inc     r0
        mov     dpl, a
        mov     a, @r0
        inc     r0
        mov     dph, a
        mov     r3, #13
fnull5:
        mov     a, @r0
        inc     r0
        movx    @dptr, a
        inc     dptr
        djnz    r3, fnull5
        pop     dpl
        pop     dph
        djnz    r2, fnull3

        mov     a, sp
        add     a, #240
        mov     sp, a

        ret



sta013_get_attenuation:
	mov	r4, #0x46
	lcall	sta013_read
	mov	dpl, r3
	ljmp	return_addr


sta013_set_attenuation:
	mov	a, r3
	add	a, #256 - max_attenuation
	jnc	sta013_set_atn2
	mov	r3, #max_attenuation
sta013_set_atn2:
	 ;mov	dptr, #mesg_sta013_vol
	 ;lcall	pstr
	 ;mov	a, r3
	 ;lcall	phex
	 ;lcall	newline
	mov	r4, #0x46		;write left attenuation
	lcall	sta013_write
	mov	r4, #0x48
	lcall	sta013_write		;write right attenuation
	ljmp	return_addr

mesg_sta013_vol:
	.db	"STA013 attenuation = ",0







;*************************************************************
;**                                                         **
;**      Detect Partitions and FAT32 Filesystem Info        **
;**                                                         **
;*************************************************************



	;detect what type of filesystem we're using, and initialize
	;all the filesystem variables for it.  Return DPL=0 if the
	;filesystem could not be detected, of DPL=non-zero if it was
	;properly detected.  Perhaps someday the non-zero code would
	;represent what type of filesystem... but today we can only
	;handle FAT32 on hard drives.

detect_filesystem:
read_mbr:
	mov	f32_sector+3, #0
	mov	f32_sector+2, #0
	mov	f32_sector+1, #0
	mov	f32_sector+0, #0

	mov	dptr, #mesg_read_mbr
	lcall	pstr
	mov	dptr, #sector_buffer
	lcall	ide_read_sector
	mov	dptr, #sector_buffer + 0x1FE
	movx	a, @dptr
	cjne	a, #0x55, bad_mbr
	inc	dptr
	movx	a, @dptr
	cjne	a, #0xAA, bad_mbr

	mov	dptr, #sector_buffer + 0x1C2
	movx	a, @dptr
	lcall	is_fat32_type
	jnc	found_partition
	lcall	print_unknown_type

	mov	dptr, #sector_buffer + 0x1D2
	movx	a, @dptr
	lcall	is_fat32_type
	jnc	found_partition
	lcall	print_unknown_type

	mov	dptr, #sector_buffer + 0x1E2
	movx	a, @dptr
	lcall	is_fat32_type
	jnc	found_partition
	lcall	print_unknown_type

	mov	dptr, #sector_buffer + 0x1F2
	movx	a, @dptr
	lcall	is_fat32_type
	jnc	found_partition
	lcall	print_unknown_type
	mov	dpl, #0
	ljmp	return_addr

bad_mbr:
	lcall	print_sector_buffer
	lcall	newline
	mov	dptr, #mesg_bad_mbr
	lcall	pstr
	mov	dpl, #0
	ljmp	return_addr


found_partition:
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	lcall	get_byte
	mov	partition_first_sector+0, a
	lcall	get_byte
	mov	partition_first_sector+1, a
	lcall	get_byte
	mov	partition_first_sector+2, a
	lcall	get_byte
	mov	partition_first_sector+3, a
	;lcall	get_byte
	;mov	partition_size+0, a
	;lcall	get_byte
	;mov	partition_size+1, a
	;lcall	get_byte
	;mov	partition_size+2, a
	;lcall	get_byte
	;mov	partition_size+3, a

	mov	dptr, #mesg_partition_at
	lcall	pstr
	mov	r0, #partition_first_sector
	push	flags
	setb	debug
	lcall	phex32_at_r0
	pop	flags

	lcall	newline
	ljmp	read_vol_id



bad_vol_id:
	lcall	print_sector_buffer
	mov	dptr, #mesg_bad_vol_id
	lcall	pstr
	lcall	print_sector_buffer
	mov	dpl, #0
	ljmp	return_addr


read_vol_id:
	push	flags
	setb	debug
	mov	dptr, #mesg_read_vol_id
	lcall	pstr

	mov	f32_sector+0, partition_first_sector+0
	mov	f32_sector+1, partition_first_sector+1
	mov	f32_sector+2, partition_first_sector+2
	mov	f32_sector+3, partition_first_sector+3
	
	mov	dptr, #sector_buffer
	lcall	ide_read_sector

	;acall	print_sector_buffer

	mov	dptr, #sector_buffer + 0x1FE
	movx	a, @dptr
	cjne	a, #0x55, bad_vol_id
	inc	dptr
	movx	a, @dptr
	cjne	a, #0xAA, bad_vol_id
	mov	dptr, #sector_buffer + 11
	movx	a, @dptr
	jnz	bad_vol_id			;require 512 byte sector
	inc	dptr
	movx    a, @dptr
	cjne	a, #2, bad_vol_id		;require 512 byte sector
	mov	dptr, #sector_buffer + 16
	movx    a, @dptr
	cjne	a, #2, bad_vol_id		;require BPB_NumFATs == 2

	mov	dptr, #mesg_vol_id_ok
	lcall	pstr

	clr	c
	mov	dptr, #sector_buffer + 36	;offset of BPB_FATSz32
	movx	a, @dptr
	rlc	a
	mov	r2, a				;r2 to r5 has BPB_FATSz32 * 2
	inc	dptr
	movx	a, @dptr
	rlc	a
	mov	r3, a
	inc	dptr
	movx	a, @dptr
	rlc	a
	mov	r4, a
	inc	dptr
	movx	a, @dptr
	rlc	a
	mov	r5, a

	mov	dptr, #mesg_sectors_for_fats
	lcall	pstr
	mov	r0, #2
	lcall	phex32_at_r0
	lcall	newline

	mov	dptr, #sector_buffer + 14	;read BPB_RsvdSecCnt
	movx	a, @dptr
	mov	reserved_sector_count+0, a
	inc	dptr
	movx	a, @dptr
	mov	reserved_sector_count+1, a

	mov	dptr, #mesg_reserved_sectors
	lcall	pstr
	mov	r0, #reserved_sector_count
	lcall	phex16_at_r0
	lcall	newline

	mov	a, r2				;add (BPB_FATSz32 * 2)
	add	a, reserved_sector_count+0	;to reserved_sector_count
	mov	first_data_sector+0, a		;and store as first_data_sector
	mov	a, r3
	addc	a, reserved_sector_count+1
	mov	first_data_sector+1, a
	mov	a, r4
	addc	a, #0
	mov	first_data_sector+2, a
	mov	a, r5
	addc	a, #0
	mov	first_data_sector+3, a

	mov	a, first_data_sector+0
	add	a, partition_first_sector+0
	mov	first_data_sector+0, a
	mov	a, first_data_sector+1
	addc	a, partition_first_sector+1
	mov	first_data_sector+1, a
	mov	a, first_data_sector+2
	addc	a, partition_first_sector+2
	mov	first_data_sector+2, a
	mov	a, first_data_sector+3
	addc	a, partition_first_sector+3
	mov	first_data_sector+3, a

	 mov	dptr, #mesg_first_data_sec
	 lcall	pstr
	 mov	r0, #first_data_sector
	 lcall	phex32_at_r0
	 lcall	newline

	mov	dptr, #sector_buffer + 13	;read BPB_SecPerClus
	movx	a, @dptr
	mov	sectors_per_cluster, a

	anl	a, #11111000b
	swap	a
	rl	a
	mov	blocks_per_cluster, a
	mov	a, sectors_per_cluster
	anl	a, #00000111b
	jz	rd_vol_id2
	inc	blocks_per_cluster
	 mov	dptr, #mesg_per_cluster4
	 lcall	pstr
rd_vol_id2:
	 mov	dptr, #mesg_per_cluster1
	 lcall	pstr
	 mov	a, sectors_per_cluster
	 lcall	pint8u
	 mov	dptr, #mesg_per_cluster2
	 lcall	pstr
	 mov	a, blocks_per_cluster
	 lcall	pint8u
	 mov	dptr, #mesg_per_cluster3
	 lcall	pstr

	mov	dptr, #sector_buffer + 44	;read BPB_RootClus
	movx	a, @dptr
	mov	root_dir_cluster+0, a
	inc	dptr
	movx	a, @dptr
	mov	root_dir_cluster+1, a
	inc	dptr
	movx	a, @dptr
	mov	root_dir_cluster+2, a
	inc	dptr
	movx	a, @dptr
	mov	root_dir_cluster+3, a

	 mov	dptr, #mesg_root_dir_at_cluster
	 lcall	pstr
	 mov	r0, #root_dir_cluster
	 lcall	phex32_at_r0
	 lcall	newline

	pop	flags
	mov	dpl, #1
	ljmp	return_addr





is_fat32_type:
	add	a, #256 - 11		;is it type 0B
	jz	isf32b
	add	a, #255			;is it type 0C
	jz	isf32b
	add	a, #12
	setb	c
	ret
isf32b:
	clr	c
	ret


print_unknown_type:
	push	acc
	mov	dptr, #msg_unknown_partion_type
	lcall	pstr
	pop	acc
	lcall	phex
	lcall	newline
	ret

get_byte:
	movx	a, @dptr
	inc	dptr
	ret



get_root_1st_cluster:
	mov	dpl, root_dir_cluster+0
	mov	dph, root_dir_cluster+1
	mov	b, root_dir_cluster+2
	mov	a, root_dir_cluster+3
	ljmp	return_addr



;*************************************************************
;**                                                         **
;**                    FAT32 Filesystem                     **
;**                                                         **
;*************************************************************

;TODO: I received a report that laptop drives formatted by win2k
;as FAT32 won't work.  Need to find a win2k machine and format
;a drive for testing.



	.equ	file_info_size, 24

	;convert a file descriptor in Acc into a the address
	;of that file's info, and return it in dptr.  This is
	;the only place that gets the file_info's address, so
	;it'll (hopefull) be less painful to add more elements
	;to a file_info if needed.
fd_to_ptr:
	anl	a, #63			;only 64 files allowed
	mov	b, #file_info_size	;sizeof(struct file_info)
	mul	ab
	add	a, #file_info & 255
	mov	dpl, a
	mov	a, #file_info >> 8
	addc	a, b
	mov	dph, a
	ret



add_a_to_dptr:
	add	a, dpl
	mov	dpl, a
	jnc	add_a2dptr2
	inc	dph
add_a2dptr2:
	ret





	;open a file.  r2/r3/r4/r5 has the 32 bit first cluster.
	;return the file descriptor (0 to 63), or -1 if failure

file_open_by_1st_cluster:
	 mov	dptr, #msg_fopen
	 lcall	pstr_d
	;first, look for an unused entry in the file_info table
	mov	r1, #0		;r1 will be the file descriptor
fopen_loop1:
	mov	a, r1
	lcall	fd_to_ptr
	movx	a, @dptr	;get the flags for this file_info
	anl	a, #1
	jz	fopen_ok	;use it if it's available
	inc	r1
	cjne	r1, #16, fopen_loop1
	;if we get here, all the file_info slots are in use
	;and we can't possibly open another file.
	 mov	dptr, #mesg_error
	 lcall	pstr_d
	mov	dpl, #255
	ret
fopen_ok:
	;now we know which file_info to use, r1 is the file
	;descriptor (index into the file_info array), and dptr
	;points to the file_info for this file

	mov	a, #1
	movx	@dptr, a	;mark the file as open
	clr	a
	mov	r3, #file_info_size - 1
fopen_clr_info:
	inc	dptr
	movx	@dptr, a	;fill the file_info with zeros
	djnz	r3, fopen_clr_info

	mov	a, r1
	lcall	fd_to_ptr
	mov	a, #12
	lcall	add_a_to_dptr
	mov	a, r2
	movx	@dptr, a	;store first_cluster_number
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	mov	a, r4
	movx	@dptr, a
	inc	dptr
	mov	a, r5
	movx	@dptr, a
	 mov	a, r1
	 lcall	pint8u_d
	 lcall	newline_d
	mov	dpl, r1
	ljmp	return_addr

















	;prepare to load some or all of a file into memory.
	;calling this function just sets up the file_info
	;entries, no data transfer happens and the drive
	;doesn't start if it's in sleep mode.  To actually
	;cause data transfer, call "file_cache_work"
	;repetitively until it has read all the data that
	;was requested (by calling here).
 
	;r0/r1/r2/r3 = 32 bit file position
	;r4/r5/r6/r7 = pointer to 32 bit length
	;b = file descriptor
file_cache:

	;very ugly hack... recycle "cluster" and "f32_sector"
	;to put the data from the eight registers into internal
	;memory.... so that it looks the way this code was
	;originally written to take the input.  The computation
	;code below should eventually be re-written to use the
	;registers directly without this ugly step.
	mov	cluster+0, r0
	mov	cluster+1, r1
	mov	cluster+2, r2
	mov	cluster+3, r3
	mov	f32_sector+0, r4
	mov	f32_sector+1, r5
	mov	f32_sector+2, r6
	mov	f32_sector+3, r7
	mov	r0, #cluster
	mov	r1, #f32_sector


	mov	file_desc, b
	 mov	dptr, #msg_cache1
	 lcall	pstr_d
	 mov	a, file_desc
	 lcall	pint8u_d
	 mov	dptr, #msg_cache2
	 lcall	pstr_d
	 lcall	phex32_at_r0
	 mov	dptr, #msg_to
	 lcall	pstr_d

	;first of all, we need to convert the program's
	;byte-based input into units of clusters.  The
	;first step is adding the beginning byte with the
	;length to get the last byte
	mov	a, @r0
	add	a, @r1
	mov	r2, a
	inc	r0
	inc	r1
	mov	a, @r0
	mov	cluster_offset+0, a
	addc	a, @r1
	mov	r5, a
	inc	r0
	inc	r1
	mov	a, @r0
	mov	cluster_offset+1, a
	addc	a, @r1
	mov	r6, a
	inc	r0
	inc	r1
	mov	a, @r0
	mov	cluster_offset+2, a
	addc	a, @r1
	mov	r7, a

	;decrement by one to get the actual last byte, not
	;the byte immediately after it (which wasn't requested)
	mov	a, r2
	add	a, #255
	 mov	r2, a
	mov	a, r5
	addc	a, #255
	mov	r5, a
	mov	a, r6
	addc	a, #255
	mov	r6, a
	mov	a, r7
	addc	a, #255
	mov	r7, a
	 mov	r0, #5
	 lcall	phex24_at_r0
	 mov	a, r2
	 lcall	phex_d
	 lcall	newline_d

	;now divide both numbers by the number of bytes in a cluster.
	;they've already been divided by 256 by discarding the lowest
	;byte

	mov	r2, sectors_per_cluster
fcache_offset_shift:
	clr	c
	mov	a, cluster_offset+2
	rrc	a
	mov	cluster_offset+2, a
	mov	a, cluster_offset+1
	rrc	a
	mov	cluster_offset+1, a
	mov	a, cluster_offset+0
	rrc	a
	mov	cluster_offset+0, a
	clr	c
	mov	a, r7
	rrc	a
	mov	r7, a
	mov	a, r6
	rrc	a
	mov	r6, a
	mov	a, r5
	rrc	a
	mov	r5, a
	mov	a, r2
	clr	c
	rrc	a
	mov	r2, a
	jnz	fcache_offset_shift

	;now we've got the range of blocks that'll be needed
	;in the cache to satisfy this request.  Write them
	;into the file_info for this file.

	 mov	dptr, #msg_req_clusters1
	 lcall	pstr_d
	 mov	r0, #cluster_offset
	 lcall	phex24_at_r0
	 mov	dptr, #msg_to
	 lcall	pstr_d
	 mov	r0, #5
	 lcall	phex24_at_r0
	 lcall	newline_d

	mov	a, file_desc
	lcall	fd_to_ptr
	movx	a, @dptr		;set "caching" bit in flags
	orl	a, #00000010b
	movx	@dptr, a
	inc	dptr
	inc	dptr			;skip unused byte

	mov	a, cluster_offset+0
	movx	@dptr, a		;store "fcache_first_cluster_offset"
	inc	dptr
	mov	a, cluster_offset+1
	movx	@dptr, a
	inc	dptr
	mov	a, cluster_offset+2
	movx	@dptr, a
	inc	dptr
	mov	a, r5
	movx	@dptr, a		;store "fcache_last_cluster_offset"
	inc	dptr
	mov	a, r6
	movx	@dptr, a
	inc	dptr
	mov	a, r7
	movx	@dptr, a
	inc	dptr

	;fetch the block number of the first cluster that's
	;already cached for this file (will be zero if none)

	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a

	;the goal now is the figure out "fcache_prev_cluster",
	;which should be the block number holding the last
	;already cached cluster that comes before
	;"fcache_first_cluster_offset".

	mov	r6, #0		;r6/r7 hold last block attempted
	mov	r7, #0
	mov	a, r3
	jnz	fcache_find_prev_loop
	mov	a, r2
	jz	fcache_prev_known

fcache_find_prev_loop:
	lcall	block_desc_addr		;get the block_info
	inc	dptr			;skip "next_block"
	inc	dptr
	inc	dptr			;skip flags
	inc	dptr			;skip unused byte
	clr	c
	movx	a, @dptr		;compare cluster offsets
	subb	a, cluster_offset+0
	inc	dptr
	movx	a, @dptr
	subb	a, cluster_offset+1
	inc	dptr
	movx	a, @dptr
	subb	a, cluster_offset+2
	jnc	fcache_prev_known	;if (file_info.cluster_offset >=
					;        cluster_offset)
	mov	a, r2
	mov	r6, a			;this block is before the first
	mov	a, r3			;cluster needed, so copy its
	mov	r7, a			;number to r6/r7
	inc	dptr
	inc	dptr
	movx	a, @dptr
	mov	r2, a			;read next cluster location
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	jnz	fcache_find_prev_loop
	mov	a, r2
	jnz	fcache_find_prev_loop

	;when we get here, r6/r7 will hold the block number
	;for the "fcache_prev_cluster" entry in file_info.
fcache_prev_known:
	 mov	dptr, #msg_prev_clust
	 lcall	pstr_d
	 mov	r0, #6
	 lcall	phex16_at_r0
	 lcall	newline_d

	mov	a, file_desc
	lcall	fd_to_ptr
	mov	a, #10
	lcall	add_a_to_dptr
	mov	a, r6
	movx	@dptr, a		;store "fcache_prev_cluster"
	inc	dptr
	mov	a, r7
	movx	@dptr, a

	;hopefully everything that will be needed to get this
	;file properly cached has been set up.

	ljmp	return_addr






























	;this function actually does the work of loading a file
	;into the cache.  Calling here will cause IDE read
	;requests to be generated, which will make the drive
	;start spinning if it was in sleep mode.  All that ius
	;needed is the file descriptor (in r4).  The specifics
	;of what is to be loaded must have been set up by calling
	;"file_cache" before calling here.  Usually, this function
	;will need to be called repetitively until it returns a
	;status that the loading is complete, or an error (out
	;of memory)
	; return values:
	;	DPL=0		some cached, keep calling
	;	DPL=1		all requested data is cached
	;	DPL=2		ran out of memory


file_cache_work:
	 mov	dptr, #msg_fcw1
	 lcall	pstr_d
	 mov	a, r4
	 lcall	phex_d
	 mov	a, #','
	 lcall	cout_d
	mov	a, r4
	mov	file_desc, a


	;check that the low priority request queue has enough space
	;to add enough requests for an entire cluster, and if it
	;doesn't, we need to stop now before malloc'ing the memory
	;to hold this cluster.  This check was moved to be the very
	;first thing, because in the common case where the main
	;program called file_cache_work repetitively, the rreq queue
	;is rapidly filled, so most of the calls will be returned
	;right away by this function until there's at least enough
	;room in the low priority rreq queue to hold the requests
	;for another cluster

fcache_queue_check:
	mov	r0, blocks_per_cluster
	inc	r0			;actually, make sure there's
					;always one extra space available
					;(ide_sleep doesn't properly handle
					;a full queue case, yet....)
	;- disable ide interrupt here
	mov	r4, clust_rreq_head
	mov	r5, clust_rreq_tail
	;- reenable ide interrupt here
	mov	a, r4
	cpl	a
	jz	fcache_qck_ok		;ok if queue complete empty
	cpl	a
fcache_qck_1:
	inc	a
	cjne	a, #clust_rreq_size, fcache_qck_2
	clr	a
fcache_qck_2:
	cjne	a, 5, fcache_qck_nxt
	;if we get here, there won't be enough room in the queue
	 mov	dptr, #msg_queue_full
	 lcall	pstr_d
	mov	dpl, #0
	ljmp	return_addr

fcache_qck_nxt:
	djnz	r0, fcache_qck_1
fcache_qck_ok:


	;the first thing we need to do is find a cluster number
	;and it's cluster offset, that's at or before the first
	;cluster offset required to be read.  There are two
	;cases, either "file_info[fd].fcache_prev_cluster" will
	;give us a block number where we get this, or if that
	;fails, we've got to start at the beginning of the file.

	mov	a, file_desc
	lcall	fd_to_ptr
	movx	a, @dptr		;read flags
	jb	acc.1, fcache_start
	;if we get here, this file isn't currently "caching", so
	;return now with an indication that there's nothing to do
	mov	dpl, #1
	ljmp	return_addr

fcache_start:
	mov	a, #10
	lcall	add_a_to_dptr
	movx	a, @dptr
	mov	prev_cluster_at_block+0, a
	inc	dptr
	movx	a, @dptr
	mov	prev_cluster_at_block+1, a
	jnz	fcache_prev
	mov	a, prev_cluster_at_block+0
	jnz	fcache_prev

	;the file_info didn't have any indication of a previous
	;cached cluster, so we'll just start at the beginning
fcache_no_prev:
	inc	dptr
	movx	a, @dptr
	mov	cluster+0, a	;get file_info.first_cluster_number
	inc	dptr
	movx	a, @dptr
	mov	cluster+1, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+2, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+3, a
	clr	a
	mov	cluster_offset+0, a	;cluster_offset is zero
	mov	cluster_offset+1, a
	mov	cluster_offset+2, a
	mov	a, file_desc
	lcall	fd_to_ptr
	mov	a, #8
	lcall	add_a_to_dptr
	movx	a, @dptr
	mov	next_cluster_at_block+0, a
	inc	dptr
	movx	a, @dptr
	mov	next_cluster_at_block+1, a
	 mov	dptr, #msg_fcw2
	 lcall	pstr_d
	 mov	dptr, #msg_fcw3
	 lcall	pstr_d
	 lcall	newline_d
	ljmp	fcache_seek_cluster

	;the file_info had a previous cluster block number, so
	;we'll look it up to get the info we need.
fcache_prev:
	 mov	dptr, #msg_fcw3
	 lcall	pstr_d
	 mov	a, #'='
	 lcall	cout_d
	 mov	r0, #prev_cluster_at_block
	 lcall	phex16_at_r0
	 lcall	newline_d
	mov	r2, prev_cluster_at_block+0
	mov	r3, prev_cluster_at_block+1
	lcall	block_desc_addr		;get the block_info
	inc	dptr			;skip "next_block"
	inc	dptr
	inc	dptr			;skip flags
	inc	dptr			;skip unused byte
	movx	a, @dptr
	mov	cluster_offset+0, a
	inc	dptr
	movx	a, @dptr
	mov	cluster_offset+1, a
	inc	dptr
	movx	a, @dptr
	mov	cluster_offset+2, a
	inc	dptr
	inc	dptr			;skip "unused" byte
	movx	a, @dptr
	mov	next_cluster_at_block+0, a
	inc	dptr
	movx	a, @dptr
	mov	next_cluster_at_block+1, a
	inc	dptr
	inc	dptr			;skip "number_of_bytes"
	inc	dptr
	movx	a, @dptr
	mov	cluster+0, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+1, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+2, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+3, a

fcache_seek_cluster:
	;we need to traverse the cluster chain until "cluster_offset"
	;is equal to "file_info[fd].fcache_first_cluster_offset".
	 mov	dptr, #msg_fcseek1
	 lcall	pstr_d
	 mov	r0, #cluster
	 lcall	phex32_at_r0
	 mov	dptr, #msg_fcseek2
	 lcall	pstr_d
	 mov	r0, #cluster_offset
	 lcall	phex24_at_r0
	 mov	dptr, #msg_fcseek3
	 lcall	pstr_d
	 mov	a, file_desc
	 lcall	fd_to_ptr
	 inc	dptr
	 inc	dptr
	 movx	a, @dptr
	 push	acc
	 inc	dptr
	 movx	a, @dptr
	 push	acc
	 inc	dptr
	 movx	a, @dptr
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 pop	acc
	 lcall	phex_d
	 lcall	newline_d

	;check if the next cluster number is really an end of file mark.
	mov	a, cluster+3
	cjne	a, #0x0F, fcache_seek2

	;Found EOF marker, which means we've read all of the file that
	;there was to read.  The main program must have requested more
	;than was actually available if we get here.

	;technically, we ought to check for 0x0FFFFFF8 to 0x0FFFFFFF
	;but no drive could have a cluster number that large.... using
	;512 byte clusters (very unlikely to have a small cluster with
	;such a large drive) would need to have 120 gig of clusters.
	;It's probably safe to just check the upper byte and assume
	;it's an end of file (or bad sector), and we need to stop.
	;If we get here, the file has been completely read and we're done.

	;does any cleanup need to be done?

	mov	a, file_desc
	lcall	fd_to_ptr
	movx	a, @dptr
	anl	a, #11111101b		;clear "caching" bit in flags
	movx	@dptr, a
	mov	dpl, #1
	ljmp	return_addr



fcache_seek2:
	mov	a, file_desc
	lcall	fd_to_ptr
	inc	dptr
	inc	dptr
	mov	r6, dpl		;keep pointer to "fcache_first_cluster_offset"
	mov	r7, dph
	movx	a, @dptr	;get "fcache_first_cluster_offset"
	cjne	a, cluster_offset+0, fcache_seek_fwd
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+1, fcache_seek_fwd
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+2, fcache_seek_fwd
	ajmp	fcache_read_cluster
fcache_seek_fwd:
	;the comparison above only did an equality test of
	;"cluster_offset" to the target offset stored in
	;file_info[fd].fcache_first_cluster_offset.  We get here when
	;they were not equal.  The values of "cluster_offset" will
	;usually be a relatively small number (5 meg file with 4k
	;clusters is 1221 clusters, relative to a 24 bit integer),
	;and it should always be less than the target.  If something
	;goes horribly wrong (memory corruption via DMA), it's quite
	;likely to make its first appearance as "cluster_offset"
	;being some unrealisticly large number, that's bigger than
	;the target.  A test should be done here to see if the
	;cluster_offset is larger than the target... and if it is
	;something went very wrong and we probably need to abort
	;back to paulmon2 right now, but maybe print a dump
	;of the first 32 blocks (all the block_info) and perhaps
	;8000 to 8FFF.
	mov	dpl, r6		;dptr points to "target"
	mov	dph, r7
	clr	c
	movx	a, @dptr
	inc	dptr
	subb	a, cluster_offset+0	;compute target - cluster_offset
	movx	a, @dptr
	inc	dptr
	subb	a, cluster_offset+1	;compute target - cluster_offset
	movx	a, @dptr
	inc	dptr
	subb	a, cluster_offset+2	;compute target - cluster_offset
	jnc	fcache_seek_ok
	;if we get here, cluster_offset was greater than the target
	;(which was file_info[fd].fcache_first_cluster_offset), which
	;makes something has gone horribly wrong.

	mov	dptr, #msg_coffset_error
	lcall	pstr
	ljmp	serious_error

fcache_seek_ok:
	mov	a, cluster_offset+0
	add	a, #1
	mov	cluster_offset+0, a
	mov	a, cluster_offset+1
	addc	a, #0
	mov	cluster_offset+1, a
	mov	a, cluster_offset+2
	addc	a, #0
	mov	cluster_offset+2, a

	lcall	fat32_next_cluster
	cjne	a, #1, fcache_seek_nope
	ajmp	fcache_seek_cluster

fcache_seek_nope:
	;fat32_next_cluster returned either 0 (try again later) or
	;2 (out of memory),
	cjne	a, #2, fcache_seek_nope2
	sjmp	fcache_out_of_memory
fcache_seek_nope2:
	mov	dpl, a
	ljmp	return_addr

msg_coffset_error:
	.db	13,10,"Error: Cluster offset > target cluster offset",13,10,0



	;when we get here, we've found a cluster that needs to
	;be read.  We get both the "cluster" and "cluster_offset",
	;and "prev_cluster_at_block" gives us the block that's
	;holding the previous cluster, so we can link it to this
	;one (or zero if none, so we'll have to link to the file_info).
fcache_read_cluster:

	 mov	dptr, #msg_read_clust1
	 lcall	pstr_d
	 mov	r0, #cluster
	 lcall	phex32_at_r0
	 mov	dptr, #msg_read_clust2
	 lcall	pstr_d
	 mov	r0, #cluster_offset
	 lcall	phex24_at_r0
	 mov	a, #')'
	 lcall	cout_d
	 lcall	newline_d

	;TODO:  (this bug will show up someday....)
	;actually, it's possible that this cluster is already
	;in the cache, sitting as the next cluster in the linked
	;list... it's be pretty silly to read it if it's already
	;in the list.  Need to check for this case and jump to
	;fcache_finishup if we don't actually need to read it.



	;now call malloc_blocks to get the memory for this cluster
	;this is the place that actually allocates the vast majority
	;of memory used by the mp3 player.

	mov	a, blocks_per_cluster
	lcall	malloc_blocks
	mov	reg6, r2			;save block # in r6/r7
	mov	reg7, r3
	jnc	fcache_rdclust2

fcache_out_of_memory:
	;if we get here, we're out of memory.
	 mov	dptr, #msg_no_mem
	 lcall	pstr

	mov	a, file_desc
	lcall	fd_to_ptr
	movx	a, @dptr
	anl	a, #11111101b		;clear "caching" bit in flags
	movx	@dptr, a
	mov	dpl, #2
	ljmp	return_addr


fcache_rdclust2:
	;convert the cluster number into an LBA sector number

	clr	c
	mov	a, cluster+0	;first, subtract 2
	subb	a, #2
	mov	f32_sector+0, a
	mov	a, cluster+1
	subb	a, #0
	mov	f32_sector+1, a
	mov	a, cluster+2
	subb	a, #0
	mov	f32_sector+2, a
	mov	a, cluster+3
	anl	a, #15
	subb	a, #0
	mov	f32_sector+3, a
				;now multiply by sectors_per_cluster
				;always power of two, so use shifts
	mov	r0, sectors_per_cluster
f32_c2l_1:
	mov	a, r0
	clr	c
	rrc	a
	mov	r0, a
	jz	f32_c2l_2
	clr	c
	mov	a, f32_sector+0
	rlc	a
	mov	f32_sector+0, a
	mov	a, f32_sector+1
	rlc	a
	mov	f32_sector+1, a
	mov	a, f32_sector+2
	rlc	a
	mov	f32_sector+2, a
	mov	a, f32_sector+3
	rlc	a
	mov	f32_sector+3, a
	sjmp	f32_c2l_1
f32_c2l_2:
	;now just add the LBA of the first data sector
	mov	a, f32_sector+0
	add	a, first_data_sector+0
	mov	f32_sector+0, a
	mov	a, f32_sector+1
	addc	a, first_data_sector+1
	mov	f32_sector+1, a
	mov	a, f32_sector+2
	addc	a, first_data_sector+2
	mov	f32_sector+2, a
	mov	a, f32_sector+3
	addc	a, first_data_sector+3
	mov	f32_sector+3, a		;now "f32_sector" has the LBA to read


	;we need to do something with "prev_cluster_at_block".
	;if it's zero, then there is no previous cluster and
	;we need to update the file_info.  If there is a previous
	;block, then we just write this number into its
	;block_info.next_cluster field.

	mov	a, prev_cluster_at_block+0
	jnz	fcache_prev_fixup
	mov	a, prev_cluster_at_block+1
	jnz	fcache_prev_fixup

	mov	a, file_desc
	lcall	fd_to_ptr
	mov	a, #8
	lcall	add_a_to_dptr
	mov	a, r6		;store r6/r7 as file_info.first_cached_cluster
	movx	@dptr, a
	inc	dptr
	mov	a, r7
	movx	@dptr, a
	sjmp	fcache_rdc_rreq
fcache_prev_fixup:
	mov	r2, prev_cluster_at_block+0
	mov	r3, prev_cluster_at_block+1
fcache_prev_fixup_loop:
	lcall	block_desc_addr
	mov	a, #8
	lcall	add_a_to_dptr
	mov	a, r6
	movx	@dptr, a	;store r6/r7 as block_info.next_cluster
	inc	dptr
	mov	a, r7
	movx	@dptr, a
	lcall	next_block
	mov	a, r2		;write it into all blocks of that cluster
	jnz	fcache_prev_fixup_loop
	mov	a, r3
	jnz	fcache_prev_fixup_loop




	;now we'll write the block_info onto each block that will
	;hold this cluster, and create low priority read requests
	;for each one.  Sometime ;in the future the IDE driver will
	;actually execute them and read this cluster into memory.
fcache_rdc_rreq:
	mov	r2, 6			;get first block that we malloc'd
	mov	r3, 7
	mov	r1, blocks_per_cluster
fcache_rdc_rreq1:
	 mov	dptr, #msg_rdc_info1
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 mov	dptr, #msg_rdc_info2
	 lcall	pstr_d
	 mov	r0, #cluster
	 lcall	phex32_at_r0
	 mov	dptr, #msg_rdc_info3
	 lcall	pstr_d
	 mov	r0, #cluster_offset
	 lcall	phex24_at_r0
	 mov	dptr, #msg_rdc_info4
	 lcall	pstr_d
	 mov	r0, #next_cluster_at_block
	 lcall	phex16_at_r0
	 lcall	newline_d

	lcall	block_desc_addr		;need to update the block_info
	inc	dptr			;skip "next_block" (that's malloc's)
	inc	dptr
	mov	a, #0x83		;flags = cluster, read pending
	movx	@dptr, a
	inc	dptr
	inc	dptr			;skip "unused" byte
	mov	a, cluster_offset+0
	movx	@dptr, a		;store "cluster_offset"
	inc	dptr
	mov	a, cluster_offset+1
	movx	@dptr, a		;store "cluster_offset"
	inc	dptr
	mov	a, cluster_offset+2
	movx	@dptr, a		;store "cluster_offset"
	inc	dptr
	clr	a
	movx	@dptr, a		;store "cluster_offset"
	inc	dptr
	mov	a, next_cluster_at_block+0
	movx	@dptr, a		;store "next_cluster"
	inc	dptr
	mov	a, next_cluster_at_block+1
	movx	@dptr, a		;store "next_cluster"
	inc	dptr
	clr	a			;hardcode bytes to 4096 (fix later)
	movx	@dptr, a		;store "number_of_bytes"
	inc	dptr
	mov	a, #0x10		;hardcode bytes to 4096 (fix later)
	movx	@dptr, a		;store "number_of_bytes"
	inc	dptr
	mov	a, cluster+0
	movx	@dptr, a		;store "cluster"
	inc	dptr
	mov	a, cluster+1
	movx	@dptr, a		;store "cluster"
	inc	dptr
	mov	a, cluster+2
	movx	@dptr, a		;store "cluster"
	inc	dptr
	mov	a, cluster+3
	movx	@dptr, a		;store "cluster"




	;if the IDE code were interrupt driven, we'd probably be
	;shutting off its interrupt somewhere near here... but
	;it's safe to do all that block_info crap with the interrupts
	;on, as long as the block isn't referenced by a queue'd request

	mov	a, clust_rreq_head
	cjne	a, #255, fcache_rdc_rreq2
	;if we get here, there were no fat sector read requests
	;so the queue is marked empty and we need to initialize
	;it.  When this someday is interrupt driven, this code
	;will need to set a flag that causes the first interrupt
	clr	a
	mov	clust_rreq_head, a
	mov	clust_rreq_tail, a
fcache_rdc_rreq2:
	inc	a
	cjne	a, #clust_rreq_size, fcache_rdc_rreq3
	clr	a
fcache_rdc_rreq3:
	;no need to check if this one fills the queue, since it
	;was checked before hand that the entire group would fit
	mov	clust_rreq_head, a

	;now turn the index into a 16 pointer into the read_request array
	mov	b, #8	;sizeof(struct read_request) == 8
	mul	ab
	add	a, #clust_read_req & 255
	mov	dpl, a
	mov	a, b
	addc	a, #clust_read_req >> 8
	mov	dph, a

	;now dptr points to the new read_request struct, so let's fill it
	mov	a, f32_sector+0
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+1
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, r2
	movx	@dptr, a	;write block to store the sectors
	inc	dptr
	mov	a, r3
	movx	@dptr, a	;write block to store the sectors
	inc	dptr
	clr	a
	movx	@dptr, a	;fill unused bytes with zero
	inc	dptr
	movx	@dptr, a	;fill unused bytes with zero
	
	;in a future world of interrupt drive IDE device drivers,
	;this is probably where it'd be save to reenable interrupts

	dec	r1
	mov	a, r1
	jz	fcache_finishup

	;advance to the next block needed for this cluster
	lcall	next_block	;get the next block we'll use
	mov	a, f32_sector+0
	add	a, #8		;advance 8 sectors (4k)
	mov	f32_sector+0, a
	mov	a, f32_sector+1
	addc	a, #0
	mov	f32_sector+1, a
	mov	a, f32_sector+2
	addc	a, #0
	mov	f32_sector+2, a
	mov	a, f32_sector+3
	addc	a, #0
	mov	f32_sector+3, a
	ljmp	fcache_rdc_rreq1



	;now the read requests are pending (maybe even getting
	;executed right now if the IDE driver is interrupt driven).
	;before we can return to the main program, there's just
	;a couple details left to clean up in file_info.  The
	;"fcache_first_cluster_offset" field needs to be incremented,
	;but if it's equal to "fcache_last_cluster_offset" then we
	;just did the last required cluster, so just return with
	;a completed status code.  If there is still more to do,
	;we'be better update "fcache_prev_cluster" to be the one
	;we just read, so that the next call can start up where
	;we just left off.

fcache_finishup:
	mov	a, file_desc
	lcall	fd_to_ptr
	inc	dptr		;skip flags
	inc	dptr		;skip unused byte
	inc	dptr		;skip "fcache_first_cluster_offset"
	inc	dptr
	inc	dptr
	movx	a, @dptr	;get file_info.fcache_last_cluster_offset
	inc	dptr		;and compare with "cluster_offset"
	cjne	a, cluster_offset+0, fcache_finishup2
	movx	a, @dptr
	cjne	a, cluster_offset+1, fcache_finishup2
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+2, fcache_finishup2

	;if we get here, this cluster we just read was the last
	;one needed to complete the cache request!

	mov	a, file_desc
	lcall	fd_to_ptr
	movx	a, @dptr
	anl	a, #11111101b		;clear "caching" bit in flags
	movx	@dptr, a
	mov	dpl, #1
	ljmp	return_addr


fcache_finishup2:
	;this wasn't the last cluster needed, so let's update
	;fcache_first_cluster_offset and fcache_prev_cluster
	mov	a, cluster_offset+0
	add	a, #1			;increment cluster_offset
	mov	cluster_offset+0, a
	mov	a, cluster_offset+1
	addc	a, #0
	mov	cluster_offset+1, a
	mov	a, cluster_offset+2
	addc	a, #0
	mov	cluster_offset+2, a
	mov	a, file_desc
	lcall	fd_to_ptr
	inc	dptr		;skip flags
	inc	dptr		;skip unused byte
	mov	a, cluster_offset+0
	movx	@dptr, a	;store fcache_first_cluster_offset
	inc	dptr
	mov	a, cluster_offset+1
	movx	@dptr, a
	inc	dptr
	mov	a, cluster_offset+2
	movx	@dptr, a
	inc	dptr
	inc	dptr		;skip fcache_last_cluster_offset
	inc	dptr
	inc	dptr
	inc	dptr		;skip first_cached_cluster
	inc	dptr
	mov	a, r6
	movx	@dptr, a	;store fcache_prev_cluster
	inc	dptr
	mov	a, r7
	movx	@dptr, a

	mov	dpl, #0
	ljmp	return_addr










































































	;lookup the next cluster of a file, given the current
	;"cluster".  Despite the word "cluster" in the title,
	;the process of finding the next cluster of a file
	;(knowing the current one) involves reading the FAT
	;sectors.  If the FAT sectors with the
	;info are available in memory, the next cluser is
	;returned in "cluster".  Acc gives the return status:

	;  Acc = 0	issued a read request, try again later
	;  Acc = 1	next cluster found, retuned in "cluster"
	;  Acc = 2	ran out of memory


fat32_next_cluster:
	 mov	dptr, #msg_nxt_clust1
	 lcall	pstr_d
	 mov	r0, #cluster
	 lcall	phex32_at_r0
	 lcall	newline_d

	;convert cluster number to a FAT sector number (div by 128)
	mov	a, cluster+1
	rlc	a
	anl	a, #11111000b	;align to a 4k group of sectors
	mov	f32_sector+0, a
	mov	a, cluster+2
	rlc	a
	mov	f32_sector+1, a
	mov	a, cluster+3
	rlc	a
	mov	f32_sector+2, a
	clr	a
	rlc	a
	mov	f32_sector+3, a

	;This is a little optimization... remember the
	;FAT sector offset and block number of the last FAT sector
	;group we used, and in the common case where we need it
	;again, we can skip the binary search, load r2/r3 with
	;the block number, and jump right to "fat32_next_cluster_finish"
	;this optimization will need 6 bytes of ram

	mov	dptr, #last_fat_sector
	movx	a, @dptr
	cjne	a, f32_sector+0, fat32_next_clust_need
	inc	dptr
	movx	a, @dptr
	cjne	a, f32_sector+1, fat32_next_clust_need
	inc	dptr
	movx	a, @dptr
	cjne	a, f32_sector+2, fat32_next_clust_need
	inc	dptr
	movx	a, @dptr
	cjne	a, f32_sector+3, fat32_next_clust_need
	mov	dptr, #last_fat_fd
	movx	a, @dptr
	cjne	a, file_desc, fat32_next_clust_need

	;if we get here, it means the requested FAT sector is in the
	;same group we previously used, and it's needed for the same
	;file (file_usage_mask bit already set properly), so we can
	;skip all this time-consuming work and just use it.
	
	mov	dptr, #last_fat_block
	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	ljmp	fat32_next_cluster_finish


fat32_next_clust_need:
	 mov	dptr, #msg_nxt_clust2
	 lcall	pstr_d
	 mov	r0, #f32_sector
	 lcall	phex32_at_r0
	 lcall	newline_d

	lcall	bin_search
	jnc	f32_next_clust_avail
	ljmp	f32_next_clust_na

f32_next_clust_avail:
	;when we get here, the 4k block we need is cached and
	;r2/r3 is holding the index in the memory tracker array
	;where we can get the block number

	 mov	dptr, #msg_nxt_clust_f1
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0

	clr	c
	mov	a, r2
	rlc	a
	mov	dpl, a
	mov	a, r3
	rlc	a
	add	a, #0xD0
	mov	dph, a		;now dptr points to block number in array

	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a		;r2/r3 now has block number w/ sectors

	 mov	dptr, #msg_nxt_clust_f2
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 lcall	newline_d

	;must read it's block_info flags and make sure it's actually
	;got the data, not just a pending xfer.  If the xfer is pending,
	;we need to return C=1 and later we'll get called again and
	;maybe the data will have been read by then.

	lcall	block_desc_addr
	inc	dptr
	inc	dptr
	movx	a, @dptr	;get "flags" from the block_info
	jnb	acc.7, fat32_next_cluster_ok

	;if we get here, the transfer is still pending, so this
	;block hasn't been filled with the data yet.  All we can
	;do is wait until it's ready
	clr	a
	ret

fat32_next_cluster_ok:
	;dptr still pointing to "flags" in the block_info
	;need to mark it's block_info usage bits that this 4k block
	;is needed by the file we're reading.
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr		;advance to beginning of "file_usage_mask"
	mov	a, file_desc
	anl	a, #00111000b
	swap	a
	rl	a
	lcall	add_a_to_dptr	;inc dptr to correct byte in "file_usage_mask"
	mov	a, file_desc
	anl	a, #00000111b
	add	a, #2
	movc	a, @a+pc	;grab mask byte to OR into "file_usage_mask"
	sjmp	fat32_nc_ok3
	.db	1, 2, 4, 8, 16, 32, 64, 128
fat32_nc_ok3:
	mov	r1, a
	movx	a, @dptr	;read file_usage_mask
	orl	a, r1
	movx	@dptr, a	;this FAT sector is in use by file_desc

	;update "last_fat_sector", "last_fat_block", and "last_fat_fd"
	mov	dptr, #last_fat_sector
	mov	a, f32_sector+0
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+1
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a
	mov	dptr, #last_fat_block
	mov	a, r2
	movx	@dptr, a
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	mov	dptr, #last_fat_fd
	mov	a, file_desc
	movx	@dptr, a

fat32_next_cluster_finish:
	mov	a, #0xB
	lcall	map_block	;the 8 sectors are now at 0xB000
	clr	c
	mov	a, cluster+0	;grab lower 10 bits of cluster number
	rlc	a		;mult by 4 (clusters are 32 bit numbers)
	mov	r2, a
	mov	a, cluster+1
	anl	a, #3
	rlc	a
	mov	r3, a
	clr	c
	mov	a, r2
	rlc	a
	mov	dpl, a
	mov	a, r3
	rlc	a
	add	a, #0xB0
	mov	dph, a		;now dptr points to cluster number
	movx	a, @dptr
	mov	cluster+0, a	;read next cluster into "cluster"
	inc	dptr
	movx	a, @dptr
	mov	cluster+1, a
	inc	dptr
	movx	a, @dptr
	mov	cluster+2, a
	inc	dptr
	movx	a, @dptr
	anl	a, #00001111b	;FAT entries are really only 28 bits
	mov	cluster+3, a

	mov	a, #1
	ret







f32_next_clust_na:
	.equ	new_fat_block, bsearch_tmp	;reuse this 16 bit variable

	;if we get here, the FAT sectors with the info we need are
	;not cached.... we need to add them.  r2/r3 points to the
	;place in the array where this will be added... need to
	;move the array around, add the block, and create a
	;read_request to fill with the sectors.

	 mov	dptr, #msg_nxt_clust_nf
	 lcall	pstr_d

	;there should be a check here to see if we'll be able
	;to add a read_request.... if there's no room in the
	;read_request queue, we need to abort here before it's
	;too late (lotsa data structures changed)
f32_nc_na_rreq_ck:
	;disable ide interrupt here
	mov	r0, fat_rreq_head
	mov	r1, fat_rreq_tail
	;re-enaable ide interrupt here
	mov	a, r0
	cpl	a
	jz	f32_nc_na_rreq_ok
	inc	r0
	cjne	r0, #fat_rreq_size, f32_nc_na_rreq_ck2
	mov	r0, #0
f32_nc_na_rreq_ck2:
	mov	a, r1
	cjne	a, 0, f32_nc_na_rreq_ok
	;if we get here, the fat (high priority) read request
	;queue is full, so we better not try to make another
	;fat sector read until there is room in the queue
	clr	a
	ret
f32_nc_na_rreq_ok:


	;check the number of blocks already stored.  We can only
	;have 2048 4k groups of FAT sectors, because the array that
	;holds their block numbers is 4k.  If the array is full,
	;return an out of memory.  It's very unlikely we'll ever
	;hit this limit, but it we do it means a crash if not checked

	mov	dptr, #mm_num_tracked
	movx	a, @dptr
	mov	r5, a		;r5/r6 will hold number of entries in array
	add	a, #1
	inc	dptr
	movx	a, @dptr
	mov	r6, a
	addc	a, #0
	anl	a, #11111000b
	jz	f32_nc_na_add_ok
	;2048 4k groups of FAT sectors are cached and we can't
	;hold any more, so even if there is more memory to malloc,
	;we can't use it.
	mov	a, #2
	ret
f32_nc_na_add_ok:


	;allocate a block that will later be filled with the
	;sectors we needed but didn't have.
	push	reg2
	push	reg3
	push	reg5
	push	reg6
	mov	a, #1
	lcall	malloc_blocks	;get a block that'll eventually hold sectors
	mov	new_fat_block+0, r2
	mov	new_fat_block+1, r3
	pop	reg6
	pop	reg5
	pop	reg3
	pop	reg2
	jnc	f32_nc_na2

	;if malloc_blocks returns C=1, we're out of memory
	mov	a, #2
	ret


f32_nc_na2:

	 mov	dptr, #msg_num_tracked
	 lcall	pstr_d
	 mov	r0, #5
	 lcall	phex16_at_r0
	 lcall	newline_d
	 mov	dptr, #msg_insert_at
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 lcall	newline_d
	 mov	dptr, #msg_mov_array
	 lcall	pstr_d

	clr	c
	mov	a, r5
	rlc	a
	mov	dpl, a		;point DPTR to the end of the array
	mov	a, r6
	rlc	a
	add	a, #0xD0
	mov	dph, a
	mov	a, dpl
	add	a, #254
	mov	dpl, a
	mov	a, dph
	addc	a, #255
	mov	dph, a
	
	clr	c
	mov	a, r5		;subtract index from number of blocks
	subb	a, r2		;to get the number we need to move
	mov	r5, a
	mov	a, r6
	subb	a, r3
	mov	r6, a
f32_fsect_add_loop:
	;this loop should someday be implemented as a fast DMA-based memcpy.
	;since the destination overlaps the source, we'd malloc a new block
	;and do a pair of memcpu operations... still very fast using the
	;DMA channel (when it's someday implemented)
	mov	a, r5		;check if number of blocks left to move is zero
	jnz	f32_fsect_add_loop2
	mov	a, r6
	jz	f32_fsect_add_ok
f32_fsect_add_loop2:
	 mov	a, #'.'
	 lcall	cout_d
	movx	a, @dptr	;read the values from the table
	mov	r0, a
	inc	dptr
	movx	a, @dptr
	mov	r1, a
	inc	dptr
	mov	a, r0
	movx	@dptr, a	;and write them 2 bytes up
	inc	dptr
	mov	a, r1
	movx	@dptr, a
	mov	a, dpl
	add	a, #251		;adjust dptr back down
	mov	dpl, a
	mov	a, dph
	addc	a, #255
	mov	dph, a
	mov	a, r5		;decrement the count of how many left to move
	add	a, #255
	mov	r5, a
	mov	a, r6
	addc	a, #255
	mov	r6, a
	sjmp	f32_fsect_add_loop
f32_fsect_add_ok:
	 lcall	newline_d
	;now we've moved the other entries of the table up to make 
	;a hole to insert the new block that'll hold the sectors
	;we are needing to have.  First we make a pointer to the hole
	clr	c
	mov	a, r2
	rlc	a
	mov	dpl, a
	mov	a, r3
	rlc	a
	add	a, #0xD0
	mov	dph, a

	;write the block address into the table
	mov	a, new_fat_block+0
	movx	@dptr, a	;write the block number into the table
	inc	dptr
	mov	a, new_fat_block+1
	movx	@dptr, a

	;increment the count of blocks holding FAT sectors
	mov	dptr, #mm_num_tracked
	movx	a, @dptr
	add	a, #1
	movx	@dptr, a
	inc	dptr
	movx	a, @dptr
	addc	a, #0
	movx	@dptr, a

	;now let's edit this block's block_info to reflect what it is
	mov	r2, new_fat_block+0
	mov	r3, new_fat_block+1
	lcall	block_desc_addr
	inc	dptr		;don't touch "next_block", that's malloc's
	inc	dptr		;it should be zero, since we malloc'd 1 block
	mov	a, #0x82
	movx	@dptr, a	;set flags: fat sector, read pending
	inc	dptr
	clr	a
	movx	@dptr, a	;unused byte = 0
	inc	dptr
	mov	a, f32_sector+0
	movx	@dptr, a	;store the sector offset of the FAT sectors
	inc	dptr		;that this block will soon be holding
	mov	a, f32_sector+1
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a
	inc	dptr
	clr	a
	movx	@dptr, a	;no files are depending on these sectors
	inc	dptr		;yet, so mark all 8 bytes as zeros
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a

	;we're almost done... finally, it's time to create a
	;read_request which will cause these sectors to be read.
	;we've already got the destination block, so all we need
	;to do is compute the LBA for this fat32 sector offset

	mov	a, f32_sector+0
	add	a, reserved_sector_count+0
	mov	f32_sector+0, a
	mov	a, f32_sector+1
	addc	a, reserved_sector_count+1
	mov	f32_sector+1, a
	mov	a, f32_sector+2
	addc	a, #0
	mov	f32_sector+2, a
	mov	a, f32_sector+3
	addc	a, #0
	mov	f32_sector+3, a

	mov	a, f32_sector+0
	add	a, partition_first_sector+0
	mov	f32_sector+0, a
	mov	a, f32_sector+1
	addc	a, partition_first_sector+1
	mov	f32_sector+1, a
	mov	a, f32_sector+2
	addc	a, partition_first_sector+2
	mov	f32_sector+2, a
	mov	a, f32_sector+3
	addc	a, partition_first_sector+3
	mov	f32_sector+3, a

	;now we've got everything we need to fill in a read
	;request, so let's try and grab one.

	; - disable ide interrupt here
	mov	a, fat_rreq_head
	cjne	a, #255, f32_fsect_rreq2
	;if we get here, there were no fat sector read requests
	;so the queue is marked empty and we need to initialize
	;it.  When this someday is interrupt driven, this code
	;will need to set a flag that causes the first interrupt
	clr	a
	mov	fat_rreq_head, a
	mov	fat_rreq_tail, a
f32_fsect_rreq2:
	inc	a
	cjne	a, #fat_rreq_size, f32_fsect_rreq3
	clr	a
f32_fsect_rreq3:
	cjne	a, fat_rreq_tail, f32_fsect_rreq4
	;if we get here, the fat sector read request queue is
	;full, so we can't issue the request now.  If this
	;happens, we're fucked, since we already shifted the
	;table around, so sooner or later we will crash.  This
	;really can't happen, because of the check above to
	;make sure there is some space avilable.
	; - re-enable ide interrupt here
	clr	a
	ret

f32_fsect_rreq4:
	mov	fat_rreq_head, a
	;now turn the index into a 16 pointer into the read_request
	;array.  This code assumes that the array size <= 256 bytes
	swap	a
	rr	a	;sizeof(struct read_request) == 8
	add	a, #fat_read_req & 255
	mov	dpl, a
	clr	a
	addc	a, #fat_read_req >> 8
	mov	dph, a
	;now dptr points to the new read_request struct, so let's fill it
	mov	a, f32_sector+0
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+1
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a	;write LBA where to read sectors
	inc	dptr
	mov	a, new_fat_block+0
	movx	@dptr, a	;write block to store the sectors
	inc	dptr
	mov	a, new_fat_block+1
	movx	@dptr, a	;write block to store the sectors
	inc	dptr
	clr	a
	movx	@dptr, a	;fill unused bytes with zero
	inc	dptr
	movx	@dptr, a	;fill unused bytes with zero

	;when the interrupt code is written, it'd be a good idea to
	;fill in the block_info first, then manipulate the read request
	;queue, so interrupts wouldn't be shut off for so long.
	; - re-enable ide interrupt here

	;acall	print_fat_sector_summary

	;that's it... we can't do anymore work until this read_request
	;is satisified, which is the job of the IDE interface code.
	clr	a
	ret


















print_fat_sector_summary:
	jb	debug, pfss1
	ret
pfss1:	mov	dptr, #mm_num_tracked
	movx	a, @dptr
	mov	r5, a		;get number of sector groups in array
	inc	dptr
	movx	a, @dptr
	mov	r6, a

	mov	dptr, #0xD000	;beginning of the fat sector array
pfss_loop:
	mov	a, r5
	jnz	pfss_cont
	mov	a, r6
	jz	pfss_done
pfss_cont:
	movx	a, @dptr
	mov	r2, a		;fetch block number into r2/r3
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	inc	dptr
	lcall	space
	lcall	space
	lcall	space
	lcall	space
	mov	r0, #2
	lcall	phex16_at_r0
	lcall	space
	mov	a, #'-'
	lcall	cout
	mov	a, #'>'
	lcall	cout
	push	dph
	push	dpl
	lcall	block_desc_addr
	mov	a, #2
	lcall	pfss_rec
	mov	a, #1
	lcall	pfss_rec
	mov	a, #1
	lcall	pfss_rec
	mov	a, #4
	lcall	pfss_rec
	mov	a, #8
	lcall	pfss_rec
	pop	dpl
	pop	dph
	mov	a, r5
	add	a, #255
	mov	r5, a
	mov	a, r6
	addc	a, #255
	mov	r6, a

	lcall	space
	mov	a, r6
	lcall	phex
	mov	a, r5
	lcall	phex

	lcall	newline
	sjmp	pfss_loop
pfss_done:
	ret

pfss_rec:
	mov	r0, a
	mov	r1, a
	lcall	space
pfss_rec2:
	movx	a, @dptr
	inc	dptr
	push	acc
	djnz	r0, pfss_rec2
pfss_rec3:
	pop	acc
	lcall	phex
	djnz	r1, pfss_rec3
	ret




msg_free_fat:
	.db	"free_fat_memory:",13,10,0


free_fat_memory:
	.equ	ff_blk_count, bsearch_tmp	;reuse this 16 bit variable

	 ;push	flags
	 ;setb	debug
	 ;lcall	print_fat_sector_summary
	 ;mov	dptr, #msg_free_fat
	 ;lcall	pstr_d
	 ;pop	flags
	mov	dptr, #mm_num_tracked
	movx	a, @dptr
	mov	ff_blk_count+0, a	;"ff_blk_count" # of blocks in list
	clr	a
	movx	@dptr, a		;and clear it to zero
	inc	dptr
	movx	a, @dptr
	mov	ff_blk_count+1, a
	clr	a
	movx	@dptr, a
	mov	dptr, #0xD000		;addr where block numbers are
free_fat_loop:
	mov	a, ff_blk_count+0
	jnz	free_fat_cont
	mov	a, ff_blk_count+1
	jz	free_fat_done
free_fat_cont:
	movx	a, @dptr
	mov	r2, a			;grab block number to free
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	inc	dptr
	push	dpl
	push	dph
	lcall	free_blocks
	pop	dph
	pop	dpl
	mov	a, ff_blk_count+0
	add	a, #255			;decrement count
	mov	ff_blk_count+0, a
	mov	a, ff_blk_count+1
	addc	a, #255
	mov	ff_blk_count+1, a
	sjmp	free_fat_loop
free_fat_done:
	mov	dptr, #last_fat_sector	;need to clear these, or else the
	mov	a, #255			;next cluster lookup will try to
	movx	@dptr, a		;use the free'd memory!
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	mov	dptr, #last_fat_fd
	movx	@dptr, a
	ljmp	return_addr


	;look for a block that contains "f32_sector" in the
	;sorted list from 0xD000 to 0xDFFF.  It's a binary search
	;of the list of blocks, each of which has a FAT sector
	;number stored in its block_info.
	;  If found, C=0 and r2/r3 is index in the array
	;  If not found, C=1 and r2/r3 is the index where
	;     it would need to be inserted.
bin_search:
	;first, find out how many blocks are in the tracker
	;array and make sure it's not zero
	mov	dptr, #mm_num_tracked
	movx	a, @dptr
	mov	r6, a
	inc	dptr
	movx	a, @dptr
	mov	r7, a
	jnz	bsearch_begin
	mov	a, r6
	jnz	bsearch_begin
	;if there are no blocks, return r2/r3 zero and C=1
	clr	a
	mov	r2, a
	mov	r3, a
	setb	c
	ret
bsearch_begin:
	;decrement r6/r7 to get the "end" index
	mov	a, r6
	add	a, #255
	mov	r6, a
	mov	a, r7
	addc	a, #255
	mov	r7, a
	;set r4/r5 to the the "begin" index (zero)
	clr	a
	mov	r4, a
	mov	r5, a
bsearch_loop:
	;compute the r2/r3 "middle" = begin + (end - begin) / 2
	clr	c
	mov	a, r6
	subb	a, r4		; LSB of (end - begin)
	mov	r2, a
	mov	a, r7
	subb	a, r5		; MSB of (end - begin)
	mov	r3, a
	clr	c
	mov	a, r3
	rrc	a		; MSB of / 2
	mov	r3, a
	mov	a, r2
	rrc	a		; LSB of / 2
	add	a, r4		; LSB of begin + ( )
	mov	r2, a
	mov	a, r3
	addc	a, r5		; MSB of begin + ( )
	mov	r3, a

	 ;mov	dptr, #msg_bsearch1
	 ;lcall	pstr_d
	 ;mov	r0, #4
	 ;lcall	phex16_at_r0
	 ;mov	dptr, #msg_bsearch2
	 ;lcall	pstr_d
	 ;mov	r0, #6
	 ;lcall	phex16_at_r0
	 ;mov	dptr, #msg_bsearch3
	 ;lcall	pstr_d
	 ;mov	r0, #2
	 ;lcall	phex16_at_r0
	 ;lcall	newline_d


	;get a pointer to the block's info.

	;save the "middle", and convert it into
	;the address within the array of block numbers
	clr	c
	mov	a, r2
	mov	bsearch_tmp, a
	rlc	a
	mov	dpl, a
	mov	a, r3
	mov	bsearch_tmp+1, a
	rlc	a
	add	a, #0xD0
	mov	dph, a

	;now read the block number from the array
	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a

	;get a pointer to the block's 16 byte info
	lcall	block_desc_addr

	;restore r2/r3 "middle"
	mov	r2, bsearch_tmp
	mov	r3, bsearch_tmp+1

	;now compare the 32 bits @r0 to those @dptr
	orl	dpl, #0x07	;point to the bits 31:24
	movx	a, @dptr
	cjne	a, f32_sector+3, bsearch_ne

	anl	dpl, #0xFE	;point to bits 23:16
	movx	a, @dptr
	cjne	a, f32_sector+2, bsearch_ne
	
	anl	dpl, #0xFC
	orl	dpl, #0x01	;point to bits 15:8
	movx	a, @dptr
	cjne	a, f32_sector+1, bsearch_ne

	anl	dpl, #0xFE	;point to bits 7:0
	movx	a, @dptr
	cjne	a, f32_sector+0, bsearch_ne

	;if we got here, it's a match and we're done!
	clr	c
	ret

bsearch_ne:
	jnc	bsearch_gt	;a > b,  @dptr > @r0
bsearch_lt:
	;if we get here, @dptr < f32_sector, so set  begin = middle + 1
	mov	a, r2
	add	a, #1
	mov	r4, a
	mov	r2, a		;also set middle = middle + 1
	mov	a, r3		;in case this was the last attempt
	addc	a, #0
	mov	r5, a
	mov	r3, a
	sjmp	bsearch_next

bsearch_gt:
	;if we get here, @dptr > f32_sector, so set  end = middle - 1
	mov	a, r2
	add	a, #255
	mov	r6, a
	mov	a, r3
	addc	a, #255
	mov	r7, a
	jnc	bsearch_not_found	;if end == -1, not found

bsearch_next:
	;if begin <= end, then we need to keep searching
	clr	c
	mov	a, r6
	subb	a, r4
	mov	a, r7
	subb	a, r5
	jc	bsearch_not_found
	ljmp	bsearch_loop

bsearch_not_found:
	setb	c
	ret













	;seek to a particular byte position specified by 32 bits
	;in r2/r3/r4/r5, in file refered by file descriptor in r6
file_seek:
	.equ	byte_count, bsearch_tmp		;reuse this 16 bit variable
	mov	file_desc, r6
	 mov	dptr, #msg_fseek1
	 lcall	pstr_d
	 mov	r0, #reg2
	 lcall	phex32_at_r0
	 lcall	newline_d
	mov	byte_count+0, r2
	mov	a, r3
	mov	byte_count+1, a
	anl	a, #0xF0
	swap	a
	mov	cluster_offset+0, a
	mov	a, r4
	anl	a, #0x0F
	swap	a
	orl	cluster_offset+0, a
	mov	a, r4
	anl	a, #0xF0
	swap	a
	mov	cluster_offset+1, a
	mov	a, r5
	anl	a, #0x0F
	swap	a
	orl	cluster_offset+1, a
	mov	a, r5
	anl	a, #0xF0
	swap	a
	mov	cluster_offset+2, a

	mov	a, sectors_per_cluster
	jb	acc.7, fseek_64k_clusters
	jb	acc.6, fseek_32k_clusters
	jb	acc.5, fseek_16k_clusters
	jb	acc.4, fseek_8k_clusters
	jb	acc.3, fseek_4k_clusters
	;if we get here, the cluster size isn't supported
	mov	dpl, #1
	ljmp	return_addr

fseek_64k_clusters:
	mov	r1, #4
	sjmp	fseek_shift
fseek_32k_clusters:
	anl	byte_count+1, #01111111b
	mov	r1, #3
	sjmp	fseek_shift
fseek_16k_clusters:
	anl	byte_count+1, #00111111b
	mov	r1, #2
	sjmp	fseek_shift
fseek_8k_clusters:
	anl	byte_count+1, #00011111b
	mov	r1, #1
	sjmp	fseek_shift
fseek_4k_clusters:
	anl	byte_count+1, #00001111b
	sjmp	fseek_shift_skip

fseek_shift:
	clr	c
	mov	a, cluster_offset+2
	rrc	a
	mov	cluster_offset+2, a
	mov	a, cluster_offset+1
	rrc	a
	mov	cluster_offset+1, a
	mov	a, cluster_offset+0
	rrc	a
	mov	cluster_offset+0, a
	djnz	r1, fseek_shift
fseek_shift_skip:

	mov	a, file_desc
	lcall	fd_to_ptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	movx	a, @dptr	;get "first_cached_cluster"
	mov	r2, a
	mov	r6, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	mov	r7, a

fseek_loop:
	mov	a, r2
	jnz	fseek_loop2
	mov	a, r3
	jnz	fseek_loop2
	ljmp	fseek_eof
fseek_loop2:
	lcall	block_desc_addr
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+0, fseek_loop_next1
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+1, fseek_loop_next2
	inc	dptr
	movx	a, @dptr
	cjne	a, cluster_offset+2, fseek_loop_next3
	sjmp	fseek_found_cluster
fseek_loop_next1:
	inc	dptr
fseek_loop_next2:
	inc	dptr
fseek_loop_next3:
	inc	dptr
	inc	dptr
	movx	a, @dptr	;get "next_cluster"
	mov	r2, a
	mov	r6, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	mov	r7, a
	sjmp	fseek_loop

fseek_found_cluster:
	;now we've found the cluster, let's get the individual
	;4k block within that cluster.
	mov	a, byte_count+1
	anl	byte_count+1, #0x0F
	anl	a, #0xF0
	jz	fseek_found
	swap	a
	mov	r1, a
fseek_block_loop:
	lcall	next_block
	djnz	r1, fseek_block_loop

fseek_found:
	 mov	dptr, #msg_fseek2
	 lcall	pstr_d
	 mov	r0, #cluster_offset
	 lcall	phex24_at_r0
	 mov	dptr, #msg_fseek3
	 lcall	pstr_d
	 mov	r0, #6
	 lcall	phex16_at_r0
	 mov	dptr, #msg_fseek4
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 mov	dptr, #msg_fseek5
	 lcall	pstr_d
	 mov	r0, #byte_count
	 lcall	phex16_at_r0
	 lcall	newline_d
	mov	a, file_desc
	lcall	fd_to_ptr
	mov	a, #16
	lcall	add_a_to_dptr
	mov	a, r2
	movx	@dptr, a	;store "pos_block"
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	mov	a, byte_count+0
	movx	@dptr, a	;store "pos_byte"
	inc	dptr
	mov	a, byte_count+1
	movx	@dptr, a
	mov	dpl, #0
	ljmp	return_addr

fseek_eof:
	 mov	dptr, #msg_fseek_eof
	 lcall	pstr_d
	mov	dpl, #1
	ljmp	return_addr








file_read_c:
	push	reg4
	acall	file_read
	pop	dpl
	jnc	file_read_c_ok
	mov	dpl, #0
file_read_c_ok:
	ljmp	return_addr



	;read r1 bytes from file descriptor r4 into a buffer
	;located at r6/r7.  The file's byte position is advanced.
	;the read is only successful if the data requested is
	;in the cache.  C=0 is returned if the data was read,
	;or C=1 if not avaialble.
file_read:
	;first we need to find a cluster at or before the
	;current read position.  Either use
	;file_info[fd].current_cluster_position, or
	;file_info[fd].first_cached_cluster
	mov	a, r4
	mov	file_desc, a
	lcall	fd_to_ptr
	mov	a, #16
	lcall	add_a_to_dptr
	mov	cluster+2, dpl		;keep a copy of dptr
	mov	cluster+3, dph

	movx	a, @dptr
	inc	dptr
	mov	r2, a			;get pos_block into r2/r3
	movx	a, @dptr
	inc	dptr
	mov	r3, a
	movx	a, @dptr
	inc	dptr
	mov	r4, a			;get pos_byte into r4/r5
	movx	a, @dptr
	anl	a, #0x0F
	orl	a, #0xB0
	mov	r5, a			;r4/r5 point to source address
	mov	a, #0x0B
	lcall	map_block

fread_copy_loop:		;this loop could be faster...
	mov	dpl, r4
	mov	dph, r5
	movx	a, @dptr		;read a byte from the cache
	inc	dptr
	mov	r4, dpl
	mov	r5, dph
	mov	dpl, r6
	mov	dph, r7
	movx	@dptr, a		;write to the user's buffer
	inc	dptr
	mov	r6, dpl
	mov	r7, dph
	mov	a, r5			;check if end of this block
	anl	a, #0xF0
	add	a, #256 - 0xB0
	jz	fread_copy_next
fread_next_block:
	;advances to the beginning of the next block
	mov	cluster+0, r2
	mov	cluster+1, r3
	lcall	next_block
	mov	a, r2
	jnz	fread_nxtblk_done
	mov	a, r3
	jnz	fread_nxtblk_done
	;if we get here, this was the last 4k block of the
	;cluster, so we need to advance to the next cluster
	mov	r2, cluster+0
	mov	r3, cluster+1
	lcall	block_desc_addr
	mov	a, #8
	lcall	add_a_to_dptr
	movx	a, @dptr		;read "next_cluster"
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	jnz	fread_nxtblk_done	;check if last cluster
	mov	a, r2
	jz	fread_eof
	;there really should be a check to see if we skipped
	;over a "gap" in the cached data!  Compare the cluster
	;offset of this cluster to the last one... they should
	;differ by only one.
fread_nxtblk_done:
	mov	a, #0x0B
	lcall	map_block
	mov	r4, #0
	mov	r5, #0xB0
	 mov	dptr, #msg_rf_next_block
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 mov	r0, #4
	 lcall	phex16_at_r0
	 lcall	newline_d
fread_copy_next:
	djnz	r1, fread_copy_loop


	;now just write the position back to the file_info
fread_finishup:
	mov	dpl, cluster+2
	mov	dph, cluster+3
	mov	a, r2			;store pos_block
	movx	@dptr, a
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	mov	a, r4			;store pos_byte
	movx	@dptr, a
	inc	dptr
	mov	a, r5
	anl	a, #0x0F
	movx	@dptr, a
	clr	c
	ret

fread_eof:
	mov	dpl, cluster+2		;keep a copy of dptr
	mov	dph, cluster+3
	clr	a
	movx	@dptr, a		;pos_block = 0
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a		;pos_byte = 0
	inc	dptr
	movx	@dptr, a
	 mov	dptr, #msg_rf_eof
	 lcall	pstr_d
	setb	c
	ret



































;this is the struct that's written into the user's buffer by dir_read
;struct file_struct {
;	byte	attrib;
;	char	short_name[13];
;	uint32	cluster;
;	uint32	size;
;	char	long_name[230];
;}


; FAT32 long filename support added by Tom Parker


	;read the next filename and other info from a directory
	;r4 = file descriptor
	;r6/r7 = buffer in external memory to transfer data
dir_read:
	.equ	user_buf, bsearch_tmp		;reuse this 16 bit variable
	.equ	ptr_tmp_buf, prev_cluster_at_block
	.equ	tmp_block, next_cluster_at_block
	mov	file_desc, r4
	mov	user_buf+0, r6
	mov	user_buf+1, r7
	mov	a, #1
	lcall	malloc_blocks		;malloc a temporary buffer
	jnc	dir_read_ok
	ljmp	dirrd_eof1		;out of memory!
dir_read_ok:
	mov	tmp_block+0, r2
	mov	tmp_block+1, r3
	mov	a, #0xC
	lcall	map_block
	mov	ptr_tmp_buf+0, #0
	mov	ptr_tmp_buf+1, #0xC0

dirrd_loop:
	mov	r4, file_desc
	mov	r1, #32
	mov	r6, ptr_tmp_buf+0
	mov	r7, ptr_tmp_buf+1
	lcall	file_read		;grab another 32 bytes
	jc	dirrd_eof
	mov	dpl, ptr_tmp_buf+0
	mov	dph, ptr_tmp_buf+1	;dptr points to first byte of filename
	movx	a, @dptr
	jz	dirrd_eof		;check for end of directory
	mov	r1, a			;save first byte for lfn index
	add	a, #256 - 0xE5
	jz	dirrd_loop		;check for unused (deleted) entry
; TODO - determine why this test was here
; From the FAT32 spec (at Microsoft's web site):
;   http://www.microsoft.com/HWDEV/storage/default.htm#Specs
; If DIR_Name[0] == 0x05, then the actual file name character for this byte
; is 0xE5. 0xE5 is actually a valid KANJI lead byte value for the character
; set used in Japan. The special 0x05 value is used so that this special
; file name case for Japan can be handled properly and not cause FAT file
; system code to think that the entry is free. 
;  Indeed this appears to be more or less incompatible with long
;  filenames.  I originally put it in the original code because of
;  that little note in the FAT32 spec, but it should probably just
;  end up being removed, since any modern system would be using
;  unicode characters in the long filename.
;	add	a, #(256 - 5 + 0xE5) & 255
;	jz	dirrd_loop		;check for unused (deleted) entry
	mov	a, ptr_tmp_buf+0
	orl	a, #00001011b		;offset 11 is attrib
	mov	dpl, a
	movx	a, @dptr		;grab the attrib byte
	anl	a, #0x3F		;ignore upper two bits of attrib
	cjne	a, #0x0F, dirrd_got_file ;not lfn entry?
	inc	dptr			;point to LDIR_Type
	movx	a, @dptr		;check if an LFN entry (=0)
	jnz	dirrd_loop
	mov	a, r1			;get LDIR_Ord
	anl	a, #0x40		;check for start of lfn entries marker (last entry)
	jnz	dirrd_lfn_first
	inc	dptr			;point to LDIR_Chksum
	movx	a, @dptr		;compare to previous checksum, restart if no match
	cjne	a, dir_read_lfn_cksum, dirrd_lfn_first
dirrd_lfn_entry:
	mov	a, ptr_tmp_buf+0
	add	a, #32
	mov	ptr_tmp_buf+0, a
	mov	a, ptr_tmp_buf+1
	addc	a, #0
	mov	ptr_tmp_buf+1, a
	anl	a, #0xF0
	cjne	a, #0xC0, dirrd_oflow
	sjmp	dirrd_loop
	;if we get here, we've use more than the 4k buffer... this
	;shouldn't happen, but if we don't do something we'll start
	;writing over other memory
dirrd_oflow:
	mov	ptr_tmp_buf+1, #0xC0
	mov     ptr_tmp_buf+0, #0
	sjmp	dirrd_loop

dirrd_eof:
	mov	r2, tmp_block+0         ;free ram block
	mov	r3, tmp_block+1
	lcall	free_blocks
	;we hit the end of the directory... we should fill in the user's
	;buffer with something reasonable.
dirrd_eof1:
	mov	dpl, user_buf+0
	mov	dph, user_buf+1
	clr	a
	mov	r1, #250
dirrd_eof2:
	movx	@dptr, a
	inc	dptr
	djnz	r1, dirrd_eof2
	mov	dpl, #1
	ljmp	return_addr

dirrd_lfn_first:
	mov	a, dpl			;point to LDIR_Chksum
	anl	a, #11100000b
	orl	a, #00001101b
	mov	dpl,a
	movx	a, @dptr		;save checksum
	mov	dir_read_lfn_cksum, a
	mov	r4, ptr_tmp_buf+0	;point to current entry
	mov	r5, ptr_tmp_buf+1
	cjne	r4, #0, dirrd_lfn_copy	;already at buffer start?
	cjne	r5, #0xC0, dirrd_lfn_copy
	ajmp	dirrd_lfn_entry		;leave this entry where it is
dirrd_lfn_copy:
	mov	r6, #0			;point to start of buffer
	mov     r7, #0xC0
	mov	r1, #32			;32 bytes
dirrd_lfn_copy_next:
	mov	dpl, r4			;source
	mov	dph, r5	
	movx	a, @dptr
	inc	dptr
	mov	r4, dpl
	mov	r5, dph
	mov	dpl, r6			;dest
	mov	dph, r7
	movx	@dptr, a
	inc	dptr
	mov	r6, dpl
	mov	r7, dph
	djnz	r1, dirrd_lfn_copy_next
	mov	ptr_tmp_buf+0, dpl
	mov	ptr_tmp_buf+1, dph
	ljmp	dirrd_loop
dirrd_got_file:
	mov	b, a			;save attrib
	clr	a
	cjne	a, user_buf+0, dirrd_copy ;check for null pointer flag
	cjne	a, user_buf+1, dirrd_copy ;check for null pointer flag
	ajmp	dirrd_exit		;skip copy
dirrd_copy:	
	mov	dpl, user_buf+0
	mov	dph, user_buf+1
	mov	a, b
	movx	@dptr, a		;attrib
	inc	dptr
	mov	r1, #11
dirrd_copy1:
	mov	r2, dpl
	mov	r3, dph
	mov	dpl, ptr_tmp_buf+0
	mov	dph, ptr_tmp_buf+1
	movx	a, @dptr		;read short filename
	inc	dptr
	mov	ptr_tmp_buf+0, dpl
	mov	ptr_tmp_buf+1, dph
	mov	dpl, r2
	mov	dph, r3
	movx	@dptr, a		;copy to user's buffer
	inc	dptr
	cjne	r1, #4, dirrd_copy2
	mov	a, #'.'
	movx	@dptr, a		;add the dot to the short name
	inc	dptr
dirrd_copy2:
	djnz	r1, dirrd_copy1

	clr	a
	movx	@dptr, a		;add a null termination
	inc	dptr
	mov	r2, dpl
	mov	r3, dph
	mov	dph, ptr_tmp_buf+1
	mov	a, ptr_tmp_buf+0
	anl	a, #11100000b
	orl	a, #00010100b
	mov	dpl, a
	movx	a, @dptr		;read cluster, high word
	inc	dptr
	mov	cluster+2, a
	movx	a, @dptr
	inc	dptr
	mov	cluster+3, a
	inc	dptr
	inc	dptr
	inc	dptr
	inc	dptr
	movx	a, @dptr		;read cluster, low word
	inc	dptr
	mov	cluster+0, a
	movx	a, @dptr
	inc	dptr
	mov	cluster+1, a
	movx	a, @dptr		;read size (into f32_sector)
	inc	dptr
	mov	f32_sector+0, a
	movx	a, @dptr
	inc	dptr
	mov	f32_sector+1, a
	movx	a, @dptr
	inc	dptr
	mov	f32_sector+2, a
	movx	a, @dptr
	inc	dptr
	mov	f32_sector+3, a
	mov	dpl, r2
	mov	dph, r3
	mov	a, cluster+0		;copy cluster to user's buffer
	movx	@dptr, a
	inc	dptr
	mov	a, cluster+1
	movx	@dptr, a
	inc	dptr
	mov	a, cluster+2
	movx	@dptr, a
	inc	dptr
	mov	a, cluster+3
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+0		;copy size into user's buffer
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+1
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a
	inc	dptr
	mov	r2, dpl			;save current user pointer
	mov	r3, dph
	mov	dpl, ptr_tmp_buf+0	;point to sfn entry
	mov	dph, ptr_tmp_buf+1
	mov	a, dpl
	anl	a, #11100000b		;round down to start of entry
	mov	dpl, a
	mov	r1, #11			;11 bytes
	clr	a
dirrd_cksum_loop:
	rr	a			;evaluate chksum & skip if no match
	mov	b, a
	movx	a, @dptr
	add	a, b
	inc	dptr
	djnz	r1, dirrd_cksum_loop
	cjne	a, dir_read_lfn_cksum, dirrd_copy_lfn_end 
	mov	dpl, ptr_tmp_buf+0
	mov	dph, ptr_tmp_buf+1
	mov	r1, #1			;block 1 first
dirrd_copy_lfn_next:
	mov	a, dpl
	anl	a, #11100000b		;round down to start of entry
	clr	c
	subb	a, #32			;move to start of previous entry
	mov	dpl, a
	mov	a, dph
	subb	a, #0
	mov	dph, a
	anl	a, #0xF0		;check we are still in this block
	cjne	a, #0xC0, dirrd_copy_lfn_end ;abort if not
	movx	a, @dptr		;check entry number
	mov	b, a			;save for end flag
	anl	a, #0x3f
	clr	c
	subb	a, r1
	jnz	dirrd_copy_lfn_end 	;out of sequence so abort
	inc	dptr			;skip LDIR_Ord
	mov	r0, #5			;chars 1-5
	lcall	dirrd_copy_lfn_block
	inc	dptr			;skip last character
	inc	dptr
	inc	dptr			;skip LDIR_Attr
	inc	dptr			;skip LDIR_Type
	inc	dptr			;skip LDIR_Chksum
	mov	r0, #6			;chars 6-11
	lcall	dirrd_copy_lfn_block
	inc	dptr			;skip last character
	inc	dptr
	inc	dptr			;skip LDIR_Fst_ClusLO
	inc	dptr
	mov	r0, #2			;chars 12-13
	lcall	dirrd_copy_lfn_block
	mov	a, b
	jb	acc.6, dirrd_copy_lfn_end ;check last entry bit
	inc	r1
	cjne	r1, #17, dirrd_copy_lfn_next ;230 chars holds ~17 lfn entries
dirrd_copy_lfn_end:
	mov	dpl, r2
	mov	dph, r3
	clr	a
	movx	@dptr, a		;add a null to ensure termination
dirrd_exit:
	mov	r2, tmp_block+0
	mov	r3, tmp_block+1
	lcall	free_blocks
	inc	dir_read_current_entry
	mov	a, dir_read_current_entry
	jnz	dirrd_exit1
	inc	dir_read_current_entry+1
dirrd_exit1:
	mov	dpl, #0
	ljmp	return_addr


dirrd_copy_lfn_loop:
	inc	dptr
	inc	dptr			;skip 2nd char of unicode
dirrd_copy_lfn_block:
	movx	a, @dptr		;get lfn character
	mov	r6, dpl
	mov	r7, dph
	mov	dpl, r2
	mov	dph, r3
	movx	@dptr, a
	inc	dptr
	mov	r2, dpl
	mov	r3, dph
	mov	dpl, r6
	mov	dph, r7
	djnz	r0, dirrd_copy_lfn_loop
	ret







.equ	ret_block, bsearch_tmp		;reuse this 16 bit variable



	;return the block number (in r2/r3) of the next block from
	;file descriptor r4.  The file's byte position is advanced
	;just past this block.   Acc returns a status code:
	; DPTR = BLOCK#, block read ok, r2/r3 has block number
	; DPTR = 0,      block's transfer is still pending, try again later
	; DPTR = 0xFFFF, end of file (or cached portion)

file_read_block:
	;first we need to find a cluster at or before the
	;current read position.  Either use
	;file_info[fd].current_cluster_position, or
	;file_info[fd].first_cached_cluster
	 mov	dptr, #msg_frd_blk_1
	 lcall	pstr_d
	mov	a, r4
	mov	file_desc, a
	lcall	fd_to_ptr
	movx	a, @dptr		;read "flags"
	anl	a, #00000010b		;mask off only "caching" bit
	mov	r5, a			;and keep in in r5 for use later
	mov	a, #16
	lcall	add_a_to_dptr
	mov	r6, dpl			;keep a copy of dptr
	mov	r7, dph

	;grab the current block position, which we'll return to the user
	movx	a, @dptr
	inc	dptr
	mov	r2, a			;get pos_block into r2/r3
	mov	ret_block+0, a
	movx	a, @dptr
	inc	dptr
	mov	r3, a
	mov	ret_block+1, a
	jnz	fread_blk_ck_pending
	mov	a, r2
	jnz	fread_blk_ck_pending
	 mov	a, #'$'
	 lcall	cout_d
	 lcall	newline_d
	mov	dptr, #0xFFFF		;return "end of file"
	ljmp	return_addr

fread_blk_ck_pending:
	;now check if the current block actually has valid data... if
	;its read_pending bit is set, we'd better return a code to
	;the main program to tell it that we can't give it the block
	lcall	block_desc_addr
	inc	dptr
	inc	dptr
	movx	a, @dptr
	rlc	a
	jnc	fread_blk_next
	mov	dptr, #0	;return transfer still pending status
	ljmp	return_addr

fread_blk_next:
	;now advance to the next block, to update the file's position
	lcall	next_block
	 mov	r0, #ret_block
	 lcall	phex16_at_r0
	 lcall	newline_d
	mov	a, r3
	jnz	fread_blk_done
	mov	a, r2
	jnz	fread_blk_done
	;we need to advance to the next cluster
	mov	r2, ret_block+0
	mov	r3, ret_block+1
	lcall	block_desc_addr
	mov	a, #8
	lcall	add_a_to_dptr
	movx	a, @dptr		;read "next_cluster"
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	cjne	r5, #2, fread_blk_done
	;if we're in caching mode, we'd better make sure that r2/r3
	;isn't zero... if it is, we would end up setting pos_block
	;to zero, which would make it impossible to read the data
	;that will soon be added to the cache, so if r2/r3 is zero
	;(no next cluster yet), we'll pretend like there is actually
	;a next cluster and it's read is still pending, when the
	;truth is that there will be a next cluster in the near future
	mov	a, r2
	jnz	fread_blk_done
	mov	a, r3
	jnz	fread_blk_done
	mov	dptr, #0	;return transfer still pending status
	ljmp	return_addr

fread_blk_done:
	;this is the easy case, where we don't need to advance to
	;the next cluster.
	mov	dpl, r6
	mov	dph, r7
	mov	a, r2
	movx	@dptr, a		;update pos_block
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	clr	a
	movx	@dptr, a		;zero pos_byte
	inc	dptr
	movx	@dptr, a
	mov	dpl, ret_block+0
	mov	dph, ret_block+1
	ljmp	return_addr





	;close file descriptor r4 and frees all the memory that
	;was used to cache that file (if any)
file_close:
	mov	a, r4
	mov	file_desc, a
	lcall	fd_to_ptr
	mov	a, #8
	lcall	add_a_to_dptr

	movx	a, @dptr
	inc	dptr
	mov	r2, a			;get "first_cached_cluster" in r2/r3
	movx	a, @dptr
	inc	dptr
	mov	r3, a

fclose_clust_loop:
	mov	a, r2
	jnz	fclose_free_clust
	mov	a, r3
	jnz	fclose_free_clust
	sjmp	fclose_clust_done
fclose_free_clust:
	lcall	block_desc_addr
	mov	a, #8
	lcall	add_a_to_dptr
	movx	a, @dptr
	inc	dptr
	mov	cluster+0, a		;get "next_cluster" in "cluster"
	movx	a, @dptr
	inc	dptr
	mov	cluster+1, a
	lcall	free_blocks		;free this cluster
	mov	r2, cluster+0
	mov	r3, cluster+1
	sjmp	fclose_clust_loop
	
fclose_clust_done:
	;when we get here, all the clusters are free.

	;TODO: need to scan all the stored FAT sectors and find the
	;ones which were being used by only this file to free them,
	;and of course make sure that none of them are marked by
	;this file anymore.  Without doing this, we'll slowly leak
	;memory... not so good

	mov	a, file_desc
	lcall	fd_to_ptr
	clr	a
	movx	@dptr, a		;mark this file desc as available
	ljmp	return_addr

































;*************************************************************
;**                                                         **
;**              IDE Disk Drive Driver Code                 **
;**                                                         **
;*************************************************************




	.equ	ide_s_idle, 1		;ready to issue a command
	.equ	ide_s_wait_drq, 2	;waiting for drive to read (drq bit)
	.equ	ide_s_wait_xfer, 3	;waiting for dma xfer to complete
	.equ	ide_s_wait_nbusy, 4	;waiting for command to finish
	.equ	ide_s_wait_sleep, 5	;waiting for drive to go to sleep
	.equ	ide_s_sleeping,	6	;drive is in full sleep mode

	;call here to do whatever IDE action is needed next.
	;eventually, this will be an interrupt service
	;routine instead of polling.  This first section just
	;examines the state of the IDE code and if whatever
	;we're waiting for (if anything) is ready, and dispatches
	;to the whatever code is needed to do the next thing.

	;return c=1 if in middle of a command, c=0 if idle
ide:
	 mov	dptr, #msg_ide_enter
	 lcall	pstr_d
	mov	a, ide_state


ide_state0:
	cjne	a, #ide_s_sleeping, ide_state1
	 mov	dptr, #msg_ide_state6
	 lcall	pstr_d
	clr	c
	ret


	;ide_s_idle: ready to issue a command
ide_state1:
	cjne	a, #ide_s_idle, ide_state2
	 mov	dptr, #msg_ide_state1
	 lcall	pstr_d
	ljmp	ide_next_cmd



	;ide_s_wait_drq: waiting for drive to read (drq bit)
ide_state2:
	cjne	a, #ide_s_wait_drq, ide_state3
	 mov	dptr, #msg_ide_state2
	 lcall	pstr_d
	mov	dptr, #ide_status
	movx	a, @dptr
	jb	acc.3, ide_state2b	;is DRQ set
	;should there be a timeout here?
	;  TODO: this really wants to have a timeout... many times
	;  that I've seen problems, this is where it tends to hang
	setb	c
	ret
ide_state2b:
	ljmp	ide_do_xfer



	;ide_s_wait_xfer: waiting for dma xfer to complete
ide_state3:
	cjne	a, #ide_s_wait_xfer, ide_state4
	 mov	dptr, #msg_ide_state3
	 lcall	pstr_d
	mov	dptr, #irq_ident
	movx	a, @dptr
	jb	acc.0, ide_state3b
	;should there be a timeout here?
	setb	c
	ret
ide_state3b:
	ljmp	ide_xfer_done




	;ide_s_wait_nbusy: waiting for command to finish
ide_state4:
	cjne	a, #ide_s_wait_nbusy, ide_state5
	 mov	dptr, #msg_ide_state4
	 lcall	pstr_d
	mov	dptr, #ide_status
	movx	a, @dptr
	jnb	acc.7, ide_state4b
	;should there be a timeout here?
	setb	c
	ret
ide_state4b:
	mov	ide_state, #ide_s_idle
	clr	c
	ret


ide_state5:
	cjne	a, #ide_s_wait_sleep, ide_state7
	 mov	dptr, #msg_ide_state5
	 lcall	pstr_d
	;should probably do something to check if it went to sleep
	;read status (clears interrupt for this command), should
	;get the drive into full sleep much faster
	mov	ide_state, #ide_s_sleeping
	clr	c
	ret


	;unknown state, we should never get here
ide_state7:
	;unknown state, should never get here.
	 mov	dptr, #msg_ide_state7
	 lcall	pstr
	mov	ide_state, #ide_s_idle
	clr	c
	ret











	;get the next request from one of the request queues
	;and start it.
ide_next_cmd:
	mov	a, fat_rreq_tail
	cjne	a, fat_rreq_head, ide_next_high
	mov	a, #255
	mov	fat_rreq_head, a	;no high priority requests
	mov	fat_rreq_tail, a
	mov	a, clust_rreq_tail
	cjne	a, clust_rreq_head, ide_next_low
	mov	a, #255
	mov	clust_rreq_head, a	;no low priority requests
	mov	clust_rreq_tail, a
	clr	c
	ret


	;there's a high priority request pending.
ide_next_high:
	inc	a
	cjne	a, #fat_rreq_size, ide_next_high2
	clr	a
ide_next_high2:
	mov	fat_rreq_tail, a
	swap	a
	rr	a		;sizeof(struct read_request) == 8
	add	a, #fat_read_req & 255
	mov	dpl, a
	clr	a
	addc	a, #fat_read_req >> 8
	mov	dph, a
	sjmp	ide_next_cmd2


	;there's a low priority request pending.
ide_next_low:
	inc	a
	cjne	a, #clust_rreq_size, ide_next_low2
	clr	a
ide_next_low2:
	mov	clust_rreq_tail, a
	mov	b, #8		;sizeof(struct read_request) == 8
	mul	ab
	add	a, #clust_read_req & 255
	mov	dpl, a
	mov	a, b
	addc	a, #clust_read_req >> 8
	mov	dph, a


	;when we get here, the request has already been
	;removed from its queue, but we still need to
	;copy its data (@dptr).
ide_next_cmd2:
	movx	a, @dptr
	inc	dptr
	mov	r4, a		;read LBA (source) into r4/r7
	movx	a, @dptr
	inc	dptr
	mov	r5, a
	movx	a, @dptr
	inc	dptr
	mov	r6, a
	movx	a, @dptr
	inc	dptr
	mov	r7, a
	movx	a, @dptr
	inc	dptr
	mov	r2, a		;read block (destination) into r2/r3
	movx	a, @dptr
	inc	dptr
	mov	r3, a
	movx	a, @dptr	;read request byte (0=read, 1=sleep)
	cjne	a, #1, ide_next_norm
	;if acc=1, we're supposed to put the drive to sleep

	 mov	dptr, #msg_ide_sleep
	 lcall	pstr_d
	mov	dptr, #ide_command
	mov	a, #ide_cmd_sleep
	movx	@dptr, a
	mov	ide_state, #ide_s_wait_sleep
	setb	c
	ret


ide_next_norm:
	 mov	dptr, #msg_ide_read
	 lcall	pstr_d
	 mov	a, r7
	 lcall	phex_d
	 mov	a, r6
	 lcall	phex_d
	 mov	a, r5
	 lcall	phex_d
	 mov	a, r4
	 lcall	phex_d
	 mov	dptr, #msg_ide_read2
	 lcall	pstr_d
	 mov	a, r3
	 lcall	phex_d
	 mov	a, r2
	 lcall	phex_d
	 lcall	newline_d

	;now we actually get to work on executing this command

	mov	a, r7
	anl	a, #0x0F
	orl	a, master_slave		;select correct drive
	mov	dptr, #ide_head		;write LBA to drive
	movx	@dptr, a
	mov	a, r6
	mov	dptr, #ide_cyl_msb
	movx	@dptr, a
	mov	a, r5
	mov	dptr, #ide_cyl_lsb
	movx	@dptr, a
	mov	a, r4
	mov	dptr, #ide_sector
	movx	@dptr, a
	mov	a, #8			;request 8 sectors (4k)
	mov	dptr, #ide_sec_cnt
	movx	@dptr, a
        mov     dptr, #ide_command
        mov     a, #ide_cmd_read_mult
        movx    @dptr, a                ;ask the drive to read it
	mov	ide_state, #ide_s_wait_drq

	;now the drive should be working on fetching these sectors!

	mov	dptr, #dram_page_cfg + (0xA * 2)
	mov	a, r2
        movx    @dptr, a		;map the destintion block to
        inc     dptr			;0xA000 - 0xAFFF (the page that's
        mov     a, r3			;reserved for this IDE driver)
        movx    @dptr, a

	;someday maybe this code will get optimized to read the
	;next request from the queue here.  There's plenty of
	;time to waste here, since the drive will be working on
	;getting the next sectors.  If we could do most of
	;that time consuming work to de-queue the next request,
	;there'd be less time wasted between reads.  Someday when
	;this is an interrupt routine, maybe we can dedicate a
	;register bank to it and pre-read r2-r7 and just have a
	;status bit that says if another request is waiting in
	;r2-r7 ready to be started ASAP.

	setb	c
	ret






	;at this point, the drive has fetched a group of 8 sectors
	;into its buffer, and we need to start a DMA transfer to
	;copy them from the drive into a block of memory, which
	;has already been mapped at 0xA000-0xAFFF.
ide_do_xfer:
        mov     dptr, #dma_ide_dest
	clr	a
        movx    @dptr, a                ;write LSB of dest addr
        inc     dptr
        mov     a, #0xA0
        movx    @dptr, a                ;write MSB of dest addr
        mov     dptr, #dma_ide_count
        clr     a
        movx    @dptr, a
        inc     dptr
        mov	a, #8
        movx    @dptr, a		;request 2048 16 bit reads
        mov     dptr, #irq_dma_ide_ack
        movx    @dptr, a                ;clear any previous interrupt

;	clr	ea	;** for testing only

        mov     dptr, #dma_ide_go
        movx    @dptr, a                ;begin the transfer

	;now the DMA controllers is rapidly copying the data
	;from the drive to DRAM memory.  With the current bus
	;arbitrator, 2 bytes are copied each time the 8051
	;asserts PSEN, which means we'll get somewhere between
	;1.23 to 2.45 Mbyte/sec transfer.  For this 4096 byte
	;block, it'll take somewhere between 1.7 ms to 3.3 ms.
	;Someday I'll make a better bus arbitrator and perhaps
	;support faster IDE access modes and burst writes to
	;the DRAM, but with the 14.7 MHz clock and a shared
	;bus between the two, it's unlikely the transfer speed
	;will ever get over 4 Mbyte/sec.  A better bus arbitrator
	;will instantly make all transfers run near 2.45
	;Mbyte/sec, instead of depending on the 8051's code
	;density (MOVX's will always temporarily pause DMA).
	;It kinda sucks that there weren't just 5 more pins
	;available on the FPGA, so that the IDE address could
	;be controlled separately from the DRAM address, to
	;overlap the two, instead of a read from the IDE to
	;a buffer followed by a write from the buffer to the
	;DRAM (all happening in the FPGA, of course).  The
	;effective read speed depends on the time it takes
	;the drive to fetch sectors into its buffer, the DMA
	;speed, and the overhead in this code to start the
	;transfers.  That all assumes that the main program
	;is running fast enough to keep at least one request
	;in the request queues.  There is a low priority
	;queue for the (easy) clusters and a high priority
	;queue for fat sectors, which take quite a bit more
	;effort to locate and are critically important for
	;finding the clusters... the hope is that if it takes
	;the main program quite a while to figure out where
	;to read a FAT sector, there'll be quite a few
	;requests sitting in the low priority queue, so this
	;IDE code can stay busy moving data at the maximum
	;rate.  The faster we can get the SIMM filled, the
	;sooner the drive can get shut off, saving batteries.


;this little test loop provides 2048 PSEN strobes
;	mov	r0, #0
;ide_dma_loop:
;	mov	a, #1			;2
;	mov	a, #1			;2
;	mov	a, #1			;2
;	djnz	r0, ide_dma_loop	;2

;	mov	a, #0x40
;	lcall	ide_check_dma

;	setb	ea

	mov	ide_state, #ide_s_wait_xfer
;	setb	c
	clr	c
	ret



	;call here to check if the IDE dma destination has 
	;a legal value.
ide_check_dma:
	mov	r0, a
	mov	dptr, #dma_ide_dest+1
	movx	a, @dptr
	mov	r1, a
	cjne	a, #0xB0, ideckdma2
	mov	dma_bug_temp, #0
	ret
ideckdma2:
	anl	a, #0xF0
	cjne	a, #0xA0, ideckdma3
	mov	dma_bug_temp, #0
	ret
ideckdma3:
	mov	a, dma_bug_temp
	jz	ideckdma4
	ret
ideckdma4:
	mov	dptr, #msg_dma_err
	lcall	pstr
	mov	a, r1
	lcall	phex
	lcall	space
	mov	a, r0
	lcall	phex
	lcall	newline
	mov	dma_bug_temp, #1
	ret


msg_dma_err:
	.db	"IDE DMA Error, Addr=",0


















	;now the xfer is done... we need to find the block_info
	;struct that describes the block we just transfered, and
	;mark it as transfered.  Finding the block_info could
	;be done with "block_desc_addr", but we need to map it
	;into our IDE page (0xA000-0xAFFF), as the main program
	;won't take kindly to this code remapping any other
	;pages when it is run as an interrupt service routine
ide_xfer_done:
	mov	dptr, #dram_page_cfg + (0xA * 2)
	movx	a, @dptr	;first, what block did we write?
	mov	r2, a		;r2/r3 holds block number
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	clr	a		;block_info always in first 32 pages
	movx	@dptr, a
	mov	dptr, #dram_page_cfg + (0xA * 2)
	mov	a, r3
	movx	@dptr, a
	mov	a, r2
	swap	a
	anl	a, #0xF0
	orl	a, #0x02	;flags in byte offset 2
	mov	dpl, a
	mov	a, r2
	swap	a
	anl	a, #0x0F
	orl	a, #0xA0
	mov	dph, a		;now dptr points to flags byte
	movx	a, @dptr
	anl	a, #0x7F	;clear "read is pending" flag
	movx	@dptr, a
	mov	ide_state, #ide_s_wait_nbusy
	setb	c
	ret



ide_wakeup_c:
	 mov	dptr, #msg_ide_wake
	 lcall	pstr_d
	mov	a, ide_state
	add	a, #256 - ide_s_wait_sleep
	jz	ide_hard_wakeup
	add	a, #(256 - ide_s_sleeping + ide_s_wait_sleep) & 255
	jz	ide_hard_wakeup
	;if we get here, the drive isn't sleeping, so we can just
	;return and everything should be ok.
	ljmp	return_addr
ide_hard_wakeup:
	lcall	ide_hard_reset
	ljmp	ide_init_c


msg_ide_wake:
	.db	"ide: wakeup",13,10,0





	;flush all pending IDE read requests
ide_flush_c:
	;disable ide interrupt here
	mov	a, clust_rreq_head
	cpl	a
	jz	ide_fl2
	clr	a
	mov	clust_rreq_head, a
	mov	clust_rreq_tail, a
ide_fl2:mov	a, fat_rreq_head
	cpl	a
	jz	ide_fl3
	clr	a
	mov	fat_rreq_head, a
	mov	fat_rreq_tail, a
ide_fl3: ;re-enable ide interrupt here
	ljmp	return_addr



	;check if the ide driver is idle
	;dpl=1 if idle, dpl=0 if more activity may be happen soon.
is_ide_idle_c:
	mov	a, ide_state
	cjne	a, #ide_s_sleeping, ideii1
	mov	dpl, #1
	ljmp	return_addr

ideii1: mov	a, clust_rreq_head
	cpl	a
	jnz	ide_not_idle
	mov	a, fat_rreq_head
	cpl	a
	jnz	ide_not_idle
	mov	a, ide_state
	cjne	a, #ide_s_idle, ide_not_idle
	mov	dpl, #1
	ljmp	return_addr
ide_not_idle:
	mov	dpl, #0
	ljmp	return_addr



ide_sleep_c:
	;- disable ide interrupt here
	mov	a, clust_rreq_head
	cjne	a, #255, idesleep2
	;if we get here, there were no fat sector read requests
	;so the queue is marked empty and we need to initialize
	;it.  When this someday is interrupt driven, this code
	;will need to set a flag that causes the first interrupt
	clr	a
	mov	clust_rreq_head, a
	mov	clust_rreq_tail, a
idesleep2:
	inc	a
	cjne	a, #clust_rreq_size, idesleep3
	clr	a
idesleep3:
	;need to check if this one fills the queue
	mov	clust_rreq_head, a

	;now turn the index into a 16 pointer into the read_request array
	mov	b, #8	;sizeof(struct read_request) == 8
	mul	ab
	add	a, #clust_read_req & 255
	mov	dpl, a
	mov	a, b
	addc	a, #clust_read_req >> 8
	mov	dph, a

	;now dptr points to the new read_request struct, so let's fill it
	clr	a
	movx	@dptr, a	;LBA = 0
	inc	dptr
	mov	a, f32_sector+1
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+2
	movx	@dptr, a
	inc	dptr
	mov	a, f32_sector+3
	movx	@dptr, a
	inc	dptr
	mov	a, r2
	movx	@dptr, a	;block = 0
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	mov	a, #1
	movx	@dptr, a	;command = 1, ide sleep
	clr	a
	inc	dptr
	movx	@dptr, a	;fill unused bytes with zero
	;-reenable ide interrupt
	ljmp	return_addr



















































































;*************************************************************
;**                                                         **
;**              OLD IDE Driver (to be removed)             **
;**                                                         **
;*************************************************************






;	;read a sector from the drive, and put it @dptr
ide_read_sector:
        push    dph
        push    dpl
        lcall   ide_busy                ;make sure drive is ready
        lcall   wr_lba                  ;tell it which sector we want
        mov     dptr, #ide_command
        mov     a, #ide_cmd_read
        movx    @dptr, a                ;ask the drive to read it
        lcall   ide_drq                 ;wait until it's got the data
        jb      acc.0, get_err
        pop     dpl
        pop     dph
        lcall   read_data               ;grab the data
        clr     a
        ret


        ;when an error occurs, we get acc.0 set from a call to ide_drq
        ;or ide_busy (which read the drive's status register).  If
        ;that error bit is set, we should jump here to read the drive's
        ;explaination of the error, to be returned to the user.  If for
        ;some reason the error code is zero (shouldn't happen), we'll
        ;return 255, so that the main program can always depend on a
        ;return of zero to indicate success.
get_err:mov     dptr, #ide_err
        movx    a, @dptr
        jz      gerr2
        ret
gerr2:  mov     a, #255
        ret




wr_lba:
	mov	a, f32_sector+3
	anl	a, #0x0F
	orl	a, master_slave		;select correct drive
	mov	dptr, #ide_head		;write LBA to drive
	movx	@dptr, a
	mov	a, f32_sector+2
	mov	dptr, #ide_cyl_msb
	movx	@dptr, a
	mov	a, f32_sector+1
	mov	dptr, #ide_cyl_lsb
	movx	@dptr, a
	mov	a, f32_sector+0
	mov	dptr, #ide_sector
	movx	@dptr, a
	mov	a, #1
	mov	dptr, #ide_sec_cnt
	movx	@dptr, a
	ret




ide_set_low_power:
	;acall	ide_drq
	;mov	dptr, #ide_features
	;mov	a, #0x9A		;set current subcommand
	;movx	@dptr, a
	;mov	dptr, #ide_sec_cnt
	;mov	a, #1			;lowest available current
	;movx	@dptr, a
	;mov	dptr, #ide_command
	;mov	a, #ide_cmd_set_features
	;movx	@dptr, a
	ret

	;put the drive into sleep mode

msg_ide_sleep:
	.db	"ide_sleep",13,10,0



ide_init_c:
	mov	master_slave, #0xF0
ideiwait:
	mov	dptr, #ide_head
	xrl	master_slave, #0x10	;try master/slave alternately
	mov	a, master_slave
	movx	@dptr, a		;select the master or slave device
	mov	dptr, #ide_status
	movx	a, @dptr
	  push	acc
	  mov	a, #'.'			;should slow down the printing of dots
	  lcall	cout

	  ; could get stuck in an infinite loop here.....
	  ;lcall	wait
	  pop	acc
	jb      acc.7, ideiwait         ;wait for BSY bit to be clear
	jnb	acc.6, ideiwait         ;wait for RDY bit to be set
	lcall	newline
	mov	dptr, #msg_found
	lcall	pstr
	lcall	pstr
	mov	dptr, #msg_master
	mov	a, master_slave
	jnb	acc.4, idei2
	mov	dptr, #msg_slave
idei2:	lcall	pstr
	;do a "set multiple mode" command to define multiple
	;mode to transfer 4k blocks of data
	 mov	dptr, #msg_set_mult
	 lcall	pstr
	mov	a, master_slave		;select correct drive
	mov	dptr, #ide_head
	movx	@dptr, a
	mov	a, #8			;8 sectors (4k) per block
	mov	dptr, #ide_sec_cnt
	movx	@dptr, a
	mov	dptr, #ide_command
	mov	a, #ide_cmd_set_mult
	movx	@dptr, a		;do the set multiple mode
	lcall	ide_busy		;wait for the drive to accept it
	 lcall	phex
	 lcall	newline
	mov	ide_state, #ide_s_wait_nbusy
	ljmp	return_addr





        ;do a hard reset on the drive, by pulsing its reset pin.
        ;this should usually be followed with a call to "ide_init".
	;it's not included inside ide_init because it may be
	;desirable to call this early in the boot up sequence, and
	;then initialize other hardware or memory while the drives
	;start spinning up their motors.
ide_hard_reset:
	 mov	dptr, #msg_ide_reset
	 lcall	pstr_d
        mov     dptr, #ide_rst_bit
        clr     a
        movx    @dptr, a                ;assert the reset pin
        acall   delay_10ms              ;IDE requires reset > 25 ms
        acall   delay_10ms
        acall   delay_10ms
        acall   delay_10ms
        mov     a, #1
        movx    @dptr, a                ;de-assert the reset pin
        acall   delay_10ms              ;brief delay while drive inits
        acall   delay_10ms              ;brief delay while drive inits

	;ask the master device for its status
	mov	dptr, #ide_head
	mov	a, #0xE0
	movx	@dptr, a
	mov	dptr, #ide_status
	movx	a, @dptr
	;ask the slave device for its status
	mov	dptr, #ide_head
	mov	a, #0xF0
	movx	@dptr, a
	mov	dptr, #ide_status
	movx	a, @dptr
        ret

ide_hard_reset_c:
	acall	ide_hard_reset
	ljmp	return_addr








        ;delay for 10 ms
        ;10ms at 7.3728 MHz is 6144 machine cycles
        ;6144 = 256 * 24
delay_10ms:
        mov     r0, #0
d1ms:   mov     r1, #10         ;1 cycle
        djnz    r1, *           ;20 cycles
        nop                     ;1 cycle
        djnz    r0, d1ms        ;2 cycles
five_cy:nop
        ret


msg_ide_reset:
	.db	"ide: hard reset",13,10,0




ide_busy:
	mov	dptr, #ide_status
	movx	a, @dptr
	  push	acc
	  mov	a, #'*'
	  lcall	cout

	  ; count get stuck in an infinite loop!
	  ;lcall	wait
	  pop	acc
	jb	acc.7, ide_busy		;wait for BSY to be clear
	ret


ide_drq:
	mov	dptr, #ide_status
	movx	a, @dptr
	  push	acc
	  mov	a, #'_'
	  lcall	cout

	  ; count get stuck in an infinite loop!
	  ; lcall	wait
	  pop	acc
	jb	acc.7, ide_drq		;wait for BSY to be clear
	jnb     acc.3, ide_drq          ;wait for DRQ to be set
	ret








print_sector_buffer:
	mov	r2, #32
	mov	dptr, #sector_buffer


;print a hexdump of memory.  R2 specifies how many 16 byte
;lines to print.  DPTR specifies where to dump from.

hexdump:
	lcall	newline
hd1:	lcall	phex16			;tell 'em the memory location
	mov	a, #':'
	lcall	cout
	mov	a, #' '
	lcall	cout
	mov	r3, #16			;r3 counts # of bytes to print
hd2:	movx	a, @dptr
	inc	dptr
	lcall	phex
	lcall	space
	djnz	r3, hd2
	clr	c
	mov	a, dpl
	subb	a, #16
	mov	dpl, a
	mov	a, dph
	subb	a, #0
	mov	dph, a

	mov	r3, #16
hd3:	movx	a, @dptr
	inc	dptr
	anl	a, #0x7F
	cjne	a, #0x7F, hd3b
	clr	a
hd3b:	add	a, #224
	jc	hd4
	clr	a
hd4:	add	a, #32
	lcall	cout
	djnz	r3, hd3
	lcall	newline
	djnz	r2, hd1
	ret




        ;Read a block of 512 bytes (one sector) from the drive
        ;and store it in memory @ DPTR
read_data:
        ajmp   read_data_pio



        ;read 512 bytes of data using fast DMA
;read_data_dma:
;        mov     a, dpl
;        mov     b, dph
;        mov     dptr, #dma_ide_dest
;        movx    @dptr, a                ;write LSB of dest addr
;        inc     dptr
;        mov     a, b
;        movx    @dptr, a                ;write LSB of dest addr
;        mov     dptr, #dma_ide_count
;        clr     a
;        movx    @dptr, a
;        inc     dptr
;        mov     a, #1
;        movx    @dptr, a                ;request 256 16 bit reads
;        mov     dptr, #irq_dma_ide_ack
;        movx    @dptr, a                ;clear any previous interrupt
;        mov     dptr, #dma_ide_go
;        movx    @dptr, a                ;begin the transfer
;        ;now we just wait for it to finish
;        mov     dptr, #irq_ident
;dma_wait:
;        movx    a, @dptr                ;2  1 psen = 2 bytes transfered
;        jnb     acc.0, dma_wait         ;2  3 psen = 6 bytes transfered
;        ret
;
        ;speed = 8 bytes per 4 cycles -> 1229 kbyte/sec
        ;  remember, this is raw transfer speed, and doesn't
        ;  include the time to set up the transfer.

        ;Ideally, we'd be executing real code instead of this
        ;busy loop.  With some luck, real code would have more
        ;psen's per second than movx (only mul and div have less).
        ;If we call into PM2, the psens (and dma) will stall!

        ;Maybe someday the DMA controller will be able to detect
        ;unused bus time and use it... that'd double the speed for
        ;this test case.  Will need to investigate actual speed once
        ;the drivers are written and somewhat mature.  A really
        ;ambitious approach would be to watch the psen fetch and
        ;do a little table lookup of the opcode, and let the state
        ;machine run full speed for the entire instruction and
        ;into the fetch of the next opcode, so we could use all
        ;the available time.  Maybe someday when I have lots of
        ;free time (highly unlikely).



        ;read 512 bytes of data using slow I/O
read_data_pio:
        mov     r5, #0
rdblk2: push    dph                     ;2
        push    dpl                     ;2
        mov     dptr, #ide_data         ;2
        movx    a, @dptr                ;2  read lsb
        mov     r2, a                   ;1
        mov     dptr, #ide_data_buf_msb ;2
        movx    a, @dptr                ;2  read msb
        mov     r3, a                   ;1
        pop     dpl                     ;2
        pop     dph                     ;2
        mov     a, r2                   ;1
        movx    @dptr, a                ;2  write lsb
        inc     dptr                    ;2
        mov     a, r3                   ;1
        movx    @dptr, a                ;2  write msb
        inc     dptr                    ;2
        djnz    r5, rdblk2              ;2
        ret

        ;speed = 2 bytes per 30 cycles -> 41 kbyte/sec





















;*************************************************************
;**                                                         **
;**                     Memory Manager                      **
;**                                                         **
;*************************************************************



        ;initialize the memory manager, figure out what
        ;SIMM is installed, and initialize the memory
        ;block descriptor table.  The first 32 blocks
        ;are always used for the table itself, regardless
        ;of what SIMM is installed.  All the rest are
        ;either free blocks, or not implemented by the SIMM.
        ;The xilinx chip needs to be configured with the
        ;DRAM controller before calling here.
        ;return: dpl=1 if an error (no SIMM), dpl=0 for success

init_memory_mgr_c:
	lcall	detect_simm_size	;R4=0 to 3, Acc=0, 4, 8, 16, or 32
	jnz	initmm2
        mov     dptr, #msg_bad_simm
        lcall   pstr
	mov	dpl, #1
        ljmp	return_addr			;error, no simm is installed
initmm2:
	mov	r7, a		 ;keep a copy of the simm size in r7.  This
        mov     dptr, #msg_found ;will get used quite a bit later, so be
        lcall   pstr		 ;sure not to clobber r7 until after it's
	mov	a, r7		 ;used to initialize "mm_num_free"
        lcall   pint8
        mov     dptr, #msg_size
        lcall   pstr


	;fill the first 32 blocks (block_info array) with zeros
	;this is time consuming and someday it should be removed,
	;but for now it makes it a lot easier to read through the
	;"block_info" memory with the memory editor
	mov	r2, #0
	mov	r3, #0
initmm3:
	mov	a, #2
	lcall	map_block
	mov	dptr, #0x2000
	lcall	clr_block
	inc	r2
	cjne	r2, #32, initmm3



        mov     r5, #0          ;r5 will count which 1MB section (0-31)
initmm4:
        mov     a, r5
	mov	r2, a
	clr	a
	mov	r3, a
	mov	a, #2
        lcall   map_block

        mov     a, r5
        lcall   is_implemented
        jz      initmm_not_implemented

        ;mov	a, #'p'
        ;lcall	cout
        ;mov	a, #'='
        ;lcall	cout
        ;mov	a, r5
        ;lcall	phex
	;lcall	space

initmm5:
        ;whatever page is mapped at 1000 to 1FFF is the previous
        ;page where we didn't write the last entry, but now we
        ;know what it will be, so write it now.
        mov     a, r5
        jz      initmm6         ;don't do this the first time
        mov     dptr, #0x1FF0
        clr     a
        movx    @dptr, a
        inc     dptr
        mov     a, r5
        movx    @dptr, a
initmm6:
        ;first, let's just rapidly fill up the linked list
        ;knowing that the last one may be wrong, since we
        ;don't know where it will point yet.  We do know
        ;that each block will always point to the next in
        ;a 1 meg page, at least initially when all the
        ;memory is free and we're just initializing.
        mov     r1, #0          ;r1 counts page within that 1MB (0-255)
        mov     dptr, #0x2000
initmm7:
        inc     r1
        mov     a, r1
        movx    @dptr, a
        inc     dptr
        mov     a, r5
        movx    @dptr, a
        mov     a, dpl
        add     a, #15
        mov     dpl, a
        mov     a, dph
        addc    a, #0
        mov     dph, a
        anl     a, #0xF0
	add	a, #0xE0
        jz      initmm7

        ;now map this page at 1000 to 1FFF, so later we can write
        ;the last entry to point to the next 1MB part
	mov	a, r5
	mov	r2, a
	mov	r3, #0
	mov	a, #1
        lcall   map_block
	sjmp	initmm9


initmm_not_implemented:
	;for a 1 meg section of memory not implemented by the SIMM,
	;we need to store 0x0F in the flags byte, to mark that those
	;4k blocks aren't implemented by the SIMM
        mov     dptr, #0x2002
initmmni:
	mov	a, #0x0F
        movx    @dptr, a
        mov     a, dpl
        add     a, #16
        mov     dpl, a
        mov     a, dph
        addc    a, #0
        mov     dph, a
        anl     a, #0xF0
	add	a, #0xE0
        jz      initmmni


initmm9:
        inc     r5
        cjne    r5, #32, initmm4

        ;the very last entry needs a NULL to terminate the list
        mov     dptr, #0x1FF0
        clr     a
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a

        ;lcall   newline

        ;now we need to make the first 32 pages not free,
        ;since they're used for tracking what memory is
        ;used and what's free.  Actually, we'll grab a 33rd
	;page as well, and use it in just a moment.
	clr	a
	mov	r2, a
	mov	r3, a
	mov	a, #2
        lcall   map_block
        mov     dptr, #0x2200
        clr     a
        movx    @dptr, a
        inc     dptr
        movx    @dptr, a


	;that 33rd block is needs to get mapped at 0x8000, and we
	;need to initialize critical structures within it, most
	;importantly, "mm_free_list", so that malloc/free will work.
	;because this block stores such important stuff, let's first
	;clear all the bytes to zero.

	mov	r2, #32
	mov	r3, #0
	mov	a, #8
	lcall	map_block
	mov	dptr, #0x8000
	lcall	clr_block
	mov	dptr, #mm_free_list
	mov	a, #33		;the 34th block is the first free
	movx	@dptr, a
	inc	dptr
	clr	a
	movx	@dptr, a

	;initialize mm_num_free = (simm size * 256) - 33
	;hopefully nothing overwrote r7 until this point.
	mov	dptr, #mm_num_free
	mov	a, #223
	movx	@dptr, a
	inc	dptr
	mov	a, r7
	dec	a
	movx	@dptr, a


	;now let's malloc several 4k block of DRAM and map them
	;to the memory pages that remain static.  Static pages:
	;  0000 - 5FFF   Application Global/Static Variables
	;  8000 - 8FFF   FAT32 and other library global variables
	;  C000 - DFFF   FAT32 array of blocks holding cached FAT sectors

	mov	a, #10
	lcall	malloc_blocks
	mov	a, #0
	lcall	map_block
	lcall	next_block
	mov	a, #1
	lcall	map_block
	lcall	next_block
	mov	a, #2
	lcall	map_block
	lcall	next_block
	mov	a, #3
	lcall	map_block
	lcall	next_block
	mov	a, #4
	lcall	map_block
	lcall	next_block
	mov	a, #5
	lcall	map_block
	lcall	next_block
	mov	a, #0xC
	lcall	map_block
	lcall	next_block
	mov	a, #0xD
	lcall	map_block

	;map blocks at 0x6000 and 0x7000, so that if something write
	;there by mistake, it won't overwrite critical info in block 0
	lcall	next_block
	mov	a, #6
	lcall	map_block
	lcall	next_block
	mov	a, #7
	lcall	map_block
	
	mov	dptr, #0xD000
	lcall	clr_block

	mov	a, #255
	mov	fat_rreq_head, a
	mov	fat_rreq_tail, a
	mov	clust_rreq_head, a
	mov	clust_rreq_tail, a
	mov	play_req_head, a
	mov	play_req_tail, a

	mov	dptr, #last_fat_sector
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a
	mov	dptr, #last_fat_fd
	movx	@dptr, a


	mov	ide_state, #ide_s_wait_nbusy
	;mov	play_state, #play_s_idle

        ;I hope that worked!
	mov	dpl, #0
	ljmp	return_addr




	;write 4096 zeros to a block, @dptr
clr_block:
	clr	a
set_block:		;set all the bytes in the block to Acc
	mov	r0, #16
clrblk2:mov	r1, #0
clrblk3:movx	@dptr, a
	inc	dptr
	djnz	r1, clrblk3
	djnz	r0, clrblk2
	ret





        ;This function returns zero if a particular simm
        ;doesn't implement a particlar 1 MB memory range,
        ;or non-zero if this simm implements that area.
        ;inputs:
        ;       acc = 1 MB memory area (0-32)
        ;       r4 = simm type (0=4M, 1=8M, 2=16M, 3=32M)
is_implemented:
	 ;push	acc
	 ;mov	a, #'i'
	 ;lcall	cout
         ;mov	a, #'='
         ;lcall	cout
	 ;pop	acc
	 ;push	acc
	 ;lcall	phex
	 ;mov	a, r4
	 ;lcall	phex
	 ;lcall	space
	 ;pop	acc
        rr      a
        rr      a
        anl     a, #7
        mov     r0, a
        mov     a, r4
        anl     a, #3
        add     a, #2
        movc    a, @a+pc
        sjmp    is_imp2
        .db     00000001b       ;4 meg = blocks 0 to 1023
        .db     00000011b       ;8 meg = blocks 0 to 2047
        .db     01010101b       ;16 meg = blocks 0 to 1023
                                ;       blocks 2048 to 3071
                                ;       blocks 4096 to 5119
                                ;       blocks 6144 to 7167
        .db     11111111b       ;32 meg = blocks 0 to 8191
is_imp2:
        cjne    r0, #0, is_imp3
        anl     a, #1
        ret
is_imp3:rr      a
        dec     r0
        sjmp    is_imp2




	;detect the SIMM that's installed, and return codes in r4 and Acc
	;r4=0, Acc=0:  no SIMM installed
	;r4=0, Acc=4:  4 megabyte SIMM
	;r4=1, Acc=8:  8 megabyte SIMM
	;r4=2, Acc=16: 16 megabyte SIMM
	;r4=3, Acc=32: 32 megabyte SIMM
detect_simm_size:
	clr	a
	mov	r2, a
	mov	r3, a
        lcall   map_block
        lcall   write_test_bytes
        lcall   check_test_bytes
        jnc     detect_simm2
	clr	a
	mov	r4, a
	ret
detect_simm2:
        ;in a smaller 4 or 8 meg simm, writing to block 2048 will
        ;overwrite whatever's in block 0, but 16 and 32 meg simms
        ;have a block 2048, so writing to block 0, then 2048, and
        ;checking to see if block 0 still is correct will tell if
        ;the simm is (4 or 8) vs (16 or 32).
	mov	r2, #2048 & 255
	mov	r3, #2048 >> 8
	clr	a
        lcall   map_block
        lcall   write_test_bytes
	clr	a
	mov	r2, a
	mov	r3, a
        lcall   map_block
        lcall   check_test_bytes
        mov     acc.1, c
        push    acc
        ;double sided simms (8 and 32 megs) implement block 1024
        ;whereas single sided simms can't implement block 1024, so
        ;writing and checking 1024 will tell if it's a single
        ;(4 or 16) or double (8 or 32).
	mov	r2, #1024 & 255
	mov	r3, #1024 >> 8
	clr	a
        lcall   map_block
        lcall   write_test_bytes
        lcall   check_test_bytes
        pop     acc
        mov     acc.0, c
        cpl     a
        anl     a, #3
        mov     r4, a           ;r4 has simm size, 0 to 3
	inc	a
        movc    a, @a+pc        ;now Acc has simm size, in Mbytes
	ret			;1 byte
        .db     4, 8, 16, 32


detect_simm_size_c:
	acall	detect_simm_size
	mov	dpl, a
	ljmp	return_addr


        ;write a simple test to a DRAM page, just write
        ;the page number a couple times, with different
        ;XOR'd with different bit patterns.  This way,
        ;we are unlikely to read garbage as good, and
        ;it will always detect a page that is corrupted
        ;by writing to a different page (small simms
        ;which appear repeated 4 times)
write_test_bytes:
        mov     dptr, #dram_page_cfg
        movx    a, @dptr        ;read page # lsb
        mov     b, a
        inc     dptr
        movx    a, @dptr        ;read page # msb
        mov     dptr, #0
        xrl     a, #0x5A
        movx    @dptr, a
        inc     dptr
        cpl     a
        movx    @dptr, a
        inc     dptr
        mov     a, b
        xrl     a, #0x9C
        movx    @dptr, a
        inc     dptr
        cpl     a
        movx    @dptr, a
        ret

        ;read a page previously written with "write_test_bytes"
        ;and return C=0 if good, C=1 if wrong bytes read back
check_test_bytes:
        mov     dptr, #dram_page_cfg
        movx    a, @dptr        ;read page # lsb
        mov     r0, a
        inc     dptr
        movx    a, @dptr        ;read page # msb
        mov     b, a
        mov     dptr, #0
        movx    a, @dptr
        xrl     a, #0x5A
        cjne    a, b, cktb_err
        inc     dptr
        movx    a, @dptr
        xrl     a, #0xA5
        cjne    a, b, cktb_err
        inc     dptr
        mov     b, r0
        movx    a, @dptr
        xrl     a, #0x9C
        cjne    a, b, cktb_err
        inc     dptr
        movx    a, @dptr
        xrl     a, #0x63
        cjne    a, b, cktb_err
        clr     c
        ret
cktb_err:
        setb    c
        ret







	;map block number r2/r3 into a page # specified by Acc
map_block:
        anl     a, #15
        rl      a
        add     a, #dram_page_cfg & 255
        mov     dpl, a
        clr     a
        addc    a, #dram_page_cfg >> 8
        mov     dph, a
        mov     a, r2
        movx    @dptr, a
        inc     dptr
        mov     a, r3
        movx    @dptr, a
        ret





malloc_blocks_c:
	mov	a, r2
	acall	malloc_blocks
	jc	malloc_c_err
	mov	dpl, r2
	mov	dpl, r3
	ret
malloc_c_err:
	mov	dptr, #0
	ret



	;Allocate 1 or more 4k blocks of memory. The number of the
	;first block is returned in R2/R3 with carry clear.  Carry
	;is set if an error occurs.
malloc_blocks:
	mov	r0, a			;r0 counts # to allocate
	mov	r1, a
	 mov	dptr, #msg_malloc1
	 lcall	pstr_d
	 mov	a, r0
	 lcall	pint8u_d
	 mov	dptr, #msg_malloc2
	 lcall	pstr_d
	mov	dptr, #mm_free_list
	movx	a, @dptr
	mov	r2, a
	mov	r6, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	mov	r7, a
malloc_loop:
	mov	a, r2			;check if we're out of memory
	mov	r4, a
	mov	a, r3
	mov	r5, a
	jnz	malloc_next
	mov	a, r2
	jnz	malloc_next
	;TODO:
	;in the case where we run out of memory inside malloc, we
	;really ought to go back and change the block_info.flags
	;that we set to 01 back to 00 (not allocated)
	setb	c			;no more memory
	ret
malloc_next:
	 ;mov	a, r3
	 ;lcall	phex_d
	 ;mov	a, r2
	 ;lcall	phex_d
	 ;mov	a, #' '
	 ;lcall	cout_d

	mov	a, r3
	jnz	malloc_next_ok
	mov	a, r2
	add	a, #223
	jc	malloc_next_ok
	mov	dptr, #msg_malloc_reserved
	lcall	pstr
	mov	a, r3
	lcall	phex
	mov	a, r2
	lcall	phex
	lcall	space
	mov	a, r7
	lcall	phex
	mov	a, r6
	lcall	phex
	lcall	newline
	ljmp	serious_error
	
malloc_next_ok:
	lcall	block_desc_addr
	movx	a, @dptr		;read block_info.next_block (lsb)
	mov	r2, a
	inc	dptr
	movx	a, @dptr		;read block_info.next_block (msb)
	mov	r3, a
	inc	dptr
        movx    a, @dptr                ;check block is flagged as free
        jz      malloc_ok
	acall	malloc_notfree_err

malloc_ok:
	mov	a, #1
	movx	@dptr, a		;set block_info.flags = 01
	djnz	r0, malloc_loop

	 lcall	newline_d
	;at this point, we've got three block numbers
	;  r1    = number of blocks we're taking
	;  r2/r3 = first block past the allocated list
	;  r4/r5 = last block in the allocated list
	;  r6/r7 = first block in the allocated list

	;adjust the free block counter
	mov	dptr, #mm_num_free
	movx	a, @dptr
	clr	c
	subb	a, r1
	movx	@dptr, a
	inc	dptr
	movx	a, @dptr
	subb	a, #0
	movx	@dptr, a

	;first, update the free list, since these aren't free now
	mov	dptr, #mm_free_list
	mov	a, r2
	movx	@dptr, a
	inc	dptr
	mov	a, r3
	movx	@dptr, a

	;now null terminate the group, so it doesn't point to the free list
	mov	r2, reg4
	mov	r3, reg5
	lcall	block_desc_addr
	clr	a
	movx	@dptr, a
	inc	dptr
	movx	@dptr, a

	;we're done...
	mov	r2, reg6
	mov	r3, reg7

	clr	c
	ret


msg_malloc_reserved:
	.db	"WARNING: malloc_blocks using reserved memory ",0

malloc_notfree_err:
         push    dpl
         push    dph
         mov	dptr, #msg_malloc_notfree
	 lcall	pstr
         mov	a, r5
	 lcall	phex
	 mov	a, r4
	 lcall	phex
         lcall   space
         mov     a,#'-'
         lcall   cout
         lcall   space
         mov	a, r3
	 lcall	phex
	 mov	a, r2
	 lcall	phex
         lcall   space
         mov     a,#'-'
         lcall   cout
         lcall   space
         pop     dph
         pop     dpl
         movx    a, @dptr
         lcall   phex
         lcall   newline
	ret


print_memory_available:
	mov	dptr, #msg_mem_avail
	lcall	pstr
	mov	dptr, #mm_num_free
	movx	a, @dptr
	push	acc
	inc	dptr
	movx	a, @dptr
	lcall	phex
	pop	acc
	lcall	phex
	lcall	space
	mov	a, #'='
	lcall	cout
	lcall	space
	lcall	count_free_blocks
	mov	a, r1
	lcall	phex
	mov	a, r0
	lcall	phex
	lcall	newline
	ljmp	return_addr

msg_mem_avail:
	.db	"Memory Available (4k blocks): ",0



	;count the number of free blocks and return it in r1/r0

count_free_blocks:
	mov	dptr, #mm_free_list
	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	mov	r0, #0
	mov	r1, #0
count_free_loop:
	mov	a, r2
	jnz	count_free_next
	mov	a, r3
	jnz	count_free_next
	ret
count_free_next:
	lcall	next_block
	mov	a, r0
	add	a, #1
	mov	r0, a
	mov	a, r1
	addc	a, #0
	mov	r1, a
	sjmp	count_free_loop


free_blocks_c:
	acall	free_blocks
	ljmp	return_addr


	;Return allocated blocks back to the free memory pool.
	;R2/R3 specifies the first block of the group
free_blocks:
	mov	dptr, #mm_free_list
	movx	a, @dptr
	mov	r6, a
	inc	dptr
	movx	a, @dptr
	mov	r7, a
	 mov	dptr, #msg_free
	 lcall	pstr_d
	mov	a, r2
	mov	r4, a
	mov	a, r3
	mov	r5, a
	mov	r1, #0		;r1 to count # freed
free_loop:
        cjne    r3, #0, free_valid1     ;check for reserved (ie < 32)
        mov     a, r2
	add	a, #223
	jc	free_valid1
	ljmp	free_reserved		;blocks 0000 to 0030 are reserved
free_valid1:
        mov     a, r3		;check less than 32M (TODO: check simm size)
        anl     a, #0xE0
        jz      free_valid2
        push    dpl
        push    dph
        mov	dptr, #msg_free_invalid
	lcall	pstr
	mov	a, r3
	lcall	phex
	mov	a, r2
	lcall	phex
        lcall   newline
        pop     dph
        pop     dpl
free_valid2:
	inc	r1
	 mov	a, #' '
	 lcall	cout_d
	 mov	a, r3
	 lcall	phex_d
	 mov	a, r2
	 lcall	phex_d
	lcall	block_desc_addr
	movx	a, @dptr		;read block_info.next_block (lsb)
	mov	r2, a
	inc	dptr
	movx	a, @dptr		;read block_info.next_block (msb)
	mov	r3, a
	inc	dptr
        movx    a, @dptr                ;check block is flagged as used
        jnz     free_ok
        push    dpl
        push    dph
        mov	dptr, #msg_free_notused
	lcall	pstr
        pop     dph
        pop     dpl
	lcall	phex16
        lcall   newline
free_ok:
	clr	a
	movx	@dptr, a		;mark block as free, block_info.flags=0
        cjne    r2, #0, free_loop
        cjne    r3, #0, free_loop
free_doit:
	;at this point, we've got two block numbers
	;  r2/r3 = last block in the group to free
	;  r4/r5 = first block in the allocated list
	;  r6/r7 = first block of the original free list
	;  dptr points to within the 16 byte desc of r2/r3's block

	 push	dpl
	 push	dph
	 mov	dptr, #msg_free1
	 lcall	pstr_d
	 mov	r0, #4
	 lcall	phex16_at_r0
	 mov	dptr, #msg_free2
	 lcall	pstr_d
	 mov	r0, #2
	 lcall	phex16_at_r0
	 mov	dptr, #msg_free3
	 lcall	pstr_d
	 mov	r0, #6
	 lcall	phex16_at_r0
	 lcall	newline_d
	 pop	dph
	 pop	dpl

	;write the original free list into the last block's "next"
	anl	dpl, #0xF0	;<--- is this right ??
	mov	a, r6
	movx	@dptr, a
	inc	dptr
	mov	a, r7
	movx	@dptr, a

	;now set the free list to begin at this group of blocks
	;which will make them the new head of the list
	mov	dptr, #mm_free_list
	mov	a, r4
	movx	@dptr, a
	inc	dptr
	mov	a, r5
	movx	@dptr, a

	;adjust the free block counter
	mov	dptr, #mm_num_free
	movx	a, @dptr
	add	a, r1
	movx	@dptr, a
	inc	dptr
	movx	a, @dptr
	addc	a, #0
	movx	@dptr, a

	ret



	;an attempt was made to free one of the critical reserved
	;blocks.  Blocks 0000 to 002F are reserved for the block_info
	;structs that manage all the memory, and block 0030 is
	;reserved for the critical variables at 0x8000 to 0x8FFF
free_reserved:
        lcall   newline
        mov	dptr, #msg_free_reserved
	lcall	pstr
	mov	a, r3
	lcall	phex
	mov	a, r2
	lcall	phex
	lcall	space
	mov	a, r5
	lcall	phex
	mov	a, r4
	lcall	phex
	lcall	space
	mov	a, r7
	lcall	phex
	mov	a, r6
	lcall	phex
	lcall	space
	mov	a, r1
	lcall	phex
	lcall	space
	mov	r0, sp
	mov	a, @r0
	lcall	phex
	dec	r0
	mov	a, @r0
	lcall	phex
        lcall   newline
        lcall   newline
	ljmp	serious_error




	;Find the next block in a group.  R2/R3 specifies
	;the current block, and is returned with the next
	;block in the list.  R2/R3 return 0000 if there
	;are no more blocks.
next_block:
	lcall	block_desc_addr
	;the first two bytes are the "next block" pointer
	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	ret

next_block_c:
	acall	next_block
	mov	dph, a
	mov	dpl, r2
	ljmp	return_addr


	;given a block number in r2/r3, map the page with the
	;block_info describing the block, and return DPTR pointing
	;to the block's block_info.  r2/r3 is not changed.
block_desc_addr:
	;first, map the block that has the 16 bytes we need
	mov	a, r3
	anl	a, #0x1F
	mov	dptr, #dram_page_cfg + (0x0E * 2)
	movx	@dptr, a
	inc	dptr
	clr	a
	movx	@dptr, a

	;now compute a pointer to the 16 bytes
	mov	a, r2
	anl	a, #0x0F
	swap	a
	mov	dpl, a
	mov	a, r2
	anl	a, #0xF0
	swap	a
	orl	a, #0xE0
	mov	dph, a
	ret











;*************************************************************
;**                                                         **
;**               Serial Port I/O Functions                 **
;**                                                         **
;*************************************************************


cin:    jnb     ri, cin
        clr     ri
        mov     a, sbuf
        ret



cout_d:	jnb	debug, return
cout:   jnb     ti, cout
        clr     ti              ;clr ti before the mov to sbuf!
        mov     sbuf, a
        ret

newline_d:
	jnb	debug, return
newline:push    acc             ;print one newline
        mov     a, #13
        lcall   cout
        mov     a, #10
        lcall   cout
        pop     acc
        ret


; Highly code efficient resursive call phex contributed
; by Alexander B. Alexandrov <abalex@cbr.spb.ru>

phex_d:
	jnb	debug, return
phex:
        acall   phex_b
phex_b: swap    a               ;SWAP A will be twice => A unchanged
phex1:  push    acc
        anl     a, #15
        add     a, #0x90        ; acc is 0x9X, where X is hex digit
        da      a               ; if A to F, C=1 and lower four bits are 0..5
        addc    a, #0x40
        da      a
        lcall   cout
        pop     acc
        ret

return:	ret

phex16_d:
	jnb	debug, return
phex16:
        push    acc
        mov     a, dph
        lcall   phex
        mov     a, dpl
        lcall   phex
        pop     acc
        ret



pstr_d:
	jnb	debug, return
pstr:   push    acc
pstr1:  clr     a
        movc    a, @a+dptr
        inc     dptr
        jz      pstr2
        mov     c, acc.7
        anl     a, #0x7F
        lcall   cout
        jc      pstr2
        sjmp    pstr1
pstr2:  pop     acc
        ret


pint8u_d:
	jnb	debug, return
pint8u: ;prints the unsigned 8 bit value in Acc in base 10
        push    b
        push    acc
        sjmp    pint8b

pint8:  ;prints the signed 8 bit value in Acc in base 10
        push    b
        push    acc
        jnb     acc.7, pint8b
        mov     a, #'-'
        lcall   cout
        pop     acc
        push    acc
        cpl     a
        add     a, #1
pint8b: mov     b, #100
        div     ab
        setb    f0
        jz      pint8c
        clr     f0
        add     a, #'0'
        lcall   cout
pint8c: mov     a, b
        mov     b, #10
        div     ab
        jnb     f0, pint8d
        jz      pint8e
pint8d: add     a, #'0'
        lcall   cout
pint8e: mov     a, b
        add     a, #'0'
        lcall   cout
        pop     acc
        pop     b
        ret


space_d:jnb	debug, return
space:	mov	a, #' '
	ljmp	cout


	;print a 32 bit number stored @r0
phex32_at_r0:
	jnb	debug, ph32e
	inc	r0
	inc	r0
	inc	r0
	mov	a, @r0
	lcall	phex
	dec	r0
ph32b:	mov	a, @r0
	lcall	phex_d
	dec	r0
ph32c:	mov	a, @r0
	lcall	phex
	dec	r0
	mov	a, @r0
	lcall	phex
ph32e:	ret

	;print a 24 bit number stored @r0
phex24_at_r0:
	jnb	debug, ph32e
	inc	r0
	inc	r0
	sjmp	ph32b

	;print a 16 bit number stored @r0
phex16_at_r0:
	jnb	debug, ph32e
	inc	r0
	sjmp	ph32c


upper:
        cjne    a, #97, upper2
upper2: jc      upper4          ;end if acc < 97
        cjne    a, #123, upper3
upper3: jnc     upper4          ;end if acc >= 123
        add     a, #224         ;convert to uppercase
upper4: ret




	;completely quit the player and return to paulmon2
quit:
	setb	debug		;why does ide_sleep not work w/out debug
	;lcall	play_abort
	;lcall	ide_flush
	;lcall	ide_sleep
	;lcall	ide
	;lcall	ide
	;lcall	ide
	;lcall	ide
	;lcall	ide
	;lcall	ide
	;lcall	ide
	;TODO: better way to shut down quickly and reliably
	lcall	ide
	lcall	newline
	mov	a, #' '
	lcall	cout

	mov	dptr, #dma_mp3_src
	movx	a, @dptr
	push	acc
	inc	dptr
	movx	a, @dptr
	lcall	phex
	pop	acc
	lcall	phex

	mov	a, #' '
	lcall	cout

	mov	dptr, #dma_mp3_count
	movx	a, @dptr
	push	acc
	inc	dptr
	movx	a, @dptr
	lcall	phex
	pop	acc
	lcall	phex

	mov	a, #' '
	lcall	cout

	mov	dptr, #irq_ident
	movx	a, @dptr
	lcall	phex

	clr	ea

	lcall	newline
	ljmp	0x006D			;go back to PM2





	;this code handles the case where we've detected serious
	;corruption of memory, and we need to abort back to the
	;monitor, but first print a dump os what's in memory and
	;then make a simple attempt to shut off the hardware...
	;but don't call routines that depend on any memory, as
	;every byte of memory is probably corrupt at this point.
serious_error:
	clr	ea

	mov	r2, #0
	mov	r3, #0

serr2:	mov	a, #7
	lcall	map_block
	mov	dptr, #msg_blk_num
	lcall	pstr
	mov	a, r3
	lcall	phex
	mov	a, r2
	lcall	phex
	lcall	newline
	mov	dptr, #0x7000
	lcall	dump_4096_bytes
	inc	r2
	cjne	r2, #32, serr2

	mov	dptr, #msg_mem8000
	lcall	pstr
	mov	dptr, #0x8000
	lcall	dump_4096_bytes

	lcall	dump_imem

	mov	dptr, #location+32
	lcall	pstr
	mov	dptr, #msg_serious_error
	lcall	pstr

	;make an attempt to shut the drive off, but don't call
	;ide_sleep, since we can't depend on it to work with
	;the memory corrupted
	mov	dptr, #ide_command
	mov	a, #ide_cmd_sleep
	movx	@dptr, a
	mov	ide_state, #ide_s_wait_sleep

	lcall	newline
	ljmp	0x006D			;go back to PM2


msg_blk_num:
	.db	"Block #",0
msg_mem8000:
	.db	"Memory, 8000 to 8FFF",13,10,0

msg_serious_error:
	.db	13,10,"A serious error was detected, which probably was the",13,10
	.db	"result of memory somehow getting corrupted.  It could be",13,10
	.db	"that your SIMM is bad.  It's also possible that there's a",13,10
	.db	"problem with the address connections to the SIMM on your",13,10
	.db	"circuit board, or there could be a bug in this firmware.",13,10
	.db	13,10,"Please try using a differeny SIMM, and if problem",13,10
	.db	"doesn't go away, try typing DEBUG while the player is",13,10
	.db	"running, to enable additional debugging info, and then",13,10
	.db	"email a copy of that debug info and these memory dumps",13,10
	.db	"to paul@pjrc.com.  Hopefully I'll be able to figure out",13,10
	.db	"what caused this to happen.",13,10,13,10,0


dump_imem:
	mov	r0, #0
	mov	r1, #8
dimem1: mov	r2, #32
	mov	a, r0
	lcall	phex
	mov	a, #':'
	lcall	cout
	lcall	space
dimem2:	mov	a, @r0
	inc	r0
	lcall	phex
	djnz	r2, dimem2
	lcall	newline
	djnz	r1, dimem1
	lcall	newline
	ret



dump_4096_bytes:
	mov	r0, #0
d4k_1:	mov	a, dph
	anl	a, #0x0F
	lcall	phex
	mov	a, dpl
	lcall	phex
	lcall	space
	mov	r1, #16
d4k_2:	movx	a, @dptr
	inc	dptr
	lcall	phex
	djnz	r1, d4k_2
	lcall	newline
	djnz	r0, d4k_1
	lcall	newline
	ret



















;*************************************************************
;**                                                         **
;**                 String and Constants                    **
;**                                                         **
;*************************************************************




mesg_read_mbr:
	.db	"Reading Master Boot Record",13,10,0
mesg_bad_mbr:
	.db	"MBR Signature Not Found",13,10,0
msg_unknown_partion_type:
	.db	"Unknown Partition Type: ",0
mesg_partition_at:
	.db	"Found FAT32 Partition at LBA = ",0

mesg_read_vol_id:
	.db	"Reading FAT32 Volume ID",13,10,0
mesg_bad_vol_id:
	.db	"Bad FAT32 Volume ID",13,10,0
mesg_first_data_sec:
	.db	"First Data Sector at LBA = ",0

mesg_playing_file:
	.db	"Playing file: ",0

mesg_vol_id_ok:
	.db	"Volume ID looks ok",13,10,0
mesg_per_cluster1:
	.db	"Cluster size is ",0
mesg_per_cluster2:
	.db	" sectors (each sector is 512 bytes),",13,10
	.db	"Clusters will be cached in ",0
mesg_per_cluster3:
	.db	" block groups (each block is 4096 bytes)",13,10,0
mesg_per_cluster4:
	.db	"Cluster size doesn't fit nicely into 4k blocks",13,10
	.db	"CACHE MEMORY USAGE WILL NOT BE EFFICIENT!",13,10,0
mesg_root_dir_at_cluster:
	.db	"Root directory starts at cluster ",0
mesg_sectors_for_fats:
	.db	"Sectors used for FAT tables: ",0
mesg_reserved_sectors:
	.db	"Reserved Sectors: ",0
mesg_attrib:
	.db	"  Attrib=",0
mesg_cluster:
	.db	"  First_Cluster=",0
mesg_size:
	.db	"  Size=",0
mesg_welcome:
	.db	13,10,13,10,13,10
	.db	"MP3 Player, Version 0.6.2.2, May 4, 2001",13,10
	.db	"Copyright (C) 2000, PJRC.COM, LLC",13,10,13,10
	.db	"Type 'QUIT' to return to the monitor",13,10,13,10,0
mesg_begin:
	.db	13,10,"MP3 Player, Begin Playback",13,10,0
mesg_xilinx_cfg:
	.db	"Configuring Xilinx XCS10XL FPGA Chip: ",0
mesg_ok:
	.db	"Ok",13,10,0
mesg_nf:
	.db     "fail", 13, 10, 0
mesg_error:
	.db	"Error!",13,10,0
mesg_reset_ide:
	.db	"Reseting IDE Devices",13,10,0
mesg_sta013_cfg:
	.db	"Configuring STA013 MP3 Decoder Chip: ",0
mesg_ide_init:
	.db	"Waiting for Hard Drive to be ready",13,10,0

mesg_sector_buffer:
	.db	13,10,"Current Sector Buffer Contents:",13,10,0

mesg_play:
	.db	"Play",13,10,0
mesg_pause:
	.db	"Pause",13,10,0
mesg_next:
	.db	"Next File",13,10,0
mesg_prev:
	.db	"Previous File",13,10,0
mesg_rand:
	.db	"Random O",0
mesg_vol_up:
	.db	"Vol Up",13,10,0
mesg_vol_down:
	.db	"Vol Down",13,10,0


str_quit:
	.db	4, "QUIT"
str_debug:
	.db	5, "DEBUG"
str_play:
	.db	4, "PLAY"
str_next:
	.db	4, "NEXT"
str_previous:
	.db	8, "PREVIOUS"

mesg_init_mm:
	.db	"Initialize Memory Manager",13,10,0
msg_bad_simm:
        .db     "Bad or Missing SIMM",13,10,0
msg_found:
        .db     "Found ",0
	.db	"IDE ",0
msg_size:
        .db     " meg SIMM",13,10,0
msg_master:
	.db	"Master",13,10,0
msg_slave:
	.db	"Slave",13,10,0

msg_nodram:
	.db	"Running Old Firmware, that doesn't use the SIMM",13,10,13,10,0

msg_ide_enter:
	.db	"ide: ",0
msg_ide_state1:
	.db	"idle",13,10,0
msg_ide_state2:
	.db	"wait DRQ",13,10,0
msg_ide_state3:
	.db	"wait DMA",13,10,0
msg_ide_state4:
	.db	"wait nbusy",13,10,0
msg_ide_state5:
	.db	"waiting to sleep",13,10,0
msg_ide_state6:
	.db	"sleeping",13,10,0
msg_ide_state7:
	.db	"illegal (oh shit)",13,10,0

msg_malloc_notfree:
        .db     "WARNING: malloc_blocks, block is already allocated - ", 0
msg_malloc1:
	.db	"malloc ",0
msg_malloc2:
	.db	" blk:",0
msg_fopen:
	.db	"fopen, fd=",0
msg_cache1:
	.db	"file_cache, fd=",0
msg_cache2:
	.db	", bytes ",0
msg_to:
	.db	" to ",0
msg_req_clusters1:
	.db	"  Req'd c_offset ",0
msg_nxt_clust1:
	.db	"  find c, cur=",0
msg_nxt_clust2:
	.db	"    search FAT ",0
msg_nxt_clust_f1:
	.db	"    FAT sector found, i=",0
msg_nxt_clust_f2:
	.db	", blk=",0
msg_nxt_clust_nf:
	.db	"    FAT sector not found in list",13,10,0
msg_mov_array:
	.db	"    moving fat sector lookup array",0
msg_ide_read:
	.db	"ide: rd 4k, lba=",0
msg_ide_read2:
	.db	" to blk ",0
msg_read_clust1:
	.db	"  rd c ",0
msg_read_clust2:
	.db	" (offset ",0
msg_num_tracked:
	.db	"    currently cached blocks w/ FAT sectors: ",0
msg_insert_at:
	.db	"    new block to insert into array at index=",0
msg_prev_clust:
	.db	"  previous cluster in block: ",0
msg_bsearch1:
	.db	"      bin_search: begin=",0
msg_bsearch2:
	.db	", end=",0
msg_bsearch3:
	.db	", middle=",0
msg_fcw1:
	.db	"f_c_w,fd=",0
msg_fcw2:
	.db	"no ",0
msg_fcw3:
	.db	"prev",0
msg_fcseek1:
	.db	"  fcache, c=",0
msg_fcseek2:
	.db	", c_offset=",0
msg_fcseek3:
	.db	", targ=",0
msg_rdc_info1:
	.db	"  mk rreq, blk=",0
msg_rdc_info2:
	.db	", c=",0
msg_rdc_info3:
	.db	", o=",0
msg_rdc_info4:
	.db	", nxt=",0
msg_fseek1:
	.db	"file_seek to byte offset=",0
msg_fseek2:
	.db	"  cluster_offset=",0
msg_fseek3:
	.db	", pos_cluster=",0
msg_fseek4:
	.db	", pos_block=",0
msg_fseek5:
	.db	", pos_byte=",0
msg_fseek_eof:
	.db	"end of file reached, unable to fseek",13,10,0
msg_play_block:
	.db	"Play blk ",0
msg_done:
	.db	13,10,"Finished, press and key: ",0
msg_free_notused:
        .db     "WARNING: free_blocks, block is already free - dptr is ", 0
msg_free_reserved:
        .db     "WARNING: free_blocks, block is reserved - ", 0
msg_free_invalid:
        .db     "WARNING: free_blocks, block is invalid - ", 0
msg_free:
	.db	"free blocks:",0
msg_free1:
	.db	", first=",0
msg_free2:
	.db	", last=",0
msg_free3:
	.db	", list=",0
msg_rf_next_block:
	.db	"  next block:",0
msg_frd_blk_1:
	.db	"frb,posb=",0
msg_set_mult:
	.db	"Set multiple mode, status=",0
msg_debug_on:
	.db	"Debug Messages Turned ON",13,10,0
msg_debug_off:
	.db	"Debug Messages Turned OFF",13,10,0
msg_queue_full:
	.db	"  ide rreq queue too full to for cluster right now",13,10,0
msg_no_mem:
	.db	"  Out of memory!",13,10,0
msg_s_reading_dir:
	.db	"s_reading_dir",13,10,0
msg_s_caching1:
	.db	"s_caching1",13,10,0
msg_s_caching2:
	.db	"s_caching2",13,10,0
msg_s_caching3:
	.db	"s_caching3",13,10,0
msg_s_caching4:
	.db	"s_caching4",13,10,0
msg_s_caching_and_playing:
	.db	"s_c-n-p",13,10,0
msg_s_playing:
	.db	"s_playing",13,10,0
msg_s_finish_playing:
	.db	"s_finish_playing",13,10,0
msg_play_queue_empty:
	.db	"play_queue_empty, queue: ",0
msg_rf_eof:
	.db	"  file_read found last cluster",13,10,0
msg_test_sta013:
        .db     "Testing STA013... ",0
msg_play_len:
        .db     "t=",0
null_headers:
        .db     0x00, 0x90, 0xFF, 0xE3, 0x20, 0xC4, 0x00, 0x00
        .db     0x00, 0x03, 0x48, 0x01, 0x80, 0x00, 0x00
        .db     0x68, 0x90, 0xFF, 0xE3, 0x22, 0xC4, 0x5B, 0x00
        .db     0x00, 0x03, 0x48, 0x01, 0xC0, 0x00, 0x00
        .db     0xD1, 0x90, 0xFF, 0xE3, 0x20, 0xC4, 0xB7, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0x39, 0x91, 0xFF, 0xE3, 0x22, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0xA2, 0x91, 0xFF, 0xE3, 0x20, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0x0A, 0x92, 0xFF, 0xE3, 0x22, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0x73, 0x92, 0xFF, 0xE3, 0x20, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0xDB, 0x92, 0xFF, 0xE3, 0x22, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0x44, 0x93, 0xFF, 0xE3, 0x20, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00
        .db     0xAC, 0x93, 0xFF, 0xE3, 0x22, 0xC4, 0xFF, 0x00
        .db     0x00, 0x03, 0x48, 0x00, 0x00, 0x00, 0x00




;*************************************************************
;**                                                         **
;**             Run Old Firmware (last resort)              **
;**                                                         **
;*************************************************************


	;attempt to run the old version of the firmware (no DRAM)
	;if it is loaded into the flash rom.
check_old_firmware:
	mov	dptr, #0xF000
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xA5, cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xE5, cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xE0, cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0xA5, cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #0x23, cant_find_old_rev
	mov	dptr, #0xF030
	clr	a
	movc	a, @a+dptr
	cjne	a, #'0', cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #'.', cant_find_old_rev
	inc	dptr
	clr	a
	movc	a, @a+dptr
	cjne	a, #'1', cant_find_old_rev
	mov	dpl, #0
	ljmp	return_addr


	;mov	dptr, #msg_nodram
	;lcall	pstr
	;;ok, it looks good, so here we go with a leap of faith....
	;ljmp	0xF040

cant_find_old_rev:
	mov	dpl, #1
	ljmp	return_addr





;*************************************************************
;**                                                         **
;**             Run Tests Of Various Routines               **
;**                                                         **
;*************************************************************



phex32_at_dptr:
	movx	a, @dptr
	inc	dptr
	push	acc
	movx	a, @dptr
	inc	dptr
	push	acc
	movx	a, @dptr
	inc	dptr
	push	acc
	movx	a, @dptr
	inc	dptr
	lcall	phex
	pop	acc
	lcall	phex
	pop	acc
	lcall	phex
	pop	acc
	lcall	phex
	ret



pstr_movx:
	movx	a, @dptr
	jz	pstr_movx_end
	inc	dptr
	lcall	cout
	sjmp	pstr_movx
pstr_movx_end:
	ret



do_ide_loop_c:
	push	reg2

	; could get stuck in an infinit loop!
	;lcall	wait
	lcall	ide
	pop	reg2
	jc	do_ide_loop_c
	djnz	r2, do_ide_loop_c
	ljmp	return_addr









; read a non-volatile parameter.  This function only reads 1 byte
; paramters, or the first byte of a parameter's data
;  Input:
;    r2  = parameter to read
;    r0  = default value (if not found)
;  Output
;    r0 = parameter value

read_param_1byte:
	mov	a, r2
	acall	find_flash_param	;try to find the parameter
	jz	rdp1b_nope
	inc	dptr			;inc past parm id number
	clr	a
	movc	a, @a+dptr
	mov	dpl, a
	ljmp	return_addr
rdp1b_nope:
	mov	dpl, r0
	ljmp	return_addr





; Find a non-volatile parameter from the flash parameter storage block
;
; Input: Acc = parameter number to find
; Output: 
;	Acc = parameter number (if found)
;	Acc = 0x00 if not found
;	DPTR = will point to first byte parameter stored (the parm id number)
;
find_flash_param:
	mov	b, a
	acall	fp_length		;get param length
	inc	r4			;plus 1 byte for parm id number
	mov	dptr, #params_storage
ffp_loop:
	clr	a
	movc	a, @a+dptr		;get param & compare
	jz	ffp_next		;skip if blank
	cjne	a, b, ffp_check_blank
	ret
ffp_check_blank:
	cjne	a, #0xff, ffp_next	;first blank is end of list
ffp_not_found:
	clr	a
	ret
ffp_next:
	mov	a, dpl			;step over long parameter
	add	a, r4
	mov	dpl, a
	jnc	ffp_loop
	inc	dph
	mov	a, dph			;check if we are still in the 4k page
	anl	a, #0xF0
	cjne	a, #params_storage >> 8, ffp_not_found
	sjmp	ffp_loop




.equ	params_long_length, 11


;
;return the length of the parameter number in acc as register r4
;
fp_length:
	jnz	fp_len_255		;zero is erased byte (len = 0)
	mov	r4, #0
	ret
fp_len_255:
	cjne	a, #255, fp_len_short
	mov	r4, #0
	ret
fp_len_short:
	jb	acc.7, fp_len_long
	mov	r4, #1			;parms 1-127 are short
	ret
fp_len_long:
	mov	r4, #params_long_length	;parms 128-254 are long
	ret




;
; Write a parameter to the flash (if necessary)
;
; parameter number in A, value at @r0
; - C=1 if an error occured
;
	;recycle bsearch_tmp to store the caller's parameters
	.equ	parm_id_byte, bsearch_tmp
	.equ	parm_data_loc, bsearch_tmp+1

write_flash_param:
	mov	parm_id_byte, a		;save parm id byte
	mov	parm_data_loc, r0	;save pointer to data

wfp_reenter:
	 ;mov	a, #'A'
	 ;lcall	cout
	mov	a, parm_id_byte
	acall	find_flash_param	;is this parm already stored?
	jz	wfp_write_param		;not found so just write
	mov	r6, dpl			;keep pointer to beginning of data
	mov	r7, dph
	inc	dptr
	mov	a, parm_id_byte
	acall	fp_length		;get length of parm's data
	mov	a, r4
	mov	r3, a			;keep a copy of the length in r3
	clr	psw.5
	mov	r0, parm_data_loc
wfp_compare_loop:
	mov	a, @r0
	mov	b, a
	clr	a
	movc	a, @a+dptr
	cjne	a, b, wfp_compare_different
	sjmp	wfp_compare_next
wfp_compare_different:
	anl	a, b            	;can patch new data into the flash?
	cjne	a, b, wfp_wipe_param	;if we can't patch, better wipe it out
	setb	psw.5			;set a flag to remember to patch
wfp_compare_next:
	inc	dptr
	inc	r0
	djnz	r4, wfp_compare_loop
	jb	psw.5, wfp_patch_parm
	clr	c			;present value is same - do nothing
	ret


	;if we get here, the old parm needs to be overwritten with zeros
wfp_wipe_param:
	 ;mov	a, #'B'
	 ;lcall	cout
	mov	dpl, r6
	mov	dph, r7
	inc	r3
wfp_wipe_loop:
	 ;mov	a, #'b'
	 ;lcall	cout
	clr	a			;overwrite with 0
	lcall	prgm			;write value to FLASH
	jc	wfp_error
	inc	dptr
	djnz	r3, wfp_wipe_loop
	sjmp	wfp_reenter


	;if we get here, we can write the new parm value on top of the
	;original one (only ones need to change into zeros).
wfp_patch_parm:
	 ;mov	a, #'C'
	 ;lcall	cout
	mov	dpl, r6
	mov	dph, r7
	inc	dptr
	mov	r0, parm_data_loc
wfp_patch_loop:
	 ;mov	a, #'c'
	 ;lcall	cout
	mov	a, @r0
	lcall	prgm			;write value to FLASH
	jc	wfp_error
	inc	r0
	inc	dptr
	djnz	r3, wfp_patch_loop
	clr	c
	ret


	;we need to write the parameter to the end of the list.  DPTR
	;already points to the end of the list
wfp_write_param:
	 ;mov	a, #'D'
	 ;lcall	cout
	mov	a, parm_id_byte
	lcall	fp_length
	inc	r4			;also need space for parm id num
	mov	a, dpl
	add	a, r4
	mov	a, dph
	addc	a, #0
	anl	a, #0xF0
	add	a, #256 - ((params_storage >> 8) & 0xF0)
	jnz	defrag_flash_params
wfp_write_thispage:
	mov	a, parm_id_byte
	lcall	prgm			;write parameter number	to FLASH
	jc	wfp_error
	inc	dptr
	dec	r4
	mov	r0, parm_data_loc
wfp_write_loop:
	 ;mov	a, #'d'
	 ;lcall	cout
	mov	a, @r0			;get value byte
	lcall	prgm			;write value to FLASH
	jc	wfp_error
	inc	dptr
	inc	r0
	djnz	r4, wfp_write_loop
	clr	c
	ret
wfp_error:
	setb	c
	ret



;
;extract all params into DRAM, erase flash & write back into flash
;
defrag_flash_params:
	mov	a, #1			;allocate 1 block of dram
	lcall	malloc_blocks
	jc	dfp_abort_nofree
	mov	a, r2
	push	acc
	mov	a, r3
	push	acc
	mov	a, #0xC			;map into e000-efff
	lcall	map_block
	mov	dptr, #params_storage   ;point to start of params
	mov	r0, #0xC0		;point to start of dram block
	mov	r1, #0
	sjmp	dfp_np_cont
dfp_abort:
	pop	acc
	mov	r3, a
	pop	acc
	mov	r2, a
	lcall	free_blocks		;free DRAM
dfp_abort_nofree:
	setb	c
	ret
dfp_next_param:
	inc	dptr
	mov	a, dph			;check for end of block	(0x?fff)
	anl	a, 0x0f
	cjne	a, #0xf, dfp_np_cont
	mov	a, dpl
	cjne	a, #0xff, dfp_np_cont
	sjmp	dfp_copy_done
dfp_np_cont:
	clr	a
        movc    a, @a+dptr
	cjne	a, #0, dfp_not_wiped
	sjmp	dfp_next_param
dfp_not_wiped:
	cjne	a, #0xff, dfp_valid_param
	sjmp	dfp_copy_done		;a blank means all done
dfp_valid_param:
	lcall	fp_length		;parameter number & data
	inc	r4
	sjmp	dfp_write_current
dfp_next_byte:
	clr	a
        movc    a, @a+dptr
dfp_write_current:
	mov	r6, dph			;swap to DRAM data pointer
	mov	r7, dpl
	mov	dph, r0
	mov	dpl, r1
	movx	@dptr, a		;save this byte
	inc	dptr
	mov	r0, dph			;swap to FLASH data pointer
	mov	r1, dpl
	mov	dph, r6
	mov	dpl, r7
	djnz	r4, dfp_nb_cont		;next byte?
	sjmp	dfp_next_param
dfp_nb_cont:
	clr	a
	movc	a, @a+dptr		;get next byte
	inc	dptr
	sjmp	dfp_next_byte
dfp_copy_done:
	mov	dptr, #params_storage	;erase storage block
	mov	a, #0x24		;flash enable command
	lcall	erblock
	jc	dfp_abort
	mov	b, r0			;save r0 (prgm kills it)
	mov	dptr, #params_storage	;init flash pointer
	mov	r2, dph
	mov	r3, dpl
	mov	dptr, #0xc000		;point to DRAM
dfp_next_data:
	movx	a, @dptr		;get data from DRAM
	mov	r6, dph			;swap to FLASH data pointer
	mov	r7, dpl
	mov	dph, r2
	mov	dpl, r3
	lcall	prgm			;burn into FLASH
	jc	dfp_abort
	inc	dptr
	mov	r2, dph			;swap to DRAM data pointer
	mov	r3, dpl
	mov	dph, r6
	mov	dpl, r7
	inc	dptr
	mov	a, dph			;compare MSB
	clr	c
	subb	a, b
	jnz	dfp_next_data
	mov	a, dpl			;compare LSB
	clr	c
	subb	a, r1
	jnz	dfp_next_data
	pop	acc
	mov	r3, a
	pop	acc
	mov	r2, a
	lcall	free_blocks		;free DRAM
	ljmp	wfp_reenter



